@@ -5,7 +5,7 @@
  *
  * Portable Windows Library
  *
- * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
+ * Copyright (c) 1993-2002 Equivalence Pty. Ltd.
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.0 (the "License"); you may not use this file except in
@@ -21,12 +21,18 @@
  *
  * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
  *
- * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
- * All Rights Reserved.
- *
  * Contributor(s): ______________________________________.
  *
  * $Log: asner.h,v $
+ * Revision 1.33  2002/11/26 23:29:18  robertj
+ * Added missing const to DecodeSubType() function.
+ *
+ * Revision 1.32  2002/11/06 22:47:23  robertj
+ * Fixed header comment (copyright etc)
+ *
+ * Revision 1.31  2002/10/31 05:50:49  robertj
+ * Changed to use new UTF-8/UCS-2 conversion functions on PString.
+ *
  * Revision 1.30  2002/10/10 14:37:40  rogerh
  * In two of the PASN classes make the protected members public. This
  * makes OpenH323 compile in Mac OS X 10.2.1
@@ -562,7 +568,7 @@ class PASN_OctetString : public PASN_ConstrainedObject
     BOOL DecodePER(PPER_Stream & strm);
     void EncodePER(PPER_Stream & strm) const;
 
-    BOOL DecodeSubType(PASN_Object &);
+    BOOL DecodeSubType(PASN_Object &) const;
     void EncodeSubType(const PASN_Object &);
 
   protected:
@@ -647,12 +653,12 @@ class PASN_BMPString : public PASN_ConstrainedObject
     PASN_BMPString(const PASN_BMPString & other);
     PASN_BMPString & operator=(const PASN_BMPString & other);
 
-    PASN_BMPString & operator=(const char * v);
-    PASN_BMPString & operator=(const PString & v) { return operator=((const char *)v); }
+    PASN_BMPString & operator=(const char * v) { return operator=(PString(v).AsUCS2()); }
+    PASN_BMPString & operator=(const PString & v) { return operator=(v.AsUCS2()); }
     PASN_BMPString & operator=(const PWORDArray & v);
     operator PString() const { return GetValue(); }
     operator PWORDArray() const { return value; }
-    PString GetValue() const;
+    PString GetValue() const { return value; }
     void GetValue(PWORDArray & v) const { v = value; }
     void SetValue(const char * v) { operator=(v); }
     void SetValue(const PString & v) { operator=((const char *)v); }
