@@ -5,7 +5,7 @@
  *
  * Portable Windows Library
  *
- * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
+ * Copyright (c) 1993-2002 Equivalence Pty. Ltd.
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.0 (the "License"); you may not use this file except in
@@ -21,12 +21,22 @@
  *
  * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
  *
- * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
- * All Rights Reserved.
- *
  * Contributor(s): ______________________________________.
  *
  * $Log: http.h,v $
+ * Revision 1.57  2002/12/03 22:37:36  robertj
+ * Removed get document that just returns a content length as the chunked
+ *   transfer encoding makes this very dangerous.
+ * Added GetTextDocument() to get a URL content into a PString.
+ * Added a version pf PostData() that gets the reponse content into a PString.
+ * Added ReadContentBody() that takes a PString, not just PBYTEArray.
+ *
+ * Revision 1.56  2002/11/06 22:47:23  robertj
+ * Fixed header comment (copyright etc)
+ *
+ * Revision 1.55  2002/10/10 04:43:43  robertj
+ * VxWorks port, thanks Martijn Roest
+ *
  * Revision 1.54  2002/10/02 08:54:34  craigs
  * Added support for XMLRPC server
  *
@@ -355,7 +365,7 @@ class PHTTP : public PInternetProtocol
     enum StatusCode {
       Continue = 100,              // 100 - Continue
       SwitchingProtocols,          // 101 - upgrade allowed
-      OK = 200,                    // 200 - request has succeeded
+      RequestOK = 200,             // 200 - request has succeeded
       Created,                     // 201 - new resource created: entity body contains URL
       Accepted,                    // 202 - request accepted, but not yet completed
       NonAuthoritativeInformation, // 203 - not definitive entity header
@@ -524,6 +534,10 @@ class PHTTPClient : public PHTTP
       PMIMEInfo & replyMIME,
       PBYTEArray & body
     );
+    BOOL ReadContentBody(
+      PMIMEInfo & replyMIME,
+      PString & body
+    );
 
 
     /** Get the document specified by the URL.
@@ -531,11 +545,12 @@ class PHTTPClient : public PHTTP
        @return
        TRUE if document is being transferred.
      */
-    BOOL GetDocument(
+    BOOL GetTextDocument(
       const PURL & url,         // Universal Resource Locator for document.
-      PINDEX & contentLength,   // Length of body to read
+      PString & document,       // Body read
       BOOL persist = TRUE
     );
+
     /** Get the document specified by the URL.
 
        @return
@@ -574,8 +589,26 @@ class PHTTPClient : public PHTTP
       BOOL persist = TRUE
     );
 
+    /** Post the data specified to the URL.
+
+       @return
+       TRUE if document is being transferred.
+     */
+    BOOL PostData(
+      const PURL & url,       // Universal Resource Locator for document.
+      PMIMEInfo & outMIME,    // MIME info in request
+      const PString & data,   // Information posted to the HTTP server.
+      PMIMEInfo & replyMIME,  // MIME info in response
+      PString & replyBody,    // Body of response
+      BOOL persist = TRUE
+    );
+
   protected:
     BOOL AssureConnect(const PURL & url, PMIMEInfo & outMIME);
+    BOOL InternalReadContentBody(
+      PMIMEInfo & replyMIME,
+      PAbstractArray & body
+    );
 
     PString userAgentName;
 };
