@@ -27,6 +27,15 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: pdirect.h,v $
+ * Revision 1.38  2002/11/20 00:13:43  robertj
+ * Fixed some documentation
+ *
+ * Revision 1.37  2002/11/19 12:07:02  robertj
+ * Added function to get root directory.
+ *
+ * Revision 1.36  2002/11/19 10:34:59  robertj
+ * Added function to extract a path as an array of directories components.
+ *
  * Revision 1.35  2002/09/16 01:08:59  robertj
  * Added #define so can select if #pragma interface/implementation is used on
  *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
@@ -340,6 +349,19 @@ class PDirectory : public PFilePathString
      */
     BOOL IsRoot() const;
 
+    /**Get the root directory of a volume.
+    
+       @return
+       root directory.
+     */
+    PDirectory GetRoot() const;
+
+    /**Get the directory path as an array of strings.
+       The first element in the array is the volume string, eg under Win32 it
+       is "c:" or "\\machine", while under unix it is an empty string.
+      */
+    PStringArray GetPath() const;
+
     /**Determine if the character #ch# is a directory path
        separator.
 
