@@ -5,7 +5,7 @@
  *
  * Portable Windows Library
  *
- * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
+ * Copyright (c) 1993-2002 Equivalence Pty. Ltd.
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.0 (the "License"); you may not use this file except in
@@ -21,12 +21,24 @@
  *
  * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
  *
- * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
- * All Rights Reserved.
- *
  * Contributor(s): ______________________________________.
  *
  * $Log: httpclnt.cxx,v $
+ * Revision 1.31  2002/12/03 22:38:35  robertj
+ * Removed get document that just returns a content length as the chunked
+ *   transfer encoding makes this very dangerous.
+ * Added GetTextDocument() to get a URL content into a PString.
+ * Added a version pf PostData() that gets the reponse content into a PString.
+ * Added ReadContentBody() that takes a PString, not just PBYTEArray.
+ * Fixed bug where conten-encoding must be checked even if there is a
+ *   full content length MIME field.
+ *
+ * Revision 1.30  2002/11/06 22:47:25  robertj
+ * Fixed header comment (copyright etc)
+ *
+ * Revision 1.29  2002/10/10 04:43:44  robertj
+ * VxWorks port, thanks Martijn Roest
+ *
  * Revision 1.28  2002/05/28 01:41:50  robertj
  * Fixed bug in reading chunked data, thanks David Iodice
  *
@@ -366,7 +378,7 @@ BOOL PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
     UnRead(http);
 
     if (http.Find("HTTP/") == P_MAX_INDEX) {
-      lastResponseCode = PHTTP::OK;
+      lastResponseCode = PHTTP::RequestOK;
       lastResponseInfo = "HTTP/0.9";
       return TRUE;
     }
@@ -393,31 +405,47 @@ BOOL PHTTPClient::ReadResponse(PMIMEInfo & replyMIME)
 }
 
 
+BOOL PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PString & body)
+{
+  BOOL ok = InternalReadContentBody(replyMIME, body);
+  body.SetSize(body.GetSize()+1);
+  return ok;
+}
+
+
 BOOL PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PBYTEArray & body)
 {
-  if (replyMIME.Contains(ContentLengthTag)) {
-    PINDEX length = replyMIME.GetInteger(ContentLengthTag);
-    body.SetSize(length);
-    return ReadBlock(body.GetPointer(), length);
-  }
+  return InternalReadContentBody(replyMIME, body);
+}
+
+
+BOOL PHTTPClient::InternalReadContentBody(PMIMEInfo & replyMIME, PAbstractArray & body)
+{
+  PCaselessString encoding = replyMIME(TransferEncodingTag);
+
+  if (encoding != ChunkedTag) {
+    if (replyMIME.Contains(ContentLengthTag)) {
+      PINDEX length = replyMIME.GetInteger(ContentLengthTag);
+      body.SetSize(length);
+      return ReadBlock(body.GetPointer(), length);
+    }
+
+    if (encoding.IsEmpty()) {
+      lastResponseCode = -1;
+      lastResponseInfo = "Unknown Transfer-Encoding extension";
+      return FALSE;
+    }
 
-  if (!replyMIME.Contains(TransferEncodingTag)) {
     // Must be raw, read to end file variety
     static const PINDEX ChunkSize = 2048;
     PINDEX bytesRead = 0;
-    while (ReadBlock(body.GetPointer(bytesRead+ChunkSize)+bytesRead, ChunkSize))
+    while (ReadBlock((char *)body.GetPointer(bytesRead+ChunkSize)+bytesRead, ChunkSize))
       bytesRead += GetLastReadCount();
 
     body.SetSize(bytesRead + GetLastReadCount());
     return GetErrorCode(LastReadError) == NoError;
   }
 
-  if (!(replyMIME(TransferEncodingTag) *= ChunkedTag)) {
-    lastResponseCode = -1;
-    lastResponseInfo = "Unknown Transfer-Encoding extension";
-    return FALSE;
-  }
-
   // HTTP1.1 chunked format
   PINDEX bytesRead = 0;
   for (;;) {
@@ -432,7 +460,7 @@ BOOL PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PBYTEArray & body)
       break;
 
     // Read the chunk
-    if (!ReadBlock(body.GetPointer(bytesRead+chunkLength)+bytesRead, chunkLength))
+    if (!ReadBlock((char *)body.GetPointer(bytesRead+chunkLength)+bytesRead, chunkLength))
       return FALSE;
     bytesRead+= chunkLength;
 
@@ -452,19 +480,15 @@ BOOL PHTTPClient::ReadContentBody(PMIMEInfo & replyMIME, PBYTEArray & body)
 }
 
 
-BOOL PHTTPClient::GetDocument(const PURL & url,
-                              PINDEX & contentLength,
-                              BOOL persist)
+BOOL PHTTPClient::GetTextDocument(const PURL & url,
+                                  PString & document,
+                                  BOOL persist)
 {
   PMIMEInfo outMIME, replyMIME;
   if (!GetDocument(url, outMIME, replyMIME, persist))
     return FALSE;
 
-  if (replyMIME.Contains(ContentLengthTag))
-    contentLength = (PINDEX)replyMIME[ContentLengthTag].AsUnsigned();
-  else
-    contentLength = P_MAX_INDEX;
-  return TRUE;
+  return ReadContentBody(replyMIME, document);
 }
 
 
@@ -473,7 +497,7 @@ BOOL PHTTPClient::GetDocument(const PURL & url,
                               PMIMEInfo & replyMIME,
                               BOOL persist)
 {
-  return ExecuteCommand(GET, url, outMIME, PString(), replyMIME, persist) == OK;
+  return ExecuteCommand(GET, url, outMIME, PString(), replyMIME, persist) == RequestOK;
 }
 
 
@@ -482,7 +506,7 @@ BOOL PHTTPClient::GetHeader(const PURL & url,
                             PMIMEInfo & replyMIME,
                             BOOL persist)
 {
-  return ExecuteCommand(HEAD, url, outMIME, PString(), replyMIME, persist) == OK;
+  return ExecuteCommand(HEAD, url, outMIME, PString(), replyMIME, persist) == RequestOK;
 }
 
 
@@ -498,7 +522,21 @@ BOOL PHTTPClient::PostData(const PURL & url,
     dataBody += "\r\n"; // Add CRLF for compatibility with some CGI servers.
   }
 
-  return ExecuteCommand(POST, url, outMIME, data, replyMIME, persist) == OK;
+  return ExecuteCommand(POST, url, outMIME, data, replyMIME, persist) == RequestOK;
+}
+
+
+BOOL PHTTPClient::PostData(const PURL & url,
+                           PMIMEInfo & outMIME,
+                           const PString & data,
+                           PMIMEInfo & replyMIME,
+                           PString & body,
+                           BOOL persist)
+{
+  if (!PostData(url, outMIME, data, replyMIME, persist))
+    return FALSE;
+
+  return ReadContentBody(replyMIME, body);
 }
 
 
