@@ -1,11 +1,11 @@
 /*
  * pxml.cxx
  *
- * XML parser support library
+ * XML parser support
  *
  * Portable Windows Library
  *
- * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
+ * Copyright (c) 2002 Equivalence Pty. Ltd.
  *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.0 (the "License"); you may not use this file except in
@@ -21,11 +21,29 @@
  *
  * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
  *
- * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
- * All Rights Reserved.
- *
  * Contributor(s): ______________________________________.
  *
+ * $Log: pxml.cxx,v $
+ * Revision 1.26  2002/12/16 06:38:59  robertj
+ * Added ability to specify certain elemets (by name) that are exempt from
+ *   the indent formatting. Useful for XML/RPC where leading white space is
+ *   not ignored by all servers.
+ *
+ * Revision 1.25  2002/12/10 04:41:16  robertj
+ * Added test for URL being empty, don't try and run auto load in background.
+ *
+ * Revision 1.24  2002/11/26 05:53:45  craigs
+ * Added ability to auto-reload from URL
+ *
+ * Revision 1.23  2002/11/21 08:08:52  craigs
+ * Changed to not overwrite XML data if load fails
+ *
+ * Revision 1.22  2002/11/19 07:37:25  craigs
+ * Added locking functions and LoadURL function
+ *
+ * Revision 1.21  2002/11/06 22:47:25  robertj
+ * Fixed header comment (copyright etc)
+ *
  */
 
 // This depends on the expat XML library by Jim Clark
@@ -39,6 +57,7 @@
 
 #include <ptclib/pxml.h>
 
+#define CACHE_BUFFER_SIZE   1024
 
 #if P_EXPAT
 
@@ -85,72 +104,82 @@ static void PXML_EndDocTypeDecl(void * userData)
 
 ////////////////////////////////////////////////////
 
-PXML::PXML(int _options)
+PXML::PXML(int options, const char * noIndentElements)
 {
-  Construct();
-  if (_options >= 0)
-    options = _options;
+  Construct(options, noIndentElements);
 }
 
-PXML::PXML(const PString & data, int _options)
+PXML::PXML(const PString & data, int options, const char * noIndentElements)
 {
-  Construct();
-  if (_options >= 0)
-    options = _options;
+  Construct(options, noIndentElements);
   Load(data);
 }
 
 PXML::~PXML()
 {
+  autoLoadTimer.Stop();
   RemoveAll();
 }
 
 PXML::PXML(const PXML & xml)
+  : noIndentElements(xml.noIndentElements)
 {
-  Construct();
+  Construct(xml.options, NULL);
 
-  options      = xml.options;
   loadFromFile = xml.loadFromFile;
   loadFilename = xml.loadFilename;
   version      = xml.version;
   encoding     = xml.encoding;
   standAlone   = xml.standAlone;
 
+  PWaitAndSignal m(xml.rootMutex);
+
   PXMLElement * oldRootElement = xml.rootElement;
   if (oldRootElement != NULL)
     rootElement = (PXMLElement *)oldRootElement->Clone(NULL);
 }
 
-void PXML::Construct()
+void PXML::Construct(int _options, const char * _noIndentElements)
 {
   rootElement    = NULL;
   currentElement = NULL;
   lastElement    = NULL;
-  options        = 0;
+  options        = _options > 0 ? _options : 0;
   loadFromFile   = FALSE;
   standAlone     = -2;
+
+  if (_noIndentElements != NULL)
+    noIndentElements = PString(_noIndentElements).Tokenise(' ', FALSE);
 }
 
 PXMLElement * PXML::SetRootElement(const PString & documentType)
 {
+  PWaitAndSignal m(rootMutex);
+
   if (rootElement != NULL)
     delete rootElement;
 
   rootElement = new PXMLElement(rootElement, documentType);
+
   return rootElement;
 }
 
 PXMLElement * PXML::SetRootElement(PXMLElement * element)
 {
+  PWaitAndSignal m(rootMutex);
+
   if (rootElement != NULL)
     delete rootElement;
 
   rootElement = element;
+
   return rootElement;
 }
 
 BOOL PXML::IsDirty() const
 {
+  PWaitAndSignal m(rootMutex);
+
   if (rootElement == NULL)
     return FALSE;
 
@@ -159,6 +188,8 @@ BOOL PXML::IsDirty() const
 
 PCaselessString PXML::GetDocumentType() const
 { 
+  PWaitAndSignal m(rootMutex);
+
   if (rootElement == NULL)
     return PCaselessString();
   return rootElement->GetName();
@@ -166,6 +197,8 @@ PCaselessString PXML::GetDocumentType() const
 
 BOOL PXML::LoadFile(const PFilePath & fn, int _options)
 {
+  PWaitAndSignal m(rootMutex);
+
   if (_options >= 0)
     options = _options;
 
@@ -186,15 +219,128 @@ BOOL PXML::LoadFile(const PFilePath & fn, int _options)
   return Load(data);
 }
 
+BOOL PXML::LoadURL(const PURL & url)
+{
+  return LoadURL(url, PMaxTimeInterval, -1);
+}
+
+
+BOOL PXML::LoadURL(const PURL & url, const PTimeInterval & timeout, int _options)
+{
+  if (url.IsEmpty()) {
+    errorString = "Cannot load empty URL";
+    errorCol = errorLine = 0;
+    return FALSE;
+  }
+
+  PString data;
+  if (url.GetScheme() == "file") 
+    return LoadFile(url.AsFilePath());
+
+  PHTTPClient client;
+  PINDEX contentLength;
+  PMIMEInfo outMIME, replyMIME;
+
+  // make sure we do not hang around for ever
+  client.SetReadTimeout(timeout);
+
+  // get the resource header information
+  if (!client.GetDocument(url, outMIME, replyMIME)) {
+    errorString = PString("Cannot load URL") & url.AsString();
+    errorCol = errorLine = 0;
+    return FALSE;
+  }
+
+  // get the length of the data
+  if (!replyMIME.Contains(PHTTPClient::ContentLengthTag))
+    contentLength = (PINDEX)replyMIME[PHTTPClient::ContentLengthTag].AsUnsigned();
+  else
+    contentLength = P_MAX_INDEX;
+
+  // download the resource into memory
+  PINDEX offs = 0;
+  for (;;) {
+    PINDEX len;
+    if (contentLength == P_MAX_INDEX)
+      len = CACHE_BUFFER_SIZE;
+    else if (offs == contentLength)
+      break;
+    else
+      len = PMIN(contentLength = offs, CACHE_BUFFER_SIZE);
+
+    if (!client.Read(offs + data.GetPointer(offs + len), len))
+      break;
+
+    len = client.GetLastReadCount();
+
+    offs += len;
+  }
+
+  return Load(data, _options);
+}
+
+BOOL PXML::StartAutoReloadURL(const PURL & url, 
+                              const PTimeInterval & timeout, 
+                              const PTimeInterval & refreshTime,
+                              int _options)
+{
+  if (url.IsEmpty()) {
+    autoLoadError = "Cannot auto-load empty URL";
+    return FALSE;
+  }
+
+  PWaitAndSignal m(autoLoadMutex);
+  autoLoadTimer.Stop();
+
+  SetOptions(_options);
+  autoloadURL      = url;
+  autoLoadWaitTime = timeout;
+  autoLoadError    = PString::Empty();
+  autoLoadTimer.SetNotifier(PCREATE_NOTIFIER(AutoReloadTimeout));
+
+  BOOL stat = AutoLoadURL();
+
+  autoLoadTimer = refreshTime;
+
+  return stat;
+}
+
+void PXML::AutoReloadTimeout(PTimer &, INT)
+{
+  PThread::Create(PCREATE_NOTIFIER(AutoReloadThread), PThread::AutoDeleteThread);
+}
+
+void PXML::AutoReloadThread(PThread &, INT)
+{
+  PWaitAndSignal m(autoLoadMutex);
+  OnAutoLoad(AutoLoadURL());
+  autoLoadTimer.Reset();
+}
+
+BOOL PXML::AutoLoadURL()
+{
+  BOOL stat = LoadURL(autoloadURL, autoLoadWaitTime);
+  if (stat)
+    autoLoadError = PString::Empty();
+  else 
+    autoLoadError = GetErrorString() + psprintf(" at line %i, column %i", GetErrorLine(), GetErrorColumn());
+  return stat;
+}
+
+BOOL PXML::StopAutoReloadURL()
+{
+  PWaitAndSignal m(autoLoadMutex);
+  autoLoadTimer.Stop();
+  return TRUE;
+}
+
+
 BOOL PXML::Load(const PString & data, int _options)
 {
   if (_options >= 0)
     options = _options;
 
-  if (rootElement != NULL) {
-    delete rootElement;
-    rootElement = NULL;
-  }
+  loadingRootElement = NULL;
 
   XML_Parser parser = XML_ParserCreate(NULL);
   XML_SetUserData(parser, this);
@@ -216,6 +362,18 @@ BOOL PXML::Load(const PString & data, int _options)
 
   XML_ParserFree(parser);
 
+  if (stat) {
+    {
+    PWaitAndSignal m(rootMutex);
+      if (rootElement != NULL) {
+        delete rootElement;
+        rootElement = NULL;
+      }
+      rootElement = loadingRootElement;
+    }
+    OnLoaded();
+  }
+
   return stat;
 }
 
@@ -232,6 +390,8 @@ BOOL PXML::Save(int _options)
 
 BOOL PXML::SaveFile(const PFilePath & fn, int _options)
 {
+  PWaitAndSignal m(rootMutex);
+
   PFile file;
   if (!file.Open(fn, PFile::WriteOnly)) 
     return FALSE;
@@ -245,6 +405,8 @@ BOOL PXML::SaveFile(const PFilePath & fn, int _options)
 
 BOOL PXML::Save(PString & data, int _options)
 {
+  PWaitAndSignal m(rootMutex);
+
   if (_options >= 0)
     options = _options;
 
@@ -256,6 +418,8 @@ BOOL PXML::Save(PString & data, int _options)
 
 void PXML::RemoveAll()
 {
+  PWaitAndSignal m(rootMutex);
+
   if (rootElement != NULL) {
     delete rootElement;
     rootElement = NULL;
@@ -280,6 +444,19 @@ PXMLElement * PXML::GetElement(PINDEX idx) const
   return (PXMLElement *)(rootElement->GetElement(idx));
 }
 
+BOOL PXML::RemoveElement(PINDEX idx)
+{
+  if (rootElement == NULL)
+    return FALSE;
+
+  if (idx >= rootElement->GetSize())
+    return FALSE;
+
+  rootElement->RemoveElement(idx);
+  return TRUE;
+}
+
+
 PINDEX PXML::GetNumElements() const
 {
 	if (rootElement == NULL) return 0;
@@ -300,8 +477,8 @@ void PXML::StartElement(const char * name, const char **attrs)
   currentElement = newElement;
   lastElement    = NULL;
 
-  if (rootElement == NULL) 
-    rootElement = currentElement;
+  if (loadingRootElement == NULL) 
+    loadingRootElement = currentElement;
 }
 
 void PXML::EndElement(const char * /*name*/)
@@ -349,9 +526,15 @@ void PXML::EndDocTypeDecl()
 {
 }
 
+BOOL PXML::IsNoIndentElement(const PString & elementName) const
+{
+  return noIndentElements.GetValuesIndex(elementName) != P_MAX_INDEX;
+}
+
+
 void PXML::PrintOn(ostream & strm) const
 {
-  BOOL newLine = (options & PXML::NewLineAfterElement) != 0;
+  BOOL newLine = (options & (PXML::Indent|PXML::NewLineAfterElement)) != 0;
 
 //<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
 
@@ -386,7 +569,7 @@ void PXML::PrintOn(ostream & strm) const
     strm << "<!DOCTYPE " << rootElement->GetName() << '>';
     if (newLine)
       strm << endl;
-    rootElement->PrintOn(strm, (options & Indent) ? 0 : -1, options);
+    rootElement->Output(strm, *this, 2);
   }
 }
 
@@ -455,12 +638,18 @@ PXMLData::PXMLData(PXMLElement * _parent, const char * data, int len)
   value = PString(data, len);
 }
 
-void PXMLData::PrintOn(ostream & strm, int indent, int options) const
+void PXMLData::Output(ostream & strm, const PXML & xml, int indent) const
 {
-  if (indent > 0)
-    strm << setw(indent) << " ";
+  int options = xml.GetOptions();
+  if (xml.IsNoIndentElement(parent->GetName()))
+    options &= ~PXML::Indent;
+
+  if (options & PXML::Indent)
+    strm << setw(indent-1) << " ";
+
   strm << value;
-  if (indent >= 0 || ((options & PXML::NewLineAfterElement) != 0))
+
+  if ((options & (PXML::Indent|PXML::NewLineAfterElement)) != 0)
     strm << endl;
 }
 
@@ -524,6 +713,16 @@ PXMLObject * PXMLElement::GetElement(PINDEX idx) const
   return &subObjects[idx];
 }
 
+BOOL PXMLElement::RemoveElement(PINDEX idx)
+{
+  if (idx >= subObjects.GetSize())
+    return FALSE;
+
+  subObjects.RemoveAt(idx);
+  return TRUE;
+}
+
+
 PString PXMLElement::GetAttribute(const PCaselessString & key) const
 {
   return attributes(key);
@@ -559,12 +758,15 @@ BOOL PXMLElement::HasAttribute(const PCaselessString & key)
   return attributes.Contains(key);
 }
 
-void PXMLElement::PrintOn(ostream & strm, int indent, int options) const
+void PXMLElement::Output(ostream & strm, const PXML & xml, int indent) const
 {
-  BOOL newLine = (options & PXML::NewLineAfterElement) != 0;
+  int options = xml.GetOptions();
+
+  BOOL newLine = (options & (PXML::Indent|PXML::NewLineAfterElement)) != 0;
+
+  if ((options & PXML::Indent) != 0)
+    strm << setw(indent-1) << " ";
 
-  if (indent > 0)
-    strm << setw(indent) << " ";
   strm << '<' << name;
 
   PINDEX i;
@@ -581,18 +783,22 @@ void PXMLElement::PrintOn(ostream & strm, int indent, int options) const
     strm << " />";
     if (newLine)
       strm << endl;
-  } else {
+  }
+  else {
+    BOOL indenting = (options & PXML::Indent) != 0 && !xml.IsNoIndentElement(name);
+
     strm << '>';
-    if (indent >= 0 || newLine)
+    if (indenting)
       strm << endl;
   
     for (i = 0; i < subObjects.GetSize(); i++) 
-      subObjects[i].PrintOn(strm, (indent < 0) ? -1 : (indent + 2), options);
+      subObjects[i].Output(strm, xml, indent + 2);
+
+    if (indenting)
+      strm << setw(indent-1) << " ";
 
-    if (indent > 0)
-      strm << setw(indent) << " ";
     strm << "</" << name << '>';
-    if (indent >= 0 || newLine)
+    if (newLine)
       strm << endl;
   }
 }
