@@ -1,13 +1,56 @@
 /*
  * pxmlrpc.cxx
  *
- * PWLib application source file for PXMLRPC
+ * XML/RPC support
  *
- * Main program entry point.
+ * Portable Windows Library
  *
- * Copyright 2002 Equivalence
+ * Copyright (c) 2002 Equivalence Pty. Ltd.
+ *
+ * The contents of this file are subject to the Mozilla Public License
+ * Version 1.0 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS"
+ * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
+ * the License for the specific language governing rights and limitations
+ * under the License.
+ *
+ * The Original Code is Portable Windows Library.
+ *
+ * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
+ *
+ * Contributor(s): ______________________________________.
  *
  * $Log: pxmlrpc.cxx,v $
+ * Revision 1.21  2002/12/16 06:53:19  robertj
+ * Added ability to specify certain elemets (by name) that are exempt from
+ *   the indent formatting. Useful for XML/RPC where leading white space is
+ *   not ignored by all servers.
+ * Allowed for some servers that only send "string" type for "int" etc
+ * Fixed problem with autodetecting reply that is a single struct.
+ *
+ * Revision 1.20  2002/12/13 01:12:24  robertj
+ * Added copy constructor and assignment operator to XML/RPC structs
+ *
+ * Revision 1.19  2002/12/10 03:51:17  robertj
+ * Fixed member variable display in structure
+ *
+ * Revision 1.18  2002/12/09 04:06:44  robertj
+ * Added macros for defining multi-argument functions
+ *
+ * Revision 1.17  2002/12/04 00:31:13  robertj
+ * Fixed GNU compatibility
+ *
+ * Revision 1.16  2002/12/04 00:15:44  robertj
+ * Changed usage of PHTTPClient so supports chunked transfer encoding.
+ * Large enhancement to create automatically encoding and decoding structures
+ *   using macros to build a class.
+ *
+ * Revision 1.15  2002/11/06 22:47:25  robertj
+ * Fixed header comment (copyright etc)
+ *
  * Revision 1.14  2002/10/08 12:26:31  craigs
  * Changed struct members to always contain name/value in that order
  *
@@ -70,7 +113,14 @@
 #include <ptclib/mime.h>
 #include <ptclib/http.h>
 
+
+static const char NoIndentElements[] = "methodName name string int boolean double dateTime.iso8601";
+
+
+/////////////////////////////////////////////////////////////////
+
 PXMLRPCBlock::PXMLRPCBlock()
+  : PXML(-1, NoIndentElements)
 {
   faultCode = P_MAX_INDEX;
   SetRootElement("methodResponse");
@@ -78,6 +128,7 @@ PXMLRPCBlock::PXMLRPCBlock()
 }
 
 PXMLRPCBlock::PXMLRPCBlock(const PString & method)
+  : PXML(-1, NoIndentElements)
 {
   faultCode = P_MAX_INDEX;
   SetRootElement("methodCall");
@@ -85,6 +136,25 @@ PXMLRPCBlock::PXMLRPCBlock(const PString & method)
   params = NULL;
 }
 
+PXMLRPCBlock::PXMLRPCBlock(const PString & method, const PXMLRPCStructBase & data)
+  : PXML(-1, NoIndentElements)
+{
+  faultCode = P_MAX_INDEX;
+  SetRootElement("methodCall");
+  rootElement->AddChild(new PXMLElement(rootElement, "methodName", method));
+  params = NULL;
+
+  for (PINDEX i = 0; i < data.GetNumVariables(); i++) {
+    PXMLRPCVariableBase & variable = data.GetVariable(i);
+    PXMLRPCStructBase * structVar = variable.GetStruct();
+    if (structVar != NULL)
+      AddParam(*structVar);
+    else
+      AddParam(CreateValueElement(new PXMLElement(NULL, variable.GetType(), variable.ToString())));
+  }
+}
+
+
 BOOL PXMLRPCBlock::Load(const PString & str)
 {
   if (!PXML::Load(str))
@@ -172,6 +242,29 @@ PXMLElement * PXMLRPCBlock::CreateStruct(const PStringToString & dict, const PSt
 }
 
 
+PXMLElement * PXMLRPCBlock::CreateStruct(const PXMLRPCStructBase & data)
+{
+  PXMLElement * structElement = new PXMLElement(NULL, "struct");
+  PXMLElement * valueElement  = PXMLRPCBlock::CreateValueElement(structElement);
+
+  PINDEX i;
+  for (i = 0; i < data.GetNumVariables(); i++) {
+    PXMLElement * element;
+    PXMLRPCVariableBase & variable = data.GetVariable(i);
+
+    PXMLRPCStructBase * nested = variable.GetStruct();
+    if (nested != NULL)
+      element = CreateStruct(*nested);
+    else
+      element = CreateScalar(variable.GetType(), variable.ToString());
+
+    structElement->AddChild(CreateMember(variable.GetName(), element));
+  }
+
+  return valueElement;
+}
+
+
 PXMLElement * PXMLRPCBlock::CreateMember(const PString & name, PXMLElement * value)
 {
   PXMLElement * member = new PXMLElement(NULL, "member");
@@ -216,6 +309,11 @@ void PXMLRPCBlock::AddBinary(const PBYTEArray & data)
   AddParam(CreateBinary(data)); 
 }
 
+void PXMLRPCBlock::AddParam(const PXMLRPCStructBase & data)
+{
+  AddParam(CreateStruct(data));
+}
+
 void PXMLRPCBlock::AddStruct(const PStringToString & dict)
 {
   AddParam(CreateStruct(dict, "string"));
@@ -281,50 +379,6 @@ BOOL PXMLRPCBlock::ValidateResponse()
   return TRUE;
 }
 
-BOOL PXMLRPCBlock::ParseStruct(PXMLElement * structElement, 
-                           PStringToString & structDict)
-{
-  if ((structElement == NULL) || (structElement->GetName() != "struct")) {
-    SetFault(PXMLRPC::ParamNotStruct, "Param is not struct");
-    PTRACE(2, "XMLRPC\t" << GetFaultText());
-    return FALSE;
-  }
-
-  PINDEX i = 0;
-  while (i < structElement->GetSize()) {
-    PXMLElement * member = structElement->GetElement("member", i++);
-    if (member != NULL) {
-      PXMLElement * nameElement = member->GetElement("name");
-      PXMLElement * element     = member->GetElement("value");
-      if ((nameElement == NULL) || (element == NULL)) {
-        PStringStream txt;
-        txt << "Member " << i << " incomplete";
-        SetFault(PXMLRPC::MemberIncomplete, txt);
-        PTRACE(2, "XMLRPC\t" << GetFaultText());
-        return FALSE;
-      }
-
-      if (nameElement->GetName() != "name") {
-        PStringStream txt;
-        txt << "Member " << i << " unnamed";
-        SetFault(PXMLRPC::MemberUnnamed, txt);
-        PTRACE(2, "XMLRPC\t" << GetFaultText());
-        return FALSE;
-      }
-
-      PString name = nameElement->GetData();
-      PString value;
-      PString type;
-      if (!ParseScalar(element, type, value))
-        return FALSE;
-
-      structDict.SetAt(name, value);
-    }
-  }
-
-  return TRUE;
-}
-
 BOOL PXMLRPCBlock::ParseScalar(PXMLElement * valueElement, 
                                    PString & type, 
                                    PString & value)
@@ -351,6 +405,123 @@ BOOL PXMLRPCBlock::ParseScalar(PXMLElement * valueElement,
   return TRUE;
 }
 
+BOOL PXMLRPCBlock::ParseStruct(PXMLElement * structElement, 
+                               PStringToString & structDict)
+{
+  if ((structElement == NULL) || (structElement->GetName() != "struct")) {
+    SetFault(PXMLRPC::ParamNotStruct, "Param is not struct");
+    PTRACE(2, "XMLRPC\t" << GetFaultText());
+    return FALSE;
+  }
+
+  for (PINDEX i = 0; i < structElement->GetSize(); i++) {
+    PString name;
+    PXMLElement * element = ParseStructElement(structElement, i, name);
+    if (element == NULL)
+      return FALSE;
+
+    PString value;
+    PString type;
+    if (!ParseScalar(element, type, value))
+      return FALSE;
+
+    structDict.SetAt(name, value);
+  }
+
+  return TRUE;
+}
+
+BOOL PXMLRPCBlock::ParseStruct(PXMLElement * structElement, 
+                               PXMLRPCStructBase & data)
+{
+  if ((structElement == NULL) || (structElement->GetName() != "struct")) {
+    SetFault(PXMLRPC::ParamNotStruct, "Param is not struct");
+    PTRACE(2, "XMLRPC\t" << GetFaultText());
+    return FALSE;
+  }
+
+  for (PINDEX i = 0; i < structElement->GetSize(); i++) {
+    PString name;
+    PXMLElement * element = ParseStructElement(structElement, i, name);
+    if (element == NULL)
+      return FALSE;
+
+    PXMLRPCVariableBase * variable = data.GetVariable(name);
+    if (variable != NULL) {
+      PXMLRPCStructBase * nested = variable->GetStruct();
+      if (nested != NULL) {
+        PXMLElement * nestedElement = (PXMLElement *)element->GetElement(0);
+        if ((nestedElement == NULL) || !nestedElement->IsElement()) {
+          SetFault(PXMLRPC::ScalarWithoutElement, "Scalar without sub-element");
+          PTRACE(2, "XMLRPC\t" << GetFaultText());
+          return FALSE;
+        }
+
+        if (nestedElement->GetName() != "struct") {
+          PTRACE(2, "RPCXML\tMember " << i << " is not struct as expected");
+          return FALSE;
+        }
+
+        if (!ParseStruct(nestedElement, *nested))
+          return FALSE;
+      }
+      else {
+        PString value;
+        PCaselessString type;
+        if (!ParseScalar(element, type, value))
+          return FALSE;
+
+        if (type != "string" && type != variable->GetType()) {
+          PTRACE(2, "RPCXML\tMember " << i << " is not of expected type: " << variable->GetType());
+          return FALSE;
+        }
+
+        variable->FromString(value);
+      }
+    }
+  }
+
+  return TRUE;
+}
+
+PXMLElement * PXMLRPCBlock::ParseStructElement(PXMLElement * structElement,
+                                               PINDEX idx,
+                                               PString & name)
+{
+  if (structElement == NULL)
+    return NULL;
+
+  PXMLElement * member = structElement->GetElement("member", idx);
+  if (member == NULL) {
+    PStringStream txt;
+    txt << "Member " << idx << " missing";
+    SetFault(PXMLRPC::MemberIncomplete, txt);
+    PTRACE(2, "XMLRPC\t" << GetFaultText());
+    return NULL;
+  }
+
+  PXMLElement * nameElement  = member->GetElement("name");
+  PXMLElement * valueElement = member->GetElement("value");
+  if ((nameElement == NULL) || (valueElement == NULL)) {
+    PStringStream txt;
+    txt << "Member " << idx << " incomplete";
+    SetFault(PXMLRPC::MemberIncomplete, txt);
+    PTRACE(2, "XMLRPC\t" << GetFaultText());
+    return NULL;
+  }
+
+  if (nameElement->GetName() != "name") {
+    PStringStream txt;
+    txt << "Member " << idx << " unnamed";
+    SetFault(PXMLRPC::MemberUnnamed, txt);
+    PTRACE(2, "XMLRPC\t" << GetFaultText());
+    return NULL;
+  }
+
+  name = nameElement->GetData();
+  return valueElement;
+}
+
 PXMLElement * PXMLRPCBlock::GetParam(PINDEX idx) const 
 { 
   if (params == NULL) 
@@ -368,6 +539,40 @@ PXMLElement * PXMLRPCBlock::GetParam(PINDEX idx) const
 }
 
 
+BOOL PXMLRPCBlock::GetParams(PXMLRPCStructBase & data)
+{
+  if (params == NULL) 
+    return FALSE;
+
+  // Special case to allow for server implementations that always return
+  // values as a struct rather than multiple parameters.
+  if (GetParamCount() == 1 &&
+          (data.GetNumVariables() > 1 || data.GetVariable(0).GetStruct() == NULL)) {
+    PString type, value;
+    if (ParseScalar(GetParam(0), type, value) && type == "struct")
+      return GetParam(0, data);
+  }
+
+  for (PINDEX i = 0; i < data.GetNumVariables(); i++) {
+    PXMLRPCVariableBase & variable = data.GetVariable(i);
+    PXMLRPCStructBase * structVar = variable.GetStruct();
+    if (structVar != NULL) {
+      if (!GetParam(i, *structVar))
+        return FALSE;
+    }
+    else {
+      PString value;
+      if (!GetExpectedParam(i, variable.GetType(), value))
+        return FALSE;
+
+      variable.FromString(value);
+    }
+  }
+
+  return TRUE;
+}
+
+
 BOOL PXMLRPCBlock::GetParam(PINDEX idx, PString & type, PString & value)
 {
   // get the parameter
@@ -441,40 +646,61 @@ BOOL PXMLRPCBlock::GetParam(PINDEX idx, PTime & val, int tz)
 }
 
 BOOL PXMLRPCBlock::GetParam(PINDEX idx, PStringToString & result)
+{
+  PXMLElement * element = GetStructParam(idx);
+  if (element == NULL)
+    return FALSE;
+
+  return ParseStruct(element, result);
+}
+
+BOOL PXMLRPCBlock::GetParam(PINDEX idx, PXMLRPCStructBase & data)
+{
+  PXMLElement * element = GetStructParam(idx);
+  if (element == NULL)
+    return FALSE;
+
+  return ParseStruct(element, data);
+}
+
+
+PXMLElement * PXMLRPCBlock::GetStructParam(PINDEX idx)
 {
   PXMLElement * param = GetParam(idx);
   if (param == NULL) {
     PTRACE(3, "XMLRPC\tCannot get struct parm " << idx);
-    return FALSE;
+    return NULL;
   }
 
   if (param->GetName() != "value") {
     SetFault(PXMLRPC::ParamNotValue, "Struct value does not contain value element");
     PTRACE(2, "RPCXML\t" << GetFaultText());
-    return FALSE;
+    return NULL;
   }
 
   PXMLElement * element = (PXMLElement *)param->GetElement(0);
   if ((element == NULL) || !element->IsElement()) {
     SetFault(PXMLRPC::ScalarWithoutElement, "Scalar without sub-element");
     PTRACE(2, "XMLRPC\t" << GetFaultText());
-    return FALSE;
+    return NULL;
   }
 
   if (element->GetName() != "struct") {
     PTRACE(2, "RPCXML\tParam " << idx << " is not struct as expected");
-    return FALSE;
+    return NULL;
   }
 
-  return ParseStruct(element, result);
+  return element;
 }
 
+
 ////////////////////////////////////////////////////////
 
-PXMLRPC::PXMLRPC(const PURL & _url)
+PXMLRPC::PXMLRPC(const PURL & _url, unsigned opts)
   : url(_url)
 {
   timeout = 10000;
+  options = opts;
 }
 
 BOOL PXMLRPC::MakeRequest(const PString & method)
@@ -503,11 +729,23 @@ BOOL PXMLRPC::MakeRequest(PXMLRPCBlock & request, PXMLRPCBlock & response)
   return FALSE;
 }
 
+BOOL PXMLRPC::MakeRequest(const PString & method, const PXMLRPCStructBase & args, PXMLRPCStructBase & reply)
+{
+  PXMLRPCBlock request(method, args);
+  PXMLRPCBlock response;
+
+  if (!MakeRequest(request, response))
+    return FALSE;
+
+  return response.GetParams(reply);
+}
+
+
 BOOL PXMLRPC::PerformRequest(PXMLRPCBlock & request, PXMLRPCBlock & response)
 {
   // create XML version of request
   PString requestXML;
-  if (!request.Save(requestXML)) {
+  if (!request.Save(requestXML, options)) {
     PStringStream txt;
     txt << "Error creating request XML ("
         << request.GetErrorLine() 
@@ -521,56 +759,46 @@ BOOL PXMLRPC::PerformRequest(PXMLRPCBlock & request, PXMLRPCBlock & response)
   // make sure the request ends with a newline
   requestXML += "\n";
 
-  PTRACE(5, "XMLRPC\tOutgoing XML is " << requestXML);
-
   // do the request
   PHTTPClient client;
   PMIMEInfo sendMIME, replyMIME;
-  sendMIME.SetAt("Server",              url.GetHostName());
+  sendMIME.SetAt("Server", url.GetHostName());
   sendMIME.SetAt(PHTTP::ContentTypeTag, "text/xml");
 
+  PTRACE(5, "XMLRPC\tOutgoing XML/RPC:\n" << url << '\n' << sendMIME << requestXML);
+
   // apply the timeout
   client.SetReadTimeout(timeout);
 
+  PString replyXML;
+
   // do the request
-  BOOL ok = client.PostData(url, sendMIME, requestXML, replyMIME);
+  BOOL ok = client.PostData(url, sendMIME, requestXML, replyMIME, replyXML);
 
-  // get length of response
-  PINDEX contentLength;
-  if (replyMIME.Contains(PHTTP::ContentLengthTag))
-    contentLength = (PINDEX)replyMIME[PHTTP::ContentLengthTag].AsUnsigned();
-  else if (ok)
-    contentLength = P_MAX_INDEX;
-  else
-    contentLength = 0;
-
-  // read the response
-  PString replyBody = client.ReadString(contentLength);
-
-  PTRACE(5, "XMLRPC\tIncomign XML is " << replyBody);
+  PTRACE(5, "XMLRPC\tIncoming XML/RPC:\n" << replyMIME << replyXML);
 
   // make sure the request worked
-  if (!ok || replyBody.IsEmpty()) {
+  if (!ok) {
     PStringStream txt;
     txt << "HTTP POST failed: "
         << client.GetLastResponseCode() << ' '
         << client.GetLastResponseInfo() << '\n'
         << replyMIME << '\n'
-        << replyBody;
+        << replyXML;
     response.SetFault(PXMLRPC::HTTPPostFailed, txt);
     PTRACE(2, "XMLRPC\t" << response.GetFaultText());
     return FALSE;
   }
 
   // parse the response
-  if (!response.Load(replyBody)) {
+  if (!response.Load(replyXML)) {
     PStringStream txt;
     txt << "Error parsing response XML ("
         << response.GetErrorLine() 
         << ") :" 
         << response.GetErrorString() << '\n';
 
-    PStringArray lines = replyBody.Lines();
+    PStringArray lines = replyXML.Lines();
     for (int offset = -2; offset <= 2; offset++) {
       int line = response.GetErrorLine() + offset;
       if (line >= 0 && line < lines.GetSize())
@@ -617,4 +845,98 @@ PString PXMLRPC::PTimeToISO8601(const PTime & time)
 
 }
 
+
+/////////////////////////////////////////////////////////////////
+
+PXMLRPCVariableBase::PXMLRPCVariableBase(const char * n, const char * t)
+  : name(n),
+    type(t != NULL ? t : "string")
+{
+  PXMLRPCStructBase::GetInitialiser().AddVariable(this);
+}
+
+
+PString PXMLRPCVariableBase::ToString() const
+{
+  PStringStream stream;
+  PrintOn(stream);
+  return stream;
+}
+
+
+void PXMLRPCVariableBase::FromString(const PString & str)
+{
+  PStringStream stream(str);
+  ReadFrom(stream);
+}
+
+
+PString PXMLRPCVariableBase::ToBase64(PAbstractArray & data) const
+{
+  return PBase64::Encode(data.GetPointer(), data.GetSize());
+}
+
+
+void PXMLRPCVariableBase::FromBase64(const PString & str, PAbstractArray & data)
+{
+  PBase64 decoder;
+  decoder.StartDecoding();
+  decoder.ProcessDecoding(str);
+  data = decoder.GetDecodedData();
+}
+
+
+/////////////////////////////////////////////////////////////////
+
+PMutex              PXMLRPCStructBase::initialiserMutex;
+PXMLRPCStructBase * PXMLRPCStructBase::initialiserInstance = NULL;
+
+
+PXMLRPCStructBase::PXMLRPCStructBase()
+{
+  variablesByOrder.DisallowDeleteObjects();
+  variablesByName.DisallowDeleteObjects();
+
+  initialiserMutex.Wait();
+  initialiserStack = initialiserInstance;
+  initialiserInstance = this;
+}
+
+
+void PXMLRPCStructBase::EndConstructor()
+{
+  initialiserInstance = initialiserStack;
+  initialiserMutex.Signal();
+}
+
+
+PXMLRPCStructBase & PXMLRPCStructBase::operator=(const PXMLRPCStructBase & other)
+{
+  for (PINDEX i = 0; i < variablesByOrder.GetSize(); i++)
+    variablesByOrder[i].Copy(other.variablesByOrder[i]);
+
+  return *this;
+}
+
+
+void PXMLRPCStructBase::PrintOn(ostream & strm) const
+{
+  for (PINDEX i = 0; i < variablesByOrder.GetSize(); i++) {
+    PXMLRPCVariableBase & var = variablesByOrder[i];
+    strm << var.GetName() << '=' << var << '\n';
+  }
+}
+
+
+void PXMLRPCStructBase::AddVariable(PXMLRPCVariableBase * var)
+{
+  variablesByOrder.Append(var);
+  variablesByName.SetAt(var->GetName(), var);
+}
+
+
 #endif 
+
+
+// End of file ///////////////////////////////////////////////////////////////
+
