@@ -22,6 +22,17 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: vxml.cxx,v $
+ * Revision 1.30  2002/12/03 22:39:14  robertj
+ * Removed get document that just returns a content length as the chunked
+ *   transfer encoding makes this very dangerous.
+ *
+ * Revision 1.29  2002/11/19 10:36:30  robertj
+ * Added functions to set anf get "file:" URL. as PFilePath and do the right
+ *   things with platform dependent directory components.
+ *
+ * Revision 1.28  2002/11/08 03:39:27  craigs
+ * Fixed problem with G.723.1 files
+ *
  * Revision 1.27  2002/09/24 13:47:41  robertj
  * Added support for more vxml commands, thanks Alexander Kovatch
  *
@@ -142,44 +153,6 @@ PDirectory PVXMLSession::cacheDir;
 PVXMLCache * PVXMLSession::resourceCache = NULL;
 PINDEX       PVXMLSession::cacheCount = 0;
 
-//////////////////////////////////////////////////////////
-
-static PURL FilenameToURL(const PFilePath & fn)
-{
-  PString url;
-#ifdef WIN32
-  url = fn.GetDirectory() + fn.GetFileName();
-  PINDEX pos = url.Find(":");
-  if (pos > 0) {
-    url = url.Left(pos) + url.Mid(pos+1);
-  }
-  url = "file://" + url;
-  url.Replace('\\', '/', TRUE);
-#else
-  url = "file://" + fn.GetDirectory() + fn.GetFileName();
-#endif
-
-  return url;
-}
-
-//////////////////////////////////////////////////////////
-
-static PFilePath URLToFilename(const PURL & url)
-{
-  PFilePath fn;
-
-#ifdef WIN32
-  PString fnStr = url.GetPathStr();
-  if ((fnStr.GetSize() > 1) && (fnStr[1] == '/'))
-    fnStr = fnStr.Left(1) + ":" + fnStr.Mid(1);
-  fnStr.Replace('/', '\\', TRUE);
-  fn = fnStr;
-#else
-  fn = url.GetPathStr();
-#endif
-
-  return fn;
-}
 
 //////////////////////////////////////////////////////////
 
@@ -291,12 +264,14 @@ BOOL PVXMLSession::Load(const PString & source)
   return FALSE;
 }
 
+
 BOOL PVXMLSession::LoadFile(const PFilePath & filename)
 {
   // create a file URL from the filename
-  return LoadURL(FilenameToURL(filename));
+  return LoadURL(filename);
 }
 
+
 BOOL PVXMLSession::LoadURL(const PURL & url)
 {
   // retreive the document (may be a HTTP get)
@@ -486,7 +461,7 @@ BOOL PVXMLSession::RetrieveResource(const PURL & url, PBYTEArray & text, PString
   // files on the local file system get loaded locally
   else if (url.GetScheme() *= "file") {
 
-    fn = URLToFilename(url);
+    fn = url.AsFilePath();
     loadFile = TRUE;
   }
 
@@ -533,25 +508,25 @@ BOOL PVXMLSession::RetrieveResource(const PURL & url, PBYTEArray & text, PString
 
 PXMLElement * PVXMLSession::FindForm(const PString & id)
 {
-	// NOTE: should have some flag to know if it is loaded
-	PXMLElement * root = xmlFile.GetRootElement();
-	if (root == NULL)
-		return NULL;
-
-	// Only handle search of top level nodes for <form> element
-	PINDEX i;
-	for (i = 0; i < root->GetSize(); i++) {
-		PXMLObject * xmlObject = root->GetElement(i); 
+  // NOTE: should have some flag to know if it is loaded
+  PXMLElement * root = xmlFile.GetRootElement();
+  if (root == NULL)
+    return NULL;
+  
+  // Only handle search of top level nodes for <form> element
+  PINDEX i;
+  for (i = 0; i < root->GetSize(); i++) {
+    PXMLObject * xmlObject = root->GetElement(i); 
     if (xmlObject->IsElement()) {
-			PXMLElement * xmlElement = (PXMLElement*)xmlObject;
-			if (
+      PXMLElement * xmlElement = (PXMLElement*)xmlObject;
+      if (
           (xmlElement->GetName() *= "form") && 
           (id.IsEmpty() || (xmlElement->GetAttribute("id") *= id))
          )
-				return xmlElement;
-		}
-	}
-	return NULL;
+        return xmlElement;
+    }
+  }
+  return NULL;
 }
 
 
@@ -1103,7 +1078,7 @@ BOOL PVXMLSession::TraverseAudio()
 
         // attempt to load the resource if a file
         else if (url.GetScheme() *= "file") {
-          fn = URLToFilename(url);
+          fn = url.AsFilePath();
           haveFn = TRUE;
         }
 
@@ -1519,7 +1494,8 @@ BOOL PVXMLOutgoingChannel::Read(void * buffer, PINDEX amount)
 
 PWAVFile * PVXMLOutgoingChannel::CreateWAVFile(const PFilePath & fn)
 { 
-  PWAVFile * file = vxml.CreateWAVFile(fn, PFile::ReadOnly, PFile::ModeDefault, GetWavFileType()); 
+  PString nfn = AdjustFn(fn);
+  PWAVFile * file = vxml.CreateWAVFile(nfn, PFile::ReadOnly, PFile::ModeDefault, GetWavFileType()); 
   if (!IsWAVFileValid(*file)) {
     delete file;
     file = NULL;
@@ -1615,12 +1591,7 @@ void PVXMLOutgoingChannelPCM::CreateSilenceFrame(PINDEX amount)
 
 ///////////////////////////////////////////////////////////////
 
-PVXMLOutgoingChannelG7231::PVXMLOutgoingChannelG7231(PVXMLSession & vxml)
-  : PVXMLOutgoingChannel(vxml)
-{
-}
-
-void PVXMLOutgoingChannelG7231::QueueFile(const PString & ofn, PINDEX repeat, PINDEX delay)
+static PString AdjustFnForG7231(const PString & ofn)
 {
   PString fn = ofn;;
 
@@ -1636,7 +1607,17 @@ void PVXMLOutgoingChannelG7231::QueueFile(const PString & ofn, PINDEX repeat, PI
       basename += "_g7231";
     fn = basename + "." + ext;
   }
-  PVXMLOutgoingChannel::QueueFile(fn, repeat, delay);
+  return fn;
+}
+
+PVXMLOutgoingChannelG7231::PVXMLOutgoingChannelG7231(PVXMLSession & vxml)
+  : PVXMLOutgoingChannel(vxml)
+{
+}
+
+PString PVXMLOutgoingChannelG7231::AdjustFn(const PString & fn)
+{
+  return AdjustFnForG7231(fn);
 }
 
 BOOL PVXMLOutgoingChannelG7231::IsWAVFileValid(PWAVFile & chan) 
@@ -1706,9 +1687,8 @@ void PVXMLQueueURLItem::Play(PVXMLOutgoingChannel & outgoingChannel)
   // open the resource
   PHTTPClient * client = new PHTTPClient;
   PMIMEInfo outMIME, replyMIME;
-  PINDEX contentLength;
-  int code = client->GetDocument(url, contentLength);
-  if ((code != 200) || (contentLength == 0)) 
+  int code = client->GetDocument(url, outMIME, replyMIME, FALSE);
+  if ((code != 200) || (replyMIME(PHTTP::TransferEncodingTag) *= PHTTP::ChunkedTag))
     delete client;
   else {
     outgoingChannel.SetReadChannel(client, TRUE);
@@ -1786,7 +1766,8 @@ BOOL PVXMLIncomingChannel::EndRecording()
 
 PWAVFile * PVXMLIncomingChannel::CreateWAVFile(const PFilePath & fn)
 { 
-  return vxml.CreateWAVFile(fn, PFile::WriteOnly, PFile::ModeDefault, GetWavFileType()); 
+  PString nfn = AdjustFn(fn);
+  return vxml.CreateWAVFile(nfn, PFile::WriteOnly, PFile::ModeDefault, GetWavFileType()); 
 }
 
 ///////////////////////////////////////////////////////////////
@@ -1829,6 +1810,12 @@ void PVXMLIncomingChannelG7231::DelayFrame(PINDEX /*len*/)
   delay.Delay(G7231_FRAME_SIZE);
 }
 
+PString PVXMLIncomingChannelG7231::AdjustFn(const PString & fn)
+{
+  return AdjustFnForG7231(fn);
+}
+
+
 /////////////////////////////////////////////////////////////////////////////////////////
 
 PVXMLGrammar::PVXMLGrammar(PXMLElement * _field)
