@@ -27,6 +27,33 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: contain.cxx,v $
+ * Revision 1.127  2002/12/16 08:10:35  robertj
+ * Fixed infinite loop when converting an illegal (incomplete) UTF-8 string
+ *   to UCS-2, thanks Chih-Wei Huang
+ *
+ * Revision 1.126  2002/11/26 01:08:33  robertj
+ * Fixed problem when using pre-initialised PStringStream greater than 255
+ *   bytes, would truncate and lose trailing null. Reported by Thien Nguyen
+ *
+ * Revision 1.125  2002/11/12 09:18:03  robertj
+ * Added PString::NumCompare() as functional equivalent of strncmp().
+ * Added PSortedStringList::GetNextStringsIndex() to do searches of binary
+ *   tree on partal strings.
+ *
+ * Revision 1.124  2002/11/01 05:10:01  robertj
+ * Fixed bug in UTF-8 to UCS-2 conversion, not compiler portable!
+ *
+ * Revision 1.123  2002/10/31 07:33:59  robertj
+ * Changed UTF-8 to UCS-2 conversion function to not include trailing null.
+ *
+ * Revision 1.122  2002/10/31 05:55:55  robertj
+ * Now comprehensively stated that a PString is ALWAYS an 8 bit string as
+ *   there are far too many inheerent assumptions every to make it 16 bit.
+ * Added UTF-8/UCS-2 conversion functions to PString.
+ *
+ * Revision 1.121  2002/10/10 04:43:44  robertj
+ * VxWorks port, thanks Martijn Roest
+ *
  * Revision 1.120  2002/08/14 00:43:40  robertj
  * Added ability to have fixed maximum length PStringStream's so does not do
  *   unwanted malloc()'s while outputing data.
@@ -1045,26 +1072,11 @@ const PString & PString::Empty()
 }
 
 
-#ifdef PHAS_UNICODE
-#define PSTRING_COPY(d, s, l) UnicodeCopy((WORD *)(d), (s), (l))
-#define PSTRING_MOVE(d, doff, s, soff, l) \
-            memmove(((WORD*)(d))+(doff), ((WORD*)(s))+(soff), (l)*sizeof(WORD))
-static void UnicodeCopy(WORD * theArray, char * src, size_t len)
-{
-  while (len-- > 0)
-    *theArray++ = *src++;
-}
-#else
-#define PSTRING_COPY(d, s, l) memcpy((d), (s), (l))
-#define PSTRING_MOVE(d, doff, s, soff, l) memmove((d)+(doff), (s)+(soff), (l))
-#endif
-
-
 PString::PString(const char * cstr)
-  : PSTRING_ANCESTOR_CLASS(cstr != NULL ? strlen(cstr)+1 : 1)
+  : PCharArray(cstr != NULL ? strlen(cstr)+1 : 1)
 {
   if (cstr != NULL)
-    PSTRING_COPY(theArray, cstr, GetSize());
+    memcpy(theArray, cstr, GetSize());
 }
 
 
@@ -1073,43 +1085,32 @@ PString::PString(const WORD * ustr)
   if (ustr == NULL)
     SetSize(1);
   else {
-    const WORD * ptr = ustr;
     PINDEX len = 0;
-    while (*ptr++ != 0)
+    while (ustr[len] != 0)
       len++;
-    SetSize(len+1);
-#ifdef PHAS_UNICODE
-    memcpy(theArray, ustr, len*sizeof(WORD))
-#else
-    char * cstr = theArray;
-    while (len-- > 0)
-      *cstr++ = (char)*ustr++;
-#endif
+    InternalFromUCS2(ustr, len);
   }
 }
 
 
 PString::PString(const char * cstr, PINDEX len)
-  : PSTRING_ANCESTOR_CLASS(len+1)
+  : PCharArray(len+1)
 {
   if (len > 0)
-    PSTRING_COPY(theArray, PAssertNULL(cstr), len);
+    memcpy(theArray, PAssertNULL(cstr), len);
 }
 
 
 PString::PString(const WORD * ustr, PINDEX len)
-  : PSTRING_ANCESTOR_CLASS(len+1)
+  : PCharArray(len+1)
 {
-  if (len > 0) {
-    PAssertNULL(ustr);
-#ifdef PHAS_UNICODE
-    memcpy(theArray, ustr, len*sizeof(WORD))
-#else
-    char * cstr = theArray;
-    while (len-- > 0)
-      *cstr++ = (char)*ustr++;
-#endif
-  }
+  InternalFromUCS2(ustr, len);
+}
+
+
+PString::PString(const PWORDArray & ustr)
+{
+  InternalFromUCS2(ustr, ustr.GetSize());
 }
 
 
@@ -1171,7 +1172,7 @@ PString::PString(ConversionType type, const char * str, ...)
       if (*str != '\0') {
         PINDEX len = *str & 0xff;
         PAssert(SetSize(len+1), POutOfMemory);
-        PSTRING_COPY(theArray, str+1, len);
+        memcpy(theArray, str+1, len);
       }
       break;
 
@@ -1179,7 +1180,7 @@ PString::PString(ConversionType type, const char * str, ...)
       if (str[0] != '\0' && str[1] != '\0') {
         PINDEX len = str[0] | (str[1] << 8);
         PAssert(SetSize(len+1), POutOfMemory);
-        PSTRING_COPY(theArray, str+2, len);
+        memcpy(theArray, str+2, len);
       }
       break;
 
@@ -1455,11 +1456,7 @@ PINDEX PString::HashFunction() const
 
 BOOL PString::IsEmpty() const
 {
-#ifdef PHAS_UNICODE
-  return *(WORD*)theArray == '\0';
-#else
   return *theArray == '\0';
-#endif
 }
 
 
@@ -1506,17 +1503,6 @@ BOOL PString::MakeUnique()
 }
 
 
-#ifdef PHAS_UNICODE
-PINDEX PString::GetLength() const
-{
-  for (len = 0; len < GetSize(); len++)
-    if (((WORD *)theArray)[len] == 0)
-      break;
-  return len;
-}
-#endif
-
-
 PString PString::operator+(const char * cstr) const
 {
   if (cstr == NULL)
@@ -1526,8 +1512,8 @@ PString PString::operator+(const char * cstr) const
   PINDEX alen = strlen(cstr)+1;
   PString str;
   str.SetSize(olen+alen);
-  PSTRING_MOVE(str.theArray, 0, theArray, 0, olen);
-  PSTRING_COPY(str.theArray+olen, cstr, alen);
+  memmove(str.theArray, theArray, olen);
+  memcpy(str.theArray+olen, cstr, alen);
   return str;
 }
 
@@ -1537,7 +1523,7 @@ PString PString::operator+(char c) const
   PINDEX olen = GetLength();
   PString str;
   str.SetSize(olen+2);
-  PSTRING_MOVE(str.theArray, 0, theArray, 0, olen);
+  memmove(str.theArray, theArray, olen);
   str.theArray[olen] = c;
   return str;
 }
@@ -1551,7 +1537,7 @@ PString & PString::operator+=(const char * cstr)
   PINDEX olen = GetLength();
   PINDEX alen = strlen(cstr)+1;
   SetSize(olen+alen);
-  PSTRING_COPY(theArray+olen, cstr, alen);
+  memcpy(theArray+olen, cstr, alen);
   return *this;
 }
 
@@ -1578,10 +1564,10 @@ PString PString::operator&(const char * cstr) const
   PString str;
   PINDEX space = olen > 0 && theArray[olen-1]!=' ' && *cstr!=' ' ? 1 : 0;
   str.SetSize(olen+alen+space);
-  PSTRING_MOVE(str.theArray, 0, theArray, 0, olen);
+  memmove(str.theArray, theArray, olen);
   if (space != 0)
     str.theArray[olen] = ' ';
-  PSTRING_COPY(str.theArray+olen+space, cstr, alen);
+  memcpy(str.theArray+olen+space, cstr, alen);
   return str;
 }
 
@@ -1592,7 +1578,7 @@ PString PString::operator&(char c) const
   PString str;
   PINDEX space = olen > 0 && theArray[olen-1] != ' ' && c != ' ' ? 1 : 0;
   str.SetSize(olen+2+space);
-  PSTRING_MOVE(str.theArray, 0, theArray, 0, olen);
+  memmove(str.theArray, theArray, olen);
   if (space != 0)
     str.theArray[olen] = ' ';
   str.theArray[olen+space] = c;
@@ -1613,7 +1599,7 @@ PString & PString::operator&=(const char * cstr)
   SetSize(olen+alen+space);
   if (space != 0)
     theArray[olen] = ' ';
-  PSTRING_COPY(theArray+olen+space, cstr, alen);
+  memcpy(theArray+olen+space, cstr, alen);
   return *this;
 }
 
@@ -1641,7 +1627,7 @@ void PString::Delete(PINDEX start, PINDEX len)
   if (len > slen - start)
     SetAt(start, '\0');
   else
-    PSTRING_MOVE(theArray, start, theArray, start+len, slen-start-len+1);
+    memmove(theArray+start, theArray+start+len, slen-start-len+1);
   MakeMinimumSize();
 }
 
@@ -1719,6 +1705,24 @@ BOOL PString::operator*=(const char * cstr) const
 }
 
 
+PObject::Comparison PString::NumCompare(const PString & str, PINDEX count, PINDEX offset) const
+{
+  PINDEX len = str.GetLength();
+  if (count > len)
+    count = len;
+  return InternalCompare(offset, count, str);
+}
+
+
+PObject::Comparison PString::NumCompare(const char * cstr, PINDEX count, PINDEX offset) const
+{
+  PINDEX len = ::strlen(cstr);
+  if (count > len)
+    count = len;
+  return InternalCompare(offset, count, cstr);
+}
+
+
 PObject::Comparison PString::InternalCompare(PINDEX offset, char c) const
 {
   char ch = theArray[offset];
@@ -1941,9 +1945,9 @@ void PString::Splice(const char * cstr, PINDEX pos, PINDEX len)
     if (clen > len)
       SetSize(newlen+1);
     if (pos+len < slen)
-      PSTRING_MOVE(theArray, pos+clen, theArray, pos+len, slen-pos-len+1);
+      memmove(theArray+pos+clen, theArray+pos+len, slen-pos-len+1);
     if (clen > 0)
-      PSTRING_COPY(theArray+pos, cstr, clen);
+      memcpy(theArray+pos, cstr, clen);
     theArray[newlen] = '\0';
   }
 }
@@ -2046,12 +2050,9 @@ PString PString::RightTrim() const
     rpos--;
   }
 
-#if defined(P_MACOSX) || defined(P_MACOS)	// make Apple gnu compiler happy
+  // make Apple & Tornado gnu compiler happy
   PString retval(theArray, rpos - theArray);
   return retval;
-#else
-  return PString(theArray, rpos - theArray);
-#endif
 }
 
 
@@ -2122,21 +2123,123 @@ double PString::AsReal() const
 }
 
 
+PWORDArray PString::AsUCS2() const
+{
+#ifdef P_HAS_G_CONVERT
+
+  gsize g_len = 0;
+  gchar * g_ucs2 = g_convert(theArray, GetSize()-1, "UCS-2", "UTF-8", 0, &g_len, 0);
+  if (g_ucs2 == NULL)
+    return PWORDArray();
+
+  PWORDArray ucs2((const WORD *)g_ucs2, (PINDEX)g_len);
+  g_free(g_ucs2)
+  return ucs2;
+
+#else
+
+  PWORDArray ucs2(GetSize()); // Always bigger than required
+
+  PINDEX count = 0;
+  PINDEX i = 0;
+  PINDEX length = GetSize()-1;
+  while (i < length) {
+    int c = theArray[i];
+    if ((c&0x80) == 0)
+      ucs2[count++] = (BYTE)theArray[i++];
+    else if ((c&0xe0) == 0xc0) {
+      if (i < length-1)
+        ucs2[count++] = (WORD)(((theArray[i  ]&0x1f)<<6)|
+                                (theArray[i+1]&0x3f));
+      i += 2;
+    }
+    else if ((c&0xf0) == 0xe0) {
+      if (i < length-2)
+        ucs2[count++] = (WORD)(((theArray[i  ]&0x0f)<<12)|
+                               ((theArray[i+1]&0x3f)<< 6)|
+                                (theArray[i+2]&0x3f));
+      i += 3;
+    }
+    else {
+      if ((c&0xf8) == 0xf0)
+        i += 4;
+      else if ((c&0xfc) == 0xf8)
+        i += 5;
+      else
+        i += 6;
+      if (i <= length)
+        ucs2[count++] = 0xffff;
+    }
+  }
+
+  ucs2.SetSize(count);
+  return ucs2;
+
+#endif
+}
+
+
+void PString::InternalFromUCS2(const WORD * ptr, PINDEX len)
+{
+  if (ptr == NULL || len == 0) {
+    *this = Empty();
+    return;
+  }
+
+#ifdef P_HAS_G_CONVERT
+
+  gsize g_len = 0;
+  gchar * g_utf8 = g_convert(ptr, len, "UTF-8", "UCS-2", 0, &g_len, 0);
+  if (g_utf8 == NULL) {
+    *this = Empty();
+    return;
+  }
+
+  SetSize(&g_len);
+  memcpy(theArray, g_char, g_len);
+  g_free(g_utf8);
+
+#else
+
+  PINDEX i;
+  PINDEX count = 1;
+  for (i = 0; i < len; i++) {
+    if (ptr[i] < 0x80)
+      count++;
+    else if (ptr[i] < 0x800)
+      count += 2;
+    else
+      count += 3;
+  }
+  SetSize(count);
+
+  count = 0;
+  for (i = 0; i < len; i++) {
+    unsigned v = *ptr++;
+    if (v < 0x80)
+      theArray[count++] = (char)v;
+    else if (v < 0x800) {
+      theArray[count++] = (char)(0xc0+(v>>6));
+      theArray[count++] = (char)(0x80+(v&0x3f));
+    }
+    else {
+      theArray[count++] = (char)(0xd0+(v>>12));
+      theArray[count++] = (char)(0x80+((v>>6)&0x3f));
+      theArray[count++] = (char)(0x80+(v&0x3f));
+    }
+  }
+
+#endif
+}
+
+
 PBYTEArray PString::ToPascal() const
 {
   PINDEX len = GetLength();
   PAssert(len < 256, "Cannot convert to PASCAL string");
   BYTE buf[256];
   buf[0] = (BYTE)len;
-#ifdef PHAS_UNICODE
-  WORD * ptr = (WORD *)theArray;
-  while (len > 0) {
-    buf[len] = (BYTE)(*ptr < 256 ? *ptr : 255);
-    len--;
-  }
-#else
   memcpy(&buf[1], theArray, len);
-#endif
   return PBYTEArray(buf, len+1);
 }
 
@@ -2179,12 +2282,20 @@ PString & PString::sprintf(const char * fmt, ...)
 
 PString & PString::vsprintf(const char * fmt, va_list arg)
 {
+#ifdef P_TORNADO
+  // The library provided with tornado 2.0 does not have the implementation
+  // for vsnprintf
+  // as workaround, just use a array size of 2000
+  PAssert(SetSize(2000), POutOfMemory);
+  ::vsprintf(theArray, fmt, arg);
+#else
   PINDEX len = theArray != NULL ? GetLength() : 0;
   PINDEX size = 0;
   do {
     size += 1000;
     PAssert(SetSize(size), POutOfMemory);
   } while (_vsnprintf(theArray+len, size-len, fmt, arg) == -1);
+#endif // P_TORNADO
 
   PAssert(MakeMinimumSize(), POutOfMemory);
   return *this;
@@ -2249,7 +2360,7 @@ PStringStream::Buffer::Buffer(PStringStream & str, PINDEX size)
   : string(str),
     fixedBufferSize(size != 0)
 {
-  string.SetSize(size != 0 ? size : 256);
+  string.SetMinSize(size != 0 ? size : 256);
   sync();
 }
 
@@ -2526,6 +2637,51 @@ PSortedStringList::PSortedStringList(const PStringList & list)
 }
 
 
+
+PINDEX PSortedStringList::GetNextStringsIndex(const PString & str) const
+{
+  PINDEX len = str.GetLength();
+
+  info->lastIndex = InternalStringSelect(str, len, info->root, info->lastElement);
+
+  if (info->lastIndex != 0) {
+    Element * prev;
+    while ((prev = info->lastElement->Predecessor()) != &Element::nil &&
+                    ((PString *)prev->data)->NumCompare(str, len) >= EqualTo) {
+      info->lastElement = prev;
+      info->lastIndex--;
+    }
+  }
+
+  return info->lastIndex;
+}
+
+
+PINDEX PSortedStringList::InternalStringSelect(const char * str,
+                                               PINDEX len,
+                                               Element * thisElement,
+                                               Element * & lastElement)
+{
+  if (thisElement == &Element::nil)
+    return 0;
+
+  switch (((PString *)thisElement->data)->NumCompare(str, len)) {
+    case PObject::LessThan :
+    {
+      PINDEX index = InternalStringSelect(str, len, thisElement->right, lastElement);
+      return thisElement->left->subTreeSize + index + 1;
+    }
+
+    case PObject::GreaterThan :
+      return InternalStringSelect(str, len, thisElement->left, lastElement);
+
+    default :
+      lastElement = thisElement;
+      return thisElement->left->subTreeSize;
+  }
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 PStringSet::PStringSet(PINDEX count, char const * const * strarr, BOOL caseless)
