@@ -27,6 +27,16 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: object.cxx,v $
+ * Revision 1.63  2002/11/22 10:14:58  robertj
+ * Fixed correct free of memory blocks in exceptional circumstances. Partially
+ *   trashed heap or objects etc.
+ *
+ * Revision 1.62  2002/10/21 12:52:27  rogerh
+ * Add throw()s to new and delete. Error reported by FreeBSD 5.0 and GCC 3.2.1
+ *
+ * Revision 1.61  2002/10/10 04:43:44  robertj
+ * VxWorks port, thanks Martijn Roest
+ *
  * Revision 1.60  2002/09/06 05:29:42  craigs
  * Reversed order of memory block check on delete to improve performance in
  * Linux debug mode
@@ -308,25 +318,41 @@ void PAssertFunc(const char * file, int line, const char * className, const char
 #undef free
 
 
+#if __GNUC__ >= 3
+void * operator new(size_t nSize) throw (std::bad_alloc)
+#else
 void * operator new(size_t nSize)
+#endif
 {
   return PMemoryHeap::Allocate(nSize, (const char *)NULL, 0, NULL);
 }
 
 
+#if __GNUC__ >= 3
+void * operator new[](size_t nSize) throw (std::bad_alloc)
+#else
 void * operator new[](size_t nSize)
+#endif
 {
   return PMemoryHeap::Allocate(nSize, (const char *)NULL, 0, NULL);
 }
 
 
+#if __GNUC__ >= 3
+void operator delete(void * ptr) throw()
+#else
 void operator delete(void * ptr)
+#endif
 {
   PMemoryHeap::Deallocate(ptr, NULL);
 }
 
 
+#if __GNUC__ >= 3
+void operator delete[](void * ptr) throw()
+#else
 void operator delete[](void * ptr)
+#endif
 {
   PMemoryHeap::Deallocate(ptr, NULL);
 }
@@ -577,18 +603,24 @@ void PMemoryHeap::Deallocate(void * ptr, const char * className)
     return;
 
   Wrapper mem;
+  Header * obj = ((Header *)ptr)-1;
 
   if (mem->isDestroyed) {
-    free(ptr);
+    free(obj);
     return;
   }
 
-  if (mem->InternalValidate(ptr, className, mem->leakDumpStream) != Ok) {
-    free(ptr);
-    return;
+  switch (mem->InternalValidate(ptr, className, mem->leakDumpStream)) {
+    case Ok :
+      break;
+    case Trashed :
+      free(ptr);
+      return;
+    case Bad :
+      free(obj);
+      return;
   }
 
-  Header * obj = ((Header *)ptr)-1;
   if (obj->prev != NULL)
     obj->prev->next = obj->next;
   else
@@ -814,16 +846,28 @@ void PMemoryHeap::InternalDumpObjectsSince(DWORD objectNumber, ostream & strm)
 
 #else // PMEMORY_CHECK
 
+#ifndef P_VXWORKS
+
+#if __GNUC__ >= 3
+void * operator new[](size_t nSize) throw (std::bad_alloc)
+#else
 void * operator new[](size_t nSize)
+#endif
 {
   return malloc(nSize);
 }
 
+#if __GNUC__ >= 3
+void operator delete[](void * ptr) throw ()
+#else
 void operator delete[](void * ptr)
+#endif
 {
   free(ptr);
 }
 
+#endif // !P_VXWORKS
+
 #endif // PMEMORY_CHECK
 
 
@@ -1507,4 +1551,34 @@ istream & operator>>(istream & stream, PUInt64 & v)
 #endif
 
 
+#ifdef P_TORNADO
+
+// the library provided with Tornado 2.0 does not contain implementation 
+// for the functions defined below, therefor the own implementation
+
+ostream & ostream::operator<<(PInt64 v)
+{
+  return *this << (long)(v >> 32) << (long)(v & 0xFFFFFFFF);
+}
+
+
+ostream & ostream::operator<<(PUInt64 v)
+{
+  return *this << (long)(v >> 32) << (long)(v & 0xFFFFFFFF);
+}
+
+istream & istream::operator>>(PInt64 & v)
+{
+  return *this >> (long)(v >> 32) >> (long)(v & 0xFFFFFFFF);
+}
+
+
+istream & istream::operator>>(PUInt64 & v)
+{
+  return *this >> (long)(v >> 32) >> (long)(v & 0xFFFFFFFF);
+}
+
+#endif // P_TORNADO
+
+
 // End Of File ///////////////////////////////////////////////////////////////
