@@ -27,6 +27,22 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: osutils.cxx,v $
+ * Revision 1.202  2002/12/18 05:31:31  robertj
+ * Moved PTimeInterval::GetInterval() to common code.
+ *
+ * Revision 1.201  2002/12/11 03:23:27  robertj
+ * Fixed deadlock in read/write mutex, at price of not having seemless upgrading
+ *   of read lock to write lock. There is now a window in which some other
+ *   thread may gain write lock from the thread that was trying to upgrade.
+ *
+ * Revision 1.200  2002/12/10 02:39:07  robertj
+ * Avoid odd trace output to stderr before trace file is set.
+ *
+ * Revision 1.199  2002/10/09 00:46:19  robertj
+ * Changed PThread::Create() so does not return PThread pointer if the thread
+ *   has been created auto-delete, the pointer is extremely dangerous to use
+ *   as it could be deleted at any time, so to remove temptation ...
+ *
  * Revision 1.198  2002/10/08 03:35:15  robertj
  * Fixed BSD warning
  *
@@ -740,14 +756,11 @@ void PTrace::SetStream(ostream * s)
 
 void PTrace::Initialise(unsigned level, const char * filename, unsigned options)
 {
+  // If we have a tracing version, then open trace file and set modes
 #if PTRACING
   PProcess & process = PProcess::Current();
 #endif
 
-  // If we have a tracing version, then open trace file and set modes
-  PTraceOptions = options;
-  PTraceLevelThreshold = level;
-
 #if PMEMORY_CHECK
   int ignoreAllocations = -1;
 #endif
@@ -771,6 +784,9 @@ void PTrace::Initialise(unsigned level, const char * filename, unsigned options)
     }
   }
 
+  PTraceOptions = options;
+  PTraceLevelThreshold = level;
+
   PTRACE(1, process.GetName()
          << "\tVersion " << process.GetVersion(TRUE)
          << " by " << process.GetManufacturer()
@@ -962,6 +978,21 @@ void PDirectory::CloneContents(const PDirectory * d)
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+// PTimeInterval
+
+DWORD PTimeInterval::GetInterval() const
+{
+  if (milliseconds <= 0)
+    return 0;
+
+  if (milliseconds >= UINT_MAX)
+    return UINT_MAX;
+
+  return (DWORD)milliseconds;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // PTimer
 
@@ -1848,7 +1879,19 @@ PThread * PThread::Create(const PNotifier & notifier,
                           const PString & threadName,
                           PINDEX stackSize)
 {
-  return new PSimpleThread(notifier, parameter, deletion, priorityLevel, threadName, stackSize);
+  PThread * thread = new PSimpleThread(notifier,
+                                       parameter,
+                                       deletion,
+                                       priorityLevel,
+                                       threadName,
+                                       stackSize);
+  if (deletion != AutoDeleteThread)
+    return thread;
+
+  // Do not return a pointer to the thread if it is auto-delete as this
+  // pointer is extremely dangerous to use, it could be deleted at any moment
+  // from now on so using the pointer could crash the program.
+  return NULL;
 }
 
 
@@ -2589,18 +2632,17 @@ void PReadWriteMutex::StartWrite()
   if (nest.writerCount > 1)
     return;
 
-  // Make sure multiple threads wanting write access can't do the following
-  // logic about unlocking readers in the same thread and wating for readers
-  // in other threads to stop reading.
-  writerMutex.Wait();
-
   // If have a read lock already in this thread then do the "real" unlock code
   // but do not change the lock count, calls to EndRead() will now just
   // decrement the count instead of doing the unlock (its already done!)
   if (nest.readerCount > 0)
     InternalEndRead();
 
-  // Do do the rest of the text book write lock
+  // Note in this gap another thread could grab the write lock, thus
+
+  // Now do the text book write lock
+  writerMutex.Wait();
+
   writerCount++;
   if (writerCount == 1)
     readerSemaphore.Wait();
@@ -2641,17 +2683,16 @@ void PReadWriteMutex::EndWrite()
   if (writerCount == 0)
     readerSemaphore.Signal();
 
-  // Before we release the writerMutex so another thread can grab the object in
-  // write only mode, check to see if there are any read lock present, if so then
-  // reacquire the read lock (not changing the count) otherwise clean up the
+  writerMutex.Signal();
+  // End of text book write unlock
+
+  // Now check to see if there was a read lock present for this thread, if so
+  // then reacquire the read lock (not changing the count) otherwise clean up the
   // memory for the nested thread info structure
   if (nest->readerCount > 0)
     InternalStartRead();
   else
     EndNest();
-
-  // Complete text book unlock
-  writerMutex.Signal();
 }
 
 
