@@ -27,6 +27,85 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: sockets.cxx,v $
+ * Revision 1.145  2002/12/16 08:04:46  robertj
+ * Fixed correct error check for gethostbyname_r, thanks Vladimir Toncar
+ *
+ * Revision 1.144  2002/12/13 04:01:46  robertj
+ * Initialised error code in gethostbyname usage.
+ *
+ * Revision 1.143  2002/12/04 00:52:59  robertj
+ * Fixed GNU warning
+ *
+ * Revision 1.142  2002/12/02 12:25:08  craigs
+ * Fixed problem with error code from gethostbyname_r not being checked correctly
+ *
+ * Revision 1.141  2002/11/24 23:47:01  robertj
+ * Fixed MSVC v5 compatibility
+ *
+ * Revision 1.140  2002/11/13 02:15:25  craigs
+ * Fixed problem with GetLocalHostName
+ *
+ * Revision 1.139  2002/11/12 11:47:53  rogerh
+ * Add a missing memset in the Psockaddr constructor
+ *
+ * Revision 1.138  2002/11/02 00:32:21  robertj
+ * Further fixes to VxWorks (Tornado) port, thanks Andreas Sikkema.
+ *
+ * Revision 1.137  2002/11/01 23:56:11  robertj
+ * Fixed GNu compatibility isse with IPv6
+ *
+ * Revision 1.136  2002/11/01 03:32:08  robertj
+ * More IPv6 fixes, thanks Sbastien Josset.
+ *
+ * Revision 1.135  2002/10/31 07:55:33  robertj
+ * Put sizeof ipv6 structure back as magic number 28 is explained by
+ *   mismatched header file and running implementation.
+ *
+ * Revision 1.134  2002/10/29 08:04:44  robertj
+ * Changed in_addr6 to more universally used in6_addr.
+ * Changed size of ipv6 address to be 28 under win32, Why? I don't know!
+ *
+ * Revision 1.133  2002/10/19 06:12:20  robertj
+ * Moved P_fd_set::Zero() from platform independent to platform dependent
+ *   code as Win32 implementation is completely different from Unix.
+ *
+ * Revision 1.132  2002/10/18 08:07:41  robertj
+ * Fixed use of FD_ZERO as (strangely) crashes on some paltforms and would
+ *   not have cleared enough of an enlarges fd_set anyway.
+ *
+ * Revision 1.131  2002/10/17 13:44:27  robertj
+ * Port to RTEMS, thanks Vladimir Nesic.
+ *
+ * Revision 1.130  2002/10/17 08:17:28  robertj
+ * Fixed incomplete changes for expandable fd_set
+ *
+ * Revision 1.129  2002/10/17 07:17:43  robertj
+ * Added ability to increase maximum file handles on a process.
+ *
+ * Revision 1.128  2002/10/17 01:24:11  robertj
+ * Fixed so internal sockaddr classes GetSize() returns correct size for
+ *   particular sockaddr it represents, thanks Sbastien Josset.
+ *
+ * Revision 1.127  2002/10/16 06:19:36  robertj
+ * Rewrite of IPv6 sockaddr code to use intelligent class to automatically
+ *   know if it is sockaddr_in or aockaddr_in6.
+ * Fixed Connect() function to work correctly on unopened socket.
+ *
+ * Revision 1.126  2002/10/10 11:38:56  robertj
+ * Added close of socket if not open in correct ip version, thanks Sbastien Josset
+ *
+ * Revision 1.125  2002/10/10 04:43:44  robertj
+ * VxWorks port, thanks Martijn Roest
+ *
+ * Revision 1.124  2002/10/09 05:37:52  robertj
+ * Fixed IPv6 version of ReadFrom() and WriteTo().
+ *
+ * Revision 1.123  2002/10/08 23:31:44  robertj
+ * Added missing GetSize() implementation in ip address.
+ *
+ * Revision 1.122  2002/10/08 12:41:52  robertj
+ * Changed for IPv6 support, thanks Sbastien Josset.
+ *
  * Revision 1.121  2002/09/23 07:17:24  robertj
  * Changes to allow winsock2 to be included.
  *
@@ -409,11 +488,96 @@
 #include <ConfigurationClass.h>
 #endif
 
+
 #include <ptlib.h>
+
 #include <ptlib/sockets.h>
 
 #include <ctype.h>
 
+
+
+#if P_HAS_IPV6
+
+class Psockaddr
+{
+  public:
+    Psockaddr() { memset(&storage, 0, sizeof(storage)); }
+    Psockaddr(const PIPSocket::Address & ip, WORD port);
+    sockaddr* operator->() const { return (sockaddr *)&storage; }
+    operator sockaddr*()   const { return (sockaddr *)&storage; }
+    socklen_t GetSize() const;
+    PIPSocket::Address GetIP() const;
+    WORD GetPort() const;
+  private:
+    sockaddr_storage storage;
+};
+
+
+Psockaddr::Psockaddr(const PIPSocket::Address & ip, WORD port)
+{
+  memset(&storage, 0, sizeof(storage));
+
+  if (ip.GetVersion() == 6) {
+    sockaddr_in6 * addr6 = (sockaddr_in6 *)&storage;
+    addr6->sin6_family = AF_INET6;
+    addr6->sin6_addr = ip;
+    addr6->sin6_port = htons(port);
+    addr6->sin6_flowinfo = 0;
+    addr6->sin6_scope_id = 0; // Should be set to the right interface....
+  }
+  else {
+    sockaddr_in * addr4 = (sockaddr_in *)&storage;
+    addr4->sin_family = AF_INET;
+    addr4->sin_addr = ip;
+    addr4->sin_port = htons(port);
+  }
+}
+
+
+socklen_t Psockaddr::GetSize() const
+{
+  switch (((sockaddr *)&storage)->sa_family) {
+    case AF_INET :
+      return sizeof(sockaddr_in);
+    case AF_INET6 :
+      // RFC 2133 (Old IPv6 spec) size is 24
+      // RFC 2553 (New IPv6 spec) size is 28
+      return sizeof(sockaddr_in6);
+    default :
+      return sizeof(storage);
+  }
+}
+
+
+PIPSocket::Address Psockaddr::GetIP() const
+{
+  switch (((sockaddr *)&storage)->sa_family) {
+    case AF_INET :
+      return ((sockaddr_in *)&storage)->sin_addr;
+    case AF_INET6 :
+      return ((sockaddr_in6 *)&storage)->sin6_addr;
+    default :
+      return 0;
+  }
+}
+
+
+WORD Psockaddr::GetPort() const
+{
+  switch (((sockaddr *)&storage)->sa_family) {
+    case AF_INET :
+      return ntohs(((sockaddr_in *)&storage)->sin_port);
+    case AF_INET6 :
+      return ntohs(((sockaddr_in6 *)&storage)->sin6_port);
+    default :
+      return 0;
+  }
+}
+
+#endif
+
+
 #if (defined(_WIN32) || defined(WINDOWS)) && !defined(__NUCLEUS_MNT__)
 static PWinSock dummyForWinSock; // Assure winsock is initialised
 #endif
@@ -471,7 +635,7 @@ class PIPCacheKey : public PObject
       { return new PIPCacheKey(*this); }
 
     PINDEX HashFunction() const
-      { return (addr.Byte2() + addr.Byte3() + addr.Byte4())%41; }
+      { return (addr[1] + addr[2] + addr[3])%41; }
 
   private:
     PIPSocket::Address addr;
@@ -506,21 +670,16 @@ static PHostByAddr & pHostByAddr()
 PIPCacheData::PIPCacheData(struct hostent * host_info, const char * original)
 {
   if (host_info == NULL) {
-    address.s_addr = 0;
+    address = 0;
     return;
   }
 
   hostname = host_info->h_name;
+  if (host_info->h_addr != NULL)
 #ifndef _WIN32_WCE
-  address = *(DWORD *)host_info->h_addr;
+    address = *(DWORD *)host_info->h_addr;
 #else
-  if( host_info->h_addr )
-	  address = PIPSocket::Address(
-		  (BYTE) host_info->h_addr[0],
-		  (BYTE) host_info->h_addr[1],
-		  (BYTE) host_info->h_addr[2],
-		  (BYTE) host_info->h_addr[3]);
-
+    address = PIPSocket::Address(h_length, host_info->h_addr);
 #endif
   aliases.AppendString(host_info->h_name);
 
@@ -545,6 +704,7 @@ static PTimeInterval GetConfigTime(const char * key, DWORD dflt)
   return cfg.GetInteger(key, dflt);
 }
 
+
 BOOL PIPCacheData::HasAged() const
 {
   static PTimeInterval retirement = GetConfigTime("Age Limit", 300000); // 5 minutes
@@ -615,13 +775,17 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
 
 #ifdef P_AIX
     struct hostent_data ht_data;
-    memset(&ht_data, 0, sizeof(ht_data)); 
+    memset(&ht_data, 0, sizeof(ht_data));
+    struct hostent host_info;
+#elif defined(P_RTEMS)
     struct hostent host_info;
 #else
     struct hostent * host_info;
 #endif
 
     int retry = 3;
+    int localErrNo = NETDB_SUCCESS;
+
     do {
 #if ( ( defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB) ) || (defined(__NUCLEUS_PLUS__) ) )
       // this function should really be a static on PIPSocket, but this would
@@ -629,41 +793,49 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
       // of a pain!
 
 
-#ifndef P_AIX	// that I get no warnings
-      int localErrNo;
+#if !defined(P_AIX) && !defined(P_RTEMS)	// that I get no warnings
       char buffer[REENTRANT_BUFFER_LEN];
       struct hostent hostEnt;
 #endif
 
 #ifdef P_LINUX
-      ::gethostbyname_r(name,
-                        &hostEnt,
-                        buffer, REENTRANT_BUFFER_LEN,
-                        &host_info,
-      		        &localErrNo);
-      		      		        
+      if (::gethostbyname_r(name,
+                            &hostEnt,
+                            buffer, REENTRANT_BUFFER_LEN,
+                            &host_info,
+      		            &localErrNo) == 0)
+        localErrNo = NETDB_SUCCESS;
       		        
 #elif defined P_AIX
       ::gethostbyname_r(name,
                         &host_info,
-                        &ht_data);		    
+                        &ht_data);
+      localErrNo = h_errno;
+
+#elif defined P_RTEMS
+      host_info = *::gethostbyname(name);
 #else
       host_info = ::gethostbyname_r(name,
 			 &hostEnt, buffer, REENTRANT_BUFFER_LEN,
 			 &localErrNo);
 #endif
 
+#elif defined P_VXWORKS
+      struct hostent hostEnt;
+      host_info = Vx_gethostbyname((char *)name, &hostEnt);
+      localErrNo = h_errno;
 #else
       host_info = ::gethostbyname(name);
+      localErrNo = h_errno;
 #endif
-    } while (h_errno == TRY_AGAIN && --retry > 0);
+    } while (localErrNo == TRY_AGAIN && --retry > 0);
 
     mutex.Wait();
 
-    if (retry == 0)
+    if (localErrNo != NETDB_SUCCESS || retry == 0)
       return NULL;
 
-#ifdef P_AIX
+#if defined(P_AIX) || defined(P_RTEMS)
     host = new PIPCacheData (&host_info, (const char*) name);
 #else
     host = new PIPCacheData(host_info, name);
@@ -738,18 +910,21 @@ PIPCacheData * PHostByAddr::GetHost(const PIPSocket::Address & addr)
 #ifdef P_AIX
     struct hostent_data ht_data;
     struct hostent host_info;
-#else    
+#elif P_RTEMS
+    struct hostent host_info;
+#else
     struct hostent * host_info;
 #endif
 
     int retry = 3;
+    int localErrNo = NETDB_SUCCESS;
     do {
 #if ( ( defined(P_PTHREADS) && !defined(P_THREAD_SAFE_CLIB) ) || ( defined(__NUCLEUS_PLUS__) ) )
       // this function should really be a static on PIPSocket, but this would
       // require allocating thread-local storage for the data and that's too much
       // of a pain!
       
-#ifndef P_AIX	// that I get no warnings
+#if !defined(P_AIX) && !defined(P_RTEMS)	// that I get no warnings
       int localErrNo;
       char buffer[REENTRANT_BUFFER_LEN];
       struct hostent hostEnt;
@@ -762,35 +937,43 @@ PIPCacheData * PHostByAddr::GetHost(const PIPSocket::Address & addr)
                         buffer, REENTRANT_BUFFER_LEN,
                         &host_info,
                         &localErrNo);
-#elif P_AIX
+#elif defined P_AIX
       ::gethostbyaddr_r((char *)&addr, sizeof(addr),
                         PF_INET, 
                         &host_info,
                         &ht_data );
+      localErrNo = h_errno;
+#elif defined P_RTEMS
+      host_info = *::gethostbyaddr((const char *)&addr, sizeof(addr), PF_INET);
+      localErrNo = h_errno;
 #else
       host_info = ::gethostbyaddr_r((const char *)&addr, sizeof(addr), PF_INET, 
                                     &hostEnt, buffer, REENTRANT_BUFFER_LEN, &localErrNo);
 #endif
 
+#elif defined P_VXWORKS
+      struct hostent hostEnt;
+      host_info = Vx_gethostbyaddr((char *)&addr, &hostEnt);
 #else
       host_info = ::gethostbyaddr((const char *)&addr, sizeof(addr), PF_INET);
+      localErrNo = h_errno;
 #if defined(_WIN32) || defined(WINDOWS)  // Kludge to avoid strange 95 bug
       extern P_IsOldWin95();
       if (P_IsOldWin95() && host_info != NULL && host_info->h_addr_list[0] != NULL)
         host_info->h_addr_list[1] = NULL;
 #endif
 #endif
-    } while (h_errno == TRY_AGAIN && --retry > 0);
+    } while (localErrNo == TRY_AGAIN && --retry > 0);
 
     mutex.Wait();
 
-    if (retry == 0)
-      return FALSE;
+    if (localErrNo != NETDB_SUCCESS || retry == 0)
+      return NULL;
 
-#ifdef P_AIX
-    host = new PIPCacheData(&host_info, inet_ntoa(addr));
+#if defined(P_AIX) || defined(P_RTEMS)
+    host = new PIPCacheData(&host_info, addr.AsString());
 #else
-    host = new PIPCacheData(host_info, inet_ntoa(addr));
+    host = new PIPCacheData(host_info, addr.AsString());
 #endif
 
     SetAt(key, host);
@@ -803,6 +986,78 @@ PIPCacheData * PHostByAddr::GetHost(const PIPSocket::Address & addr)
 }
 
 
+//////////////////////////////////////////////////////////////////////////////
+// P_fd_set
+
+#ifdef _MSC_VER
+#pragma warning(push)
+#pragma warning(disable:4127)
+#endif
+
+P_fd_set::P_fd_set()
+{
+  Construct();
+  Zero();
+}
+
+
+P_fd_set::P_fd_set(SOCKET fd)
+{
+  Construct();
+  Zero();
+  FD_SET(fd, set);
+}
+
+
+P_fd_set & P_fd_set::operator=(SOCKET fd)
+{
+  PAssert(fd < max_fd, PInvalidParameter);
+  Zero();
+  FD_SET(fd, set);
+  return *this;
+}
+
+
+P_fd_set & P_fd_set::operator+=(SOCKET fd)
+{
+  PAssert(fd < max_fd, PInvalidParameter);
+  FD_SET(fd, set);
+  return *this;
+}
+
+
+P_fd_set & P_fd_set::operator-=(SOCKET fd)
+{
+  PAssert(fd < max_fd, PInvalidParameter);
+  FD_CLR(fd, set);
+  return *this;
+}
+
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
+
+
+//////////////////////////////////////////////////////////////////////////////
+// P_timeval
+
+P_timeval::P_timeval()
+{
+  tval.tv_usec = 0;
+  tval.tv_sec = 0;
+  infinite = FALSE;
+}
+
+
+P_timeval & P_timeval::operator=(const PTimeInterval & time)
+{
+  infinite = time == PMaxTimeInterval;
+  tval.tv_usec = (long)(time.GetMilliSeconds()%1000)*1000;
+  tval.tv_sec = time.GetSeconds();
+  return *this;
+}
+
+
 //////////////////////////////////////////////////////////////////////////////
 // PSocket
 
@@ -882,7 +1137,7 @@ BOOL PSocket::Shutdown(ShutdownValue value)
 
 WORD PSocket::GetProtocolByName(const PString & name)
 {
-#if !defined(BE_BONELESS) && !defined(__NUCLEUS_PLUS__) && !defined(_WIN32_WCE)
+#if !defined(BE_BONELESS) && !defined(__NUCLEUS_PLUS__) && !defined(_WIN32_WCE) && !defined(P_VXWORKS)
   struct protoent * ent = getprotobyname(name);
   if (ent != NULL)
     return ent->p_proto;
@@ -894,7 +1149,7 @@ WORD PSocket::GetProtocolByName(const PString & name)
 
 PString PSocket::GetNameByProtocol(WORD proto)
 {
-#if !defined(BE_BONELESS) && !defined(__NUCLEUS_PLUS__) && !defined(_WIN32_WCE)
+#if !defined(BE_BONELESS) && !defined(__NUCLEUS_PLUS__) && !defined(_WIN32_WCE) && !defined(P_VXWORKS)
   struct protoent * ent = getprotobynumber(proto);
   if (ent != NULL)
     return ent->p_name;
@@ -924,6 +1179,9 @@ WORD PSocket::GetPortByService(const char * protocol, const PString & service)
 #elif defined(_WIN32_WCE)
   PAssertAlways("PSocket::GetPortByService: problem for WindowsCE as no port given.");
   return 0;
+#elif defined(P_VXWORKS)
+  PAssertAlways("PSocket::GetPortByService: problem as no ::getservbyname in VxWorks");
+  return 0;
 #else
   PINDEX space = service.FindOneOf(" \t\r\n");
   struct servent * serv = ::getservbyname(service(0, space-1), protocol);
@@ -954,7 +1212,7 @@ PString PSocket::GetServiceByPort(WORD port) const
 
 PString PSocket::GetServiceByPort(const char * protocol, WORD port)
 {
-#if !defined(BE_BONELESS) && !defined(__NUCLEUS_PLUS__) && !defined(_WIN32_WCE)
+#if !defined(BE_BONELESS) && !defined(__NUCLEUS_PLUS__) && !defined(_WIN32_WCE) && !defined(P_VXWORKS)
   struct servent * serv = ::getservbyport(htons(port), protocol);
   if (serv != NULL)
     return PString(serv->s_name);
@@ -1006,28 +1264,16 @@ int PSocket::Select(PSocket & sock1,
   int h1 = sock1.GetHandle();
   int h2 = sock2.GetHandle();
 
-#ifdef _MSC_VER
-#pragma warning(disable:4018 4127)
-#endif
-  fd_set readfds;
-  FD_ZERO(&readfds);
-  FD_SET(h1, &readfds);
-  FD_SET(h2, &readfds);
-  fd_set writefds;
-  FD_ZERO(&writefds);
-  fd_set exceptfds;
-  FD_ZERO(&exceptfds);
-#ifdef _MSC_VER
-#pragma warning(default:4018 4127)
-#endif
+  P_fd_set readfds;
+  readfds += h1;
+  readfds += h2;
 
   PIntArray allfds(4);
   allfds[0] = h1;
   allfds[1] = 1;
   allfds[2] = h2;
   allfds[3] = 1;
-  int rval = os_select(PMAX(h1, h2)+1,
-                                readfds, writefds, exceptfds, allfds, timeout);
+  int rval = os_select(PMAX(h1, h2)+1, (fd_set*)readfds, NULL, NULL, allfds, timeout);
 
   Errors lastError;
   int osError;
@@ -1035,9 +1281,9 @@ int PSocket::Select(PSocket & sock1,
     return lastError;
 
   rval = 0;
-  if (FD_ISSET(h1, &readfds))
+  if (readfds.IsPresent(h1))
     rval -= 1;
-  if (FD_ISSET(h2, &readfds))
+  if (readfds.IsPresent(h2))
     rval -= 2;
 
   return rval;
@@ -1091,43 +1337,37 @@ PChannel::Errors PSocket::Select(SelectList & read,
   PINDEX nextfd = 0;
   PIntArray allfds(2*(read.GetSize()+write.GetSize()+except.GetSize()));
 
-#ifdef _MSC_VER
-#pragma warning(disable:4018 4127)
-#endif
-  fd_set readfds;
-  FD_ZERO(&readfds);
+  P_fd_set readfds;
   PINDEX i;
   for (i = 0; i < read.GetSize(); i++) {
     if (!read[i].IsOpen())
       return NotOpen;
     int h = read[i].GetHandle();
-    FD_SET(h, &readfds);
+    readfds += h;
     if (h > maxfds)
       maxfds = h;
     allfds[nextfd++] = h;
     allfds[nextfd++] = 1;
   }
 
-  fd_set writefds;
-  FD_ZERO(&writefds);
+  P_fd_set writefds;
   for (i = 0; i < write.GetSize(); i++) {
     if (!write[i].IsOpen())
       return NotOpen;
     int h = write[i].GetHandle();
-    FD_SET(h, &writefds);
+    writefds += h;
     if (h > maxfds)
       maxfds = h;
     allfds[nextfd++] = h;
     allfds[nextfd++] = 2;
   }
 
-  fd_set exceptfds;
-  FD_ZERO(&exceptfds);
+  P_fd_set exceptfds;
   for (i = 0; i < except.GetSize(); i++) {
     if (!except[i].IsOpen())
       return NotOpen;
     int h = except[i].GetHandle();
-    FD_SET(h, &exceptfds);
+    exceptfds += h;
     if (h > maxfds)
       maxfds = h;
     allfds[nextfd++] = h;
@@ -1137,7 +1377,7 @@ PChannel::Errors PSocket::Select(SelectList & read,
 #pragma warning(default:4018 4127)
 #endif
 
-  int retval = os_select(maxfds+1,readfds,writefds,exceptfds,allfds,timeout);
+  int retval = os_select(maxfds+1,(fd_set*)readfds,(fd_set*)writefds,(fd_set*)exceptfds,allfds,timeout);
 
   Errors lastError;
   int osError;
@@ -1149,21 +1389,21 @@ PChannel::Errors PSocket::Select(SelectList & read,
       int h = read[i].GetHandle();
       if (h < 0)
         return Interrupted;
-      if (!FD_ISSET(h, &readfds))
+      if (!readfds.IsPresent(h))
         read.RemoveAt(i--);
     }
     for (i = 0; i < write.GetSize(); i++) {
       int h = write[i].GetHandle();
       if (h < 0)
         return Interrupted;
-      if (!FD_ISSET(h, &writefds))
+      if (!writefds.IsPresent(h))
         write.RemoveAt(i--);
     }
     for (i = 0; i < except.GetSize(); i++) {
       int h = except[i].GetHandle();
       if (h < 0)
         return Interrupted;
-      if (!FD_ISSET(h, &exceptfds))
+      if (!exceptfds.IsPresent(h))
         except.RemoveAt(i--);
     }
   }
@@ -1206,12 +1446,23 @@ void PIPSocket::ClearNameCache()
 
 PString PIPSocket::GetName() const
 {
-  PString name;
+#if P_HAS_IPV6
+
+  Psockaddr sa;
+  socklen_t size = sa.GetSize();
+  if (getpeername(os_handle, sa, &size) == 0)
+    return GetHostName(sa.GetIP()) + psprintf(":%u", sa.GetPort());
+
+#else
+
   sockaddr_in address;
   socklen_t size = sizeof(address);
   if (getpeername(os_handle, (struct sockaddr *)&address, &size) == 0)
-    name = GetHostName(address.sin_addr) + psprintf(":%u", ntohs(address.sin_port));
-  return name;
+    return GetHostName(address.sin_addr) + psprintf(":%u", ntohs(address.sin_port));
+
+#endif
+
+  return PString::Empty();
 }
 
 
@@ -1243,13 +1494,13 @@ PString PIPSocket::GetHostName(const PString & hostname)
 PString PIPSocket::GetHostName(const Address & addr)
 {
   if (addr == 0)
-    return addr;
+    return addr.AsString();
 
   PString hostname;
   if (pHostByAddr().GetHostName(addr, hostname))
     return hostname;
 
-  return addr;
+  return addr.AsString();
 }
 
 
@@ -1309,18 +1560,24 @@ PStringArray PIPSocket::GetHostAliases(const Address & addr)
 
 BOOL PIPSocket::GetLocalAddress(Address & addr)
 {
-  sockaddr_in address;
-  socklen_t size = sizeof(address);
-  if (!ConvertOSError(::getsockname(os_handle,(struct sockaddr*)&address,&size)))
-    return FALSE;
-
-  addr = address.sin_addr;
-  return TRUE;
+  WORD dummy;
+  return GetLocalAddress(addr, dummy);
 }
 
-
 BOOL PIPSocket::GetLocalAddress(Address & addr, WORD & portNum)
 {
+#if P_HAS_IPV6
+
+  Psockaddr sa;
+  socklen_t size = sa.GetSize();
+  if (!ConvertOSError(::getsockname(os_handle, sa, &size)))
+    return FALSE;
+
+  addr = sa.GetIP();
+  portNum = sa.GetPort();
+
+#else
+
   sockaddr_in address;
   socklen_t size = sizeof(address);
   if (!ConvertOSError(::getsockname(os_handle,(struct sockaddr*)&address,&size)))
@@ -1328,23 +1585,33 @@ BOOL PIPSocket::GetLocalAddress(Address & addr, WORD & portNum)
 
   addr = address.sin_addr;
   portNum = ntohs(address.sin_port);
+
+#endif
+
   return TRUE;
 }
- 
+
+
 BOOL PIPSocket::GetPeerAddress(Address & addr)
 {
-  sockaddr_in address;
-  socklen_t size = sizeof(address);
-  if (!ConvertOSError(::getpeername(os_handle,(struct sockaddr*)&address,&size)))
-    return FALSE;
-
-  addr = address.sin_addr;
-  return TRUE;
+  WORD portNum;
+  return GetPeerAddress(addr, portNum);
 }
 
-
 BOOL PIPSocket::GetPeerAddress(Address & addr, WORD & portNum)
 {
+#if P_HAS_IPV6
+
+  Psockaddr sa;
+  socklen_t size = sa.GetSize();
+  if (!ConvertOSError(::getpeername(os_handle, sa, &size)))
+    return FALSE;
+
+  addr = sa.GetIP();
+  portNum = sa.GetPort();
+
+#else
+
   sockaddr_in address;
   socklen_t size = sizeof(address);
   if (!ConvertOSError(::getpeername(os_handle,(struct sockaddr*)&address,&size)))
@@ -1352,33 +1619,32 @@ BOOL PIPSocket::GetPeerAddress(Address & addr, WORD & portNum)
 
   addr = address.sin_addr;
   portNum = ntohs(address.sin_port);
+
+#endif
+
   return TRUE;
 }
 
 
 PString PIPSocket::GetLocalHostName()
 {
-  PString name;
+  Address addr;
 
-  sockaddr_in address;
-  socklen_t size = sizeof(address);
-  if (ConvertOSError(::getsockname(os_handle, (struct sockaddr *)&address, &size)))
-    name = GetHostName(address.sin_addr);
+  if (GetLocalAddress(addr))
+    return GetHostName(addr);
 
-  return name;
+  return PString::Empty();
 }
 
 
 PString PIPSocket::GetPeerHostName()
 {
-  PString name;
+  Address addr;
 
-  sockaddr_in address;
-  socklen_t size = sizeof(address);
-  if (ConvertOSError(::getpeername(os_handle, (struct sockaddr *)&address, &size)))
-    name = GetHostName(address.sin_addr);
+  if (GetPeerAddress(addr))
+    return GetHostName(addr);
 
-  return name;
+  return PString::Empty();
 }
 
 
@@ -1418,13 +1684,46 @@ BOOL PIPSocket::Connect(const Address & iface, WORD localPort, const Address & a
   // make sure we have a port
   PAssert(port != 0, "Cannot connect socket without setting port");
 
+#if P_HAS_IPV6
+
+  Psockaddr sa(addr, port);
+
+  // attempt to create a socket with the right family
+  if (!OpenSocket(sa->sa_family))
+    return FALSE;
+
+  if (localPort != 0 || iface.IsValid()) {
+    Psockaddr bind_sa(iface, localPort);
+
+    if (sa->sa_family != bind_sa->sa_family) {
+      os_close();
+      return FALSE;
+    }
+
+    if (!SetOption(SO_REUSEADDR, 0)) {
+      os_close();
+      return FALSE;
+    }
+    
+    if (!ConvertOSError(::bind(os_handle, bind_sa, bind_sa.GetSize()))) {
+      os_close();
+      return FALSE;
+    }
+  }
+  
+  // attempt to connect
+  if (os_connect(sa, sa.GetSize()))
+    return TRUE;
+  
+#else
+
   // attempt to create a socket
   if (!OpenSocket())
     return FALSE;
 
   // attempt to connect
   sockaddr_in sin;
-  if (localPort != 0 || iface != INADDR_ANY) {
+  if (localPort != 0 || iface.IsValid()) {
     if (!SetOption(SO_REUSEADDR, 0)) {
       os_close();
       return FALSE;
@@ -1446,6 +1745,8 @@ BOOL PIPSocket::Connect(const Address & iface, WORD localPort, const Address & a
   if (os_connect((struct sockaddr *)&sin, sizeof(sin)))
     return TRUE;
 
+#endif
+
   os_close();
   return FALSE;
 }
@@ -1466,60 +1767,114 @@ BOOL PIPSocket::Listen(const Address & bindAddr,
   if (newPort != 0)
     port = newPort;
 
-  // close the port if it is already open
+#if P_HAS_IPV6
+  Psockaddr bind_sa(bindAddr, port); 
+
+  if (IsOpen()) {
+    int socketType;
+    if (!GetOption(SO_TYPE, socketType, SOL_SOCKET) || bind_sa->sa_family != socketType)
+      Close();
+  }
+#endif
+
   if (!IsOpen()) {
     // attempt to create a socket
+#if P_HAS_IPV6
+    if (!OpenSocket(bind_sa->sa_family))
+      return FALSE;
+#else
     if (!OpenSocket())
       return FALSE;
+#endif
   }
+  
+  // attempt to listen
+  if (!SetOption(SO_REUSEADDR, reuse == CanReuseAddress ? 1 : 0)) {
+    os_close();
+    return FALSE;
+  }
+
+#if P_HAS_IPV6
+
+  if (ConvertOSError(::bind(os_handle, bind_sa, bind_sa.GetSize()))) {
+    Psockaddr sa;
+    socklen_t size = sa.GetSize();
+    if (!ConvertOSError(::getsockname(os_handle, sa, &size)))
+      return FALSE;
+
+    port = sa.GetPort();
+    return TRUE;
+  }
+
+#else
 
   // attempt to listen
-  if (SetOption(SO_REUSEADDR, reuse == CanReuseAddress ? 1 : 0)) {
-    // attempt to listen
-    sockaddr_in sin;
-    memset(&sin, 0, sizeof(sin));
-    sin.sin_family      = AF_INET;
-    sin.sin_addr.s_addr = bindAddr;
-    sin.sin_port        = htons(port);       // set the port
+  sockaddr_in sin;
+  memset(&sin, 0, sizeof(sin));
+  sin.sin_family      = AF_INET;
+  sin.sin_addr.s_addr = bindAddr;
+  sin.sin_port        = htons(port);       // set the port
 
 #ifdef __NUCLEUS_NET__
-    int bind_result;
-    if (port == 0)
-      bind_result = ::bindzero(os_handle, (struct sockaddr*)&sin, sizeof(sin));
-    else
-      bind_result = ::bind(os_handle, (struct sockaddr*)&sin, sizeof(sin));
-    if (ConvertOSError(bind_result)) {
+  int bind_result;
+  if (port == 0)
+    bind_result = ::bindzero(os_handle, (struct sockaddr*)&sin, sizeof(sin));
+  else
+    bind_result = ::bind(os_handle, (struct sockaddr*)&sin, sizeof(sin));
+  if (ConvertOSError(bind_result))
 #else
-    if (ConvertOSError(::bind(os_handle, (struct sockaddr*)&sin, sizeof(sin)))) {
+  if (ConvertOSError(::bind(os_handle, (struct sockaddr*)&sin, sizeof(sin))))
 #endif
-      socklen_t size = sizeof(sin);
-      if (ConvertOSError(::getsockname(os_handle, (struct sockaddr*)&sin, &size))) {
-        port = ntohs(sin.sin_port);
-        return TRUE;
-      }
+  {
+    socklen_t size = sizeof(sin);
+    if (ConvertOSError(::getsockname(os_handle, (struct sockaddr*)&sin, &size))) {
+      port = ntohs(sin.sin_port);
+      return TRUE;
     }
   }
 
+#endif
+
   os_close();
   return FALSE;
 }
 
 
+///////////////////////////////////////////////////////////////////////////////
+// PIPSocket::Address
+
+static PIPSocket::Address loopback4(127,0,0,1);
+static PIPSocket::Address broadcast4(INADDR_BROADCAST);
+static in_addr inaddr_empty;
+#if P_HAS_IPV6
+static PIPSocket::Address loopback6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\001");
+static in6_addr inaddr6_empty;
+#endif
+
+
+const PIPSocket::Address & PIPSocket::Address::GetLoopback()
+{
+  return loopback4;
+}
+
+
+#if P_HAS_IPV6
+const PIPSocket::Address & PIPSocket::Address::GetLoopback6()
+{
+  return loopback6;
+}
+#endif
+
+
+const PIPSocket::Address & PIPSocket::Address::GetBroadcast()
+{
+  return broadcast4;
+}
+
+
 PIPSocket::Address::Address()
 {
-  s_addr = inet_addr("127.0.0.1");
-}
-
-
-PIPSocket::Address::Address(const in_addr & addr)
-{
-  s_addr = addr.s_addr;
-}
-
-
-PIPSocket::Address::Address(const Address & addr)
-{
-  s_addr = addr.s_addr;
+  *this = loopback4;
 }
 
 
@@ -1529,6 +1884,42 @@ PIPSocket::Address::Address(const PString & dotNotation)
 }
 
 
+PIPSocket::Address::Address(PINDEX len, const BYTE * bytes)
+{
+  switch (len) {
+#if P_HAS_IPV6
+    case 16 :
+      version = 6;
+      memcpy(&v.six, bytes, len);
+      break;
+#endif
+    case 4 :
+      version = 4;
+      memcpy(&v.four, bytes, len);
+      break;
+
+    case 0 :
+      version = 0;
+  }
+}
+
+
+PIPSocket::Address::Address(const in_addr & addr)
+{
+  version = 4;
+  v.four = addr;
+}
+
+
+#if P_HAS_IPV6
+PIPSocket::Address::Address(const in6_addr & addr)
+{
+  version = 6;
+  v.six = addr;
+}
+#endif
+
+
 #ifdef __NUCLEUS_NET__
 PIPSocket::Address::Address(const struct id_struct & addr)
 {
@@ -1545,101 +1936,199 @@ PIPSocket::Address & PIPSocket::Address::operator=(const struct id_struct & addr
   return *this;
 }
 #endif
-
+ 
 
 PIPSocket::Address & PIPSocket::Address::operator=(const in_addr & addr)
 {
-  s_addr = addr.s_addr;
+  version = 4;
+  v.four = addr;
   return *this;
 }
 
-
-PIPSocket::Address & PIPSocket::Address::operator=(const Address & addr)
+#if P_HAS_IPV6
+PIPSocket::Address & PIPSocket::Address::operator=(const in6_addr & addr)
 {
-  s_addr = addr.s_addr;
+  version = 6;
+  v.six = addr;
   return *this;
 }
+#endif
+
+
+PObject::Comparison PIPSocket::Address::Compare(const PObject & obj) const
+{
+  const PIPSocket::Address & other = (const PIPSocket::Address &)obj;
+
+  if (version < other.version)
+    return LessThan;
+  if (version > other.version)
+    return GreaterThan;
+
+#if P_HAS_IPV6
+  if (version == 6) {
+    int result = memcmp(&v.six, &other.v.six, sizeof(v.six));
+    if (result < 0)
+      return LessThan;
+    if (result > 0)
+      return GreaterThan;
+    return EqualTo;
+  }
+#endif
+
+  if ((DWORD)*this < other)
+    return LessThan;
+  if ((DWORD)*this > other)
+    return GreaterThan;
+  return EqualTo;
+}
+
+
+#if P_HAS_IPV6
+BOOL PIPSocket::Address::operator==(in6_addr & addr) const
+{
+  PIPSocket::Address a(addr);
+  return Compare(a) == EqualTo;
+}
+#endif
+
+
+BOOL PIPSocket::Address::operator==(in_addr & addr) const
+{
+  PIPSocket::Address a(addr);
+  return Compare(a) == EqualTo;
+}
+
+
+BOOL PIPSocket::Address::operator==(DWORD dw) const
+{
+  if (dw != 0)
+    return (DWORD)*this == dw;
+
+  return !IsValid();
+}
 
 
 PIPSocket::Address & PIPSocket::Address::operator=(const PString & dotNotation)
 {
-  if (::strspn(dotNotation, "0123456789.") < ::strlen(dotNotation))
-    s_addr = 0;
-  else {
-    s_addr = inet_addr((const char *)dotNotation);
-    if (s_addr == (DWORD)INADDR_NONE)
-      s_addr = 0;
+#if P_HAS_IPV6
+
+  struct addrinfo *res;
+  struct addrinfo hints= {AI_PASSIVE, 0, SOCK_STREAM, 0, 0, NULL, NULL, NULL };
+  
+  version = 0;
+  
+  hints.ai_family = PF_UNSPEC;//PF_INET6; Could be IPv4: x.x.x.x or IPv6: x:x:x:x::x
+  hints.ai_socktype = 0;//SOCK_STREAM;
+  
+  if (getaddrinfo((const char *)dotNotation, NULL , &hints, &res) == 0) {
+    if (res->ai_family == PF_INET6) {
+      // IPv6 addr
+      version = 6;
+      struct sockaddr_in6 * addr_in6 = (struct sockaddr_in6 *)res->ai_addr;
+      v.six = addr_in6->sin6_addr;
+    } else {
+      // IPv4 addr
+      version = 4;
+      struct sockaddr_in * addr_in = (struct sockaddr_in *)res->ai_addr;
+      v.four = addr_in->sin_addr;
+    }
+    freeaddrinfo(res);
   }
+
+#else //P_HAS_IPV6
+
+  if (::strspn(dotNotation, "0123456789.") < ::strlen(dotNotation))
+    *this = 0;
+  else {
+    version = 4;
+    v.four.s_addr = inet_addr((const char *)dotNotation);
+    if (v.four.s_addr == (DWORD)INADDR_NONE)
+      v.four.s_addr = 0;
+  }
+
+#endif
+
   return *this;
 }
 
 
 PString PIPSocket::Address::AsString() const
 {
-  return inet_ntoa(*this);
+#if P_HAS_IPV6
+  if (version == 6) {
+    PString str;
+    Psockaddr sa(*this, 0);
+    PAssertOS(getnameinfo(sa, sa.GetSize(), str.GetPointer(1024), 1024, NULL, 0, NI_NUMERICHOST) == 0);
+    PINDEX percent = str.Find('%'); // used for scoped address e.g. fe80::1%ne0, (ne0=network interface 0)
+    if (percent != P_MAX_INDEX)
+      str[percent] = '\0';
+    str.MakeMinimumSize();
+    return str;
+  }
+#endif
+
+  return inet_ntoa(v.four);
 }
 
 
 BOOL PIPSocket::Address::FromString(const PString & dotNotation)
 {
-  if (::strspn(dotNotation, "0123456789.") < ::strlen(dotNotation))
-    return FALSE;
+  (*this) = dotNotation;
+  return IsValid();
 
-  PINDEX dot1 = dotNotation.Find('.');
-  if (dot1 == P_MAX_INDEX)
-    return FALSE;
-
-  PINDEX dot2 = dotNotation.Find('.', dot1+1);
-  if (dot2 == P_MAX_INDEX)
-    return FALSE;
-
-  PINDEX dot3 = dotNotation.Find('.', dot2+1);
-  if (dot3 == P_MAX_INDEX)
-    return FALSE;
-
-  unsigned b1 = dotNotation(0, dot1-1).AsUnsigned();
-  if (b1 > 255)
-    return FALSE;
-
-  unsigned b2 = dotNotation(dot1+1, dot2-1).AsUnsigned();
-  if (b2 > 255)
-    return FALSE;
-
-  unsigned b3 = dotNotation(dot2+1, dot3-1).AsUnsigned();
-  if (b3 > 255)
-    return FALSE;
-
-  unsigned b4 = dotNotation.Mid(dot3+1).AsUnsigned();
-  if (b4 > 255)
-    return FALSE;
-
-  *this = PIPSocket::Address((BYTE)b1, (BYTE)b2, (BYTE)b3, (BYTE)b4);
-  return TRUE;
 }
 
 
 PIPSocket::Address::operator PString() const
 {
-  return inet_ntoa(*this);
+  return AsString();
 }
 
 
+PIPSocket::Address::operator in_addr() const
+{
+  if (version != 4)
+    return inaddr_empty;
+
+  return v.four;
+}
+
+
+#if P_HAS_IPV6
+PIPSocket::Address::operator in6_addr() const
+{
+  if (version != 6)
+    return inaddr6_empty;
+
+  return v.six;
+}
+#endif
+
+
 BYTE PIPSocket::Address::operator[](PINDEX idx) const
 {
   PASSERTINDEX(idx);
+#if P_HAS_IPV6
+  if (version == 6) {
+    PAssert(idx <= 15, PInvalidParameter);
+    return v.six.s6_addr[idx];
+  }
+#endif
+
   PAssert(idx <= 3, PInvalidParameter);
-  return ((BYTE *)this)[idx];
+  return ((BYTE *)&v.four)[idx];
 }
 
 
 ostream & operator<<(ostream & s, const PIPSocket::Address & a)
 {
-  return s << inet_ntoa(a);
+  return s << a.AsString();
 }
 
 
 istream & operator>>(istream & s, PIPSocket::Address & a)
 {
+/// Not IPv6 ready !!!!!!!!!!!!!
   char dot1, dot2, dot3;
   unsigned b1, b2, b3, b4;
   s >> b1 >> dot1 >> b2 >> dot2 >> b3 >> dot3 >> b4;
@@ -1651,6 +2140,58 @@ istream & operator>>(istream & s, PIPSocket::Address & a)
 }
 
 
+PINDEX PIPSocket::Address::GetSize() const
+{
+  switch (version) {
+#if P_HAS_IPV6
+    case 6 :
+      return 16;
+#endif
+
+    case 4 :
+      return 4;
+  }
+
+  return 0;
+}
+
+
+BOOL PIPSocket::Address::IsValid() const
+{
+  switch (version) {
+#if P_HAS_IPV6
+    case 6 :
+      return memcmp(&v.six, &inaddr6_empty, sizeof(v.six)) != 0;
+#endif
+
+    case 4 :
+      return (DWORD)*this != INADDR_ANY;
+  }
+  return FALSE;
+}
+
+
+BOOL PIPSocket::Address::IsLoopback() const
+{
+#if P_HAS_IPV6
+  if (version == 6)
+    return *this == loopback6;
+#endif
+  return *this == loopback4;
+}
+
+
+BOOL PIPSocket::Address::IsBroadcast() const
+{
+#if P_HAS_IPV6
+  if (version == 6) // In IPv6, no broadcast exist. Only multicast
+    return FALSE;
+#endif
+
+  return *this == broadcast4;
+}
+
+
 PIPSocket::InterfaceEntry::InterfaceEntry(const PString & _name,
                                           const Address & _addr,
                                           const Address & _mask,
@@ -1740,12 +2281,20 @@ PObject * PTCPSocket::Clone() const
 }
 
 
+// By default IPv4 only adresses
 BOOL PTCPSocket::OpenSocket()
 {
   return ConvertOSError(os_handle = os_socket(AF_INET, SOCK_STREAM, 0));
 }
 
 
+// ipAdressFamily should be AF_INET or AF_INET6
+BOOL PTCPSocket::OpenSocket(int ipAdressFamily) 
+{
+  return ConvertOSError(os_handle = os_socket(ipAdressFamily, SOCK_STREAM, 0));
+}
+
+
 const char * PTCPSocket::GetProtocolName() const
 {
   return "tcp";
@@ -1793,12 +2342,23 @@ BOOL PTCPSocket::Accept(PSocket & socket)
 {
   PAssert(socket.IsDescendant(PIPSocket::Class()), "Invalid listener socket");
 
+#if P_HAS_IPV6
+
+  Psockaddr sa;
+  PINDEX size = sa.GetSize();
+  if (!os_accept(socket, sa, &size))
+    return FALSE;
+
+#else
+
   sockaddr_in address;
   address.sin_family = AF_INET;
   PINDEX size = sizeof(address);
   if (!os_accept(socket, (struct sockaddr *)&address, &size))
     return FALSE;
 
+#endif
+
   port = ((PIPSocket &)socket).GetPort();
   return TRUE;
 }
@@ -1810,6 +2370,8 @@ BOOL PTCPSocket::WriteOutOfBand(void const * buf, PINDEX len)
   PAssertAlways("WriteOutOfBand unavailable on Nucleus Plus");
   //int count = NU_Send(os_handle, (char *)buf, len, 0);
   int count = ::send(os_handle, (const char *)buf, len, 0);
+#elif defined(P_VXWORKS)
+  int count = ::send(os_handle, (char *)buf, len, MSG_OOB);
 #else
   int count = ::send(os_handle, (const char *)buf, len, MSG_OOB);
 #endif
@@ -1842,6 +2404,17 @@ BOOL PIPDatagramSocket::ReadFrom(void * buf, PINDEX len,
 {
   lastReadCount = 0;
 
+#if P_HAS_IPV6
+
+  Psockaddr sa;
+  PINDEX size = sa.GetSize();
+  if (os_recvfrom(buf, len, 0, sa, &size)) {
+    addr = sa.GetIP();
+    port = sa.GetPort();
+  }
+
+#else
+
   sockaddr_in sockAddr;
   PINDEX addrLen = sizeof(sockAddr);
   if (os_recvfrom(buf, len, 0, (struct sockaddr *)&sockAddr, &addrLen)) {
@@ -1849,6 +2422,8 @@ BOOL PIPDatagramSocket::ReadFrom(void * buf, PINDEX len,
     port = ntohs(sockAddr.sin_port);
   }
 
+#endif
+
   return lastReadCount > 0;
 }
 
@@ -1858,12 +2433,21 @@ BOOL PIPDatagramSocket::WriteTo(const void * buf, PINDEX len,
 {
   lastWriteCount = 0;
 
+#if P_HAS_IPV6
+
+  Psockaddr sa(addr, port);
+  return os_sendto(buf, len, 0, sa, sa.GetSize()) && lastWriteCount >= len;
+
+#else
+
   sockaddr_in sockAddr;
   sockAddr.sin_family = AF_INET;
   sockAddr.sin_addr = addr;
   sockAddr.sin_port = htons(port);
   return os_sendto(buf, len, 0, (struct sockaddr *)&sockAddr, sizeof(sockAddr))
          && lastWriteCount >= len;
+
+#endif
 }
 
 
@@ -1907,6 +2491,10 @@ BOOL PUDPSocket::OpenSocket()
   return ConvertOSError(os_handle = os_socket(AF_INET, SOCK_DGRAM, 0));
 }
 
+BOOL PUDPSocket::OpenSocket(int ipAdressFamily)
+{
+  return ConvertOSError(os_handle = os_socket(ipAdressFamily, SOCK_DGRAM, 0));
+}
 
 const char * PUDPSocket::GetProtocolName() const
 {
