@@ -27,6 +27,27 @@
  * Contributor(s): Loopback feature: Philip Edelbrock <phil@netroedge.com>.
  *
  * $Log: oss.cxx,v $
+ * Revision 1.55  2002/12/12 09:03:56  rogerh
+ * On two FreeBSD machines, Read() calls from the sound card were not blocking
+ * correctly and returned with less bytes than asked for. This made OpenH323
+ * close the sound channel.  Add a FreeBSD workaround so Read() loops until it
+ * has all the bytes requested.
+ *
+ * Revision 1.54  2002/12/03 23:03:54  rogerh
+ * oops - remove some test code which should not have been committed
+ *
+ * Revision 1.53  2002/12/03 19:11:58  rogerh
+ * Open sound device in non blocking mode incase it is already open.
+ *
+ * Revision 1.52  2002/11/28 12:15:24  rogerh
+ * Change SetVolume/GetVolume to use the mic and not the igain for the input
+ * volume.
+ * Our target audience are likely to be using mics and many broken
+ * sound drivers do not implement igain properly.
+ *
+ * Revision 1.51  2002/10/17 12:57:24  robertj
+ * Added ability to increase maximum file handles on a process.
+ *
  * Revision 1.50  2002/10/15 10:42:45  rogerh
  * Fix loopback mode, which was broken in a recent change.
  *
@@ -198,6 +219,7 @@
 #pragma implementation "sound.h"
 
 #include <ptlib.h>
+#include <ptlib/socket.h>
 
 #ifdef P_LINUX
 #include <sys/soundcard.h>
@@ -548,12 +570,13 @@ PStringArray PSoundChannel::GetDeviceNames(Directions /*dir*/)
         ::close(fd);
       }
       else {
+
         // mixer failed but this could still be a valid dsp...
         // warning this is just a hack to make this work on strange mixer and dsp configurations
         // on my machine the first sound card registers 1 mixer and 2 dsp, so when my webcam
         // registers itself as dsp2 this test would fail...
-        int fd = ::open(dsp[cardnum], O_RDONLY);
-        if (fd >= 0) {
+        int fd = ::open(dsp[cardnum], O_RDONLY | O_NONBLOCK);
+        if (fd >= 0 || errno == EBUSY) {
           devices.AppendString(dsp[cardnum]);
           ::close(fd);
         }
@@ -562,8 +585,9 @@ PStringArray PSoundChannel::GetDeviceNames(Directions /*dir*/)
     else {
       // No mixer available, try and open it directly, this could fail if
       // the device happens to be open already
-      int fd = ::open(dsp[cardnum], O_RDONLY);
-      if (fd >= 0) {
+      int fd = ::open(dsp[cardnum], O_RDONLY | O_NONBLOCK);
+
+      if (fd >= 0 || errno == EBUSY) {
         devices.AppendString(dsp[cardnum]);
         ::close(fd);
       }
@@ -848,6 +872,26 @@ BOOL PSoundChannel::Read(void * buf, PINDEX len)
 
   if (os_handle > 0) {
     PTRACE(6, "OSS\tRead start");
+
+#if defined(P_FREEBSD)
+    PINDEX total = 0;
+    while (total < len) {
+      PINDEX bytes = 0;
+      while (!ConvertOSError(bytes = ::read(os_handle, (void *)(((unsigned int)buf) + total), len-total))) {
+        if (GetErrorCode() != Interrupted) {
+          PTRACE(6, "OSS\tRead failed");
+          return FALSE;
+        }
+        PTRACE(6, "OSS\tRead interrupted");
+      }
+      total += bytes;
+      if (total != len)
+        PTRACE(6, "OSS\tRead completed short - " << total << " vs " << len << ". Reading more data");
+    }
+    lastReadCount = total;
+
+#else
+
     while (!ConvertOSError(lastReadCount = ::read(os_handle, (void *)buf, len))) {
       if (GetErrorCode() != Interrupted) {
         PTRACE(6, "OSS\tRead failed");
@@ -855,6 +899,8 @@ BOOL PSoundChannel::Read(void * buf, PINDEX len)
       }
       PTRACE(6, "OSS\tRead interrupted");
     }
+#endif
+
 
     if (lastReadCount != len)
       PTRACE(6, "OSS\tRead completed short - " << lastReadCount << " vs " << len);
@@ -1146,12 +1192,9 @@ BOOL PSoundChannel::StartRecording()
   if (os_handle == 0)
     return TRUE;
 
-  fd_set fds;
-  FD_ZERO(&fds);
-  FD_SET(os_handle, &fds);
-
-  struct timeval instant = {0, 0};
-  return ConvertOSError(::select(1, &fds, NULL, NULL, &instant));
+  P_fd_set fds = os_handle;
+  P_timeval instant;
+  return ConvertOSError(::select(1, fds, NULL, NULL, instant));
 }
 
 
@@ -1224,7 +1267,7 @@ BOOL PSoundChannel::SetVolume(unsigned newVal)
   if (direction  == Player) 
     rc = ::ioctl(os_handle, MIXER_WRITE(SOUND_MIXER_VOLUME), &deviceVol);
    else 
-    rc = ::ioctl(os_handle, MIXER_WRITE(SOUND_MIXER_IGAIN), &deviceVol);
+    rc = ::ioctl(os_handle, MIXER_WRITE(SOUND_MIXER_MIC), &deviceVol);
 
   if (rc < 0) {
     PTRACE(1, "PSoundChannel::SetVolume failed : " << ::strerror(errno));
@@ -1243,7 +1286,7 @@ BOOL  PSoundChannel::GetVolume(unsigned &devVol)
   if (direction == Player)
     rc = ::ioctl(os_handle, MIXER_READ(SOUND_MIXER_VOLUME), &vol);
   else
-    rc = ::ioctl(os_handle, MIXER_READ(SOUND_MIXER_IGAIN), &vol);
+    rc = ::ioctl(os_handle, MIXER_READ(SOUND_MIXER_MIC), &vol);
   
   if (rc < 0) {
     PTRACE(1,  "PSoundChannel::GetVolume failed : " << ::strerror(errno)) ;
