@@ -27,6 +27,30 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: osutil.cxx,v $
+ * Revision 1.76  2002/11/22 10:14:07  robertj
+ * QNX port, thanks Xiaodan Tang
+ *
+ * Revision 1.75  2002/11/20 01:55:06  robertj
+ * Fixed to follow new semantics of GetPath(), first entry is volume which on
+ *   UNix machines is always an empty string. Also collapses consecutive
+ *   slashes as they are meaningless.
+ *
+ * Revision 1.74  2002/11/19 11:21:30  robertj
+ * Changed PFilePath so can be empty string, indicating illegal path.
+ * Added function to extract a path as an array of directories components.
+ *
+ * Revision 1.73  2002/10/22 07:42:52  robertj
+ * Added extra debugging for file handle and thread leak detection.
+ *
+ * Revision 1.72  2002/10/17 13:44:27  robertj
+ * Port to RTEMS, thanks Vladimir Nesic.
+ *
+ * Revision 1.71  2002/10/17 12:57:24  robertj
+ * Added ability to increase maximum file handles on a process.
+ *
+ * Revision 1.70  2002/10/10 04:43:44  robertj
+ * VxWorks port, thanks Martijn Roest
+ *
  * Revision 1.69  2002/06/06 09:28:42  robertj
  * Fixed problems with canonicalising directories now PINDEX is signed.
  *
@@ -173,8 +197,12 @@
 
 
 #include <fcntl.h>
+#ifdef P_VXWORKS
+#include <sys/times.h>
+#else
 #include <time.h>
 #include <sys/time.h>
+#endif
 #include <ctype.h>
 
 #if defined(P_LINUX)
@@ -226,6 +254,20 @@
 #include <stdio.h>
 #include <mntent.h>
 
+#elif defined(P_VXWORKS)
+#define P_USE_STRFTIME
+
+#elif defined(P_RTEMS)
+#define P_USE_STRFTIME
+#include <time.h>
+#include <stdio.h>
+#define random() rand()
+#define srandom(a) srand(a)
+
+#elif defined(P_QNX)
+#include <sys/dcmd_blk.h>
+#include <sys/statvfs.h>
+#define P_USE_STRFTIME
 #endif
 
 #ifdef P_USE_LANGINFO
@@ -255,6 +297,33 @@ static char *tzname[2] = { "STD", "DST" };
 #define new PNEW
 
 
+int PX_NewHandle(const char * clsName, int fd)
+{
+  if (fd < 0)
+    return fd;
+
+  static int lowWaterMark = INT_MAX;
+  static int highWaterMark = 0;
+  if (fd > highWaterMark) {
+    highWaterMark = fd;
+    lowWaterMark = fd;
+
+    int maxHandles = PProcess::Current().GetMaxHandles();
+    if (fd < (maxHandles-maxHandles/20))
+      PTRACE(4, "PWLib\tFile handle high water mark set: " << fd << ' ' << clsName);
+    else
+      PTRACE(1, "PWLib\tFile handle high water mark within 5% of maximum: " << fd << ' ' << clsName);
+  }
+
+  if (fd < lowWaterMark) {
+    lowWaterMark = fd;
+    PTRACE(4, "PWLib\tFile handle low water mark set: " << fd << ' ' << clsName);
+  }
+
+  return fd;
+}
+
+
 static PString CanonicaliseDirectory (const PString & path)
 
 {
@@ -306,9 +375,12 @@ static PString CanonicaliseDirectory (const PString & path)
   return canonical_path;
 }
 
-static PString CanonicaliseFilename(const PString & filename)
 
+static PString CanonicaliseFilename(const PString & filename)
 {
+  if (filename.IsEmpty())
+    return filename;
+
   PINDEX p;
   PString dirname;
 
@@ -327,8 +399,10 @@ static PString CanonicaliseFilename(const PString & filename)
 PInt64 PString::AsInt64(unsigned base) const
 {
   char * dummy;
-#if defined(P_SOLARIS) || defined(__BEOS__) || defined (P_AIX) || defined(P_IRIX)
+#if defined(P_SOLARIS) || defined(__BEOS__) || defined (P_AIX) || defined(P_IRIX) || defined (P_QNX)
   return strtoll(theArray, &dummy, base);
+#elif defined(P_VXWORKS) || defined(P_RTEMS)
+  return strtol(theArray, &dummy, base);
 #else
   return strtoq(theArray, &dummy, base);
 #endif
@@ -337,30 +411,16 @@ PInt64 PString::AsInt64(unsigned base) const
 PUInt64 PString::AsUnsigned64(unsigned base) const
 {
   char * dummy;
-#if defined(P_SOLARIS) || defined(__BEOS__) || defined (P_AIX) || defined (P_IRIX)
+#if defined(P_SOLARIS) || defined(__BEOS__) || defined (P_AIX) || defined (P_IRIX) || defined (P_QNX)
   return strtoull(theArray, &dummy, base);
+#elif defined(P_VXWORKS) || defined(P_RTEMS)
+  return strtoul(theArray, &dummy, base);
 #else
   return strtouq(theArray, &dummy, base);
 #endif
 }
 
 
-///////////////////////////////////////////////////////////////////////////////
-//
-// time interval
-
-struct timeval * PTimeInterval::AsTimeVal(struct timeval & buffer) const
-{
-  if (*this == PMaxTimeInterval)
-    return NULL;
-
-  buffer.tv_usec = (milliseconds % 1000) * 1000;
-  buffer.tv_sec  =  milliseconds / 1000;
-  return &buffer;
-}
-
-
-
 ///////////////////////////////////////////////////////////////////////////////
 //
 // timer
@@ -369,9 +429,15 @@ struct timeval * PTimeInterval::AsTimeVal(struct timeval & buffer) const
 PTimeInterval PTimer::Tick()
 
 {
+#ifdef P_VXWORKS
+  struct timespec ts;
+  clock_gettime(0,&ts);
+  return (int)(ts.tv_sec*10000) + ts.tv_nsec/100000L;
+#else
   struct timeval tv;
   ::gettimeofday (&tv, NULL);
   return (PInt64)(tv.tv_sec) * 1000 + tv.tv_usec/1000L;
+#endif // P_VXWORKS
 }
 
 
@@ -523,7 +589,11 @@ BOOL PDirectory::Create(const PString & p, int perm)
   PString str = p;
   if (p[last] == '/')
     str = p.Left(last);
+#ifdef P_VXWORKS
+  return mkdir(str) == 0;
+#else		
   return mkdir(str, perm) == 0;
+#endif
 }
 
 BOOL PDirectory::Remove(const PString & p)
@@ -537,6 +607,18 @@ PString PDirectory::GetVolume() const
 {
   PString volume;
 
+#if defined(P_QNX)
+  int fd;
+  char mounton[257];
+
+  if ((fd = open(operator+("."), O_RDONLY)) != -1) {
+	mounton[256] = 0;
+	devctl(fd, DCMD_FSYS_MOUNTED_ON, mounton, 256, 0);
+	close(fd);
+	volume = strdup(mounton);
+  } 
+
+#else
   struct stat status;
   if (stat(operator+("."), &status) != -1) {
     dev_t my_dev = status.st_dev;
@@ -592,13 +674,17 @@ PString PDirectory::GetVolume() const
     }
     endfsent();
 
-#else
+#elif defined (P_VXWORKS)
 
+  PAssertAlways("Get Volume - not implemented for VxWorks");
+  return PString::Empty();
+
+#else
 #warning Platform requires implemetation of GetVolume()
 
 #endif
-
   }
+#endif
 
   return volume;
 }
@@ -617,7 +703,7 @@ BOOL PDirectory::GetVolumeSpace(PInt64 & total, PInt64 & free, DWORD & clusterSi
   free = fs.f_bavail*(PInt64)fs.f_bsize;
   return TRUE;
 
-#elif defined(P_AIX)
+#elif defined(P_AIX) || defined(P_VXWORKS)
 
   struct statfs fs;
   if (statfs((char *) ((const char *)operator+(".") ), &fs) == -1)
@@ -652,6 +738,18 @@ BOOL PDirectory::GetVolumeSpace(PInt64 & total, PInt64 & free, DWORD & clusterSi
   free = fs.f_bfree*(PInt64)fs.f_bsize;
   return TRUE;
 
+#elif defined(P_QNX)
+
+  struct statvfs fs;
+
+  if (statvfs(operator+("."), &fs) == -1)
+    return FALSE;
+
+  clusterSize = fs.f_bsize;
+  total = fs.f_blocks*(PInt64)fs.f_bsize;
+  free = fs.f_bavail*(PInt64)fs.f_bsize;
+  return TRUE;
+
 #else
 
 #warning Platform requires implemetation of GetVolumeSpace()
@@ -668,6 +766,29 @@ PDirectory PDirectory::GetParent() const
   return *this + "..";
 }
 
+PStringArray PDirectory::GetPath() const
+{
+  PStringArray path;
+
+  if (IsEmpty())
+    return path;
+
+  PStringArray tokens = Tokenise("/");
+
+  path.SetSize(tokens.GetSize()+1);
+
+  PINDEX count = 1; // First path field is volume name, empty under unix
+  for (PINDEX i = 0; i < tokens.GetSize(); i++) {
+    if (!tokens[i])
+     path[count++] = tokens[i];
+  }
+
+  path.SetSize(count);
+
+  return path;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // PFile
@@ -696,11 +817,24 @@ BOOL PFile::Open(OpenMode mode, int opt)
   if (path.IsEmpty()) {
     char templateStr[3+6+1];
     strcpy(templateStr, "PWL");
+#ifndef P_VXWORKS
+#ifdef P_RTEMS
+    _reent _reent_data;
+    os_handle = _mkstemp_r(&_reent_data, templateStr);
+#else
     os_handle = mkstemp(templateStr);
+#endif // P_RTEMS
     if (!ConvertOSError(os_handle))
       return FALSE;
 
   } else {
+#else
+    static int number = 0;
+    sprintf(templateStr+3, "%06d", number++);
+    path = templateStr;
+  }
+  {
+#endif // !P_VXWORKS
     int oflags = 0;
     switch (mode) {
       case ReadOnly :
@@ -730,11 +864,15 @@ BOOL PFile::Open(OpenMode mode, int opt)
       oflags |= O_TRUNC;
 
 
-    if (!ConvertOSError(os_handle = ::open(path, oflags, DEFAULT_FILE_MODE)))
+    if (!ConvertOSError(os_handle = PX_NewHandle(GetClass(), ::open(path, oflags, DEFAULT_FILE_MODE))))
       return FALSE;
   }
 
+#ifndef P_VXWORKS
   return ConvertOSError(::fcntl(os_handle, F_SETFD, 1));
+#else
+  return TRUE;
+#endif
 }
 
 
@@ -784,8 +922,35 @@ BOOL PFile::Move(const PFilePath & oldname, const PFilePath & newname, BOOL forc
 }
 
 
+BOOL PFile::Exists(const PFilePath & name)
+{ 
+#ifdef P_VXWORKS
+  // access function not defined for VxWorks
+  // as workaround, open the file in read-only mode
+  // if it succeeds, the file exists
+  PFile file(name, ReadOnly, MustExist);
+  BOOL exists = file.IsOpen();
+  if(exists == TRUE)
+    file.Close();
+  return exists;
+#else
+  return access(name, 0) == 0; 
+#endif // P_VXWORKS
+}
+
+
 BOOL PFile::Access(const PFilePath & name, OpenMode mode)
 {
+#ifdef P_VXWORKS
+  // access function not defined for VxWorks
+  // as workaround, open the file in specified mode
+  // if it succeeds, the access is allowed
+  PFile file(name, mode, ModeDefault);
+  BOOL access = file.IsOpen();
+  if(access == TRUE)
+    file.Close();
+  return access;
+#else	
   int accmode;
 
   switch (mode) {
@@ -802,6 +967,7 @@ BOOL PFile::Access(const PFilePath & name, OpenMode mode)
   }
 
   return access(name, accmode) == 0;
+#endif // P_VXWORKS
 }
 
 
@@ -810,9 +976,14 @@ BOOL PFile::GetInfo(const PFilePath & name, PFileInfo & status)
   status.type = PFileInfo::UnknownFileType;
 
   struct stat s;
+#ifdef P_VXWORKS
+  if (stat(name, &s) != OK)
+#else	
   if (lstat(name, &s) != 0)
+#endif // P_VXWORKS
     return FALSE;
 
+#ifndef P_VXWORKS
   if (S_ISLNK(s.st_mode)) {
     status.type = PFileInfo::SymbolicLink;
     if (stat(name, &s) != 0) {
@@ -824,7 +995,9 @@ BOOL PFile::GetInfo(const PFilePath & name, PFileInfo & status)
       return TRUE;
     }
   } 
-  else if (S_ISREG(s.st_mode))
+  else 
+#endif // !P_VXWORKS
+  if (S_ISREG(s.st_mode))
     status.type = PFileInfo::RegularFile;
   else if (S_ISDIR(s.st_mode))
     status.type = PFileInfo::SubDirectory;
@@ -834,10 +1007,10 @@ BOOL PFile::GetInfo(const PFilePath & name, PFileInfo & status)
     status.type = PFileInfo::CharDevice;
   else if (S_ISBLK(s.st_mode))
     status.type = PFileInfo::BlockDevice;
-#ifndef __BEOS__
+#if !defined(__BEOS__) && !defined(P_VXWORKS)
   else if (S_ISSOCK(s.st_mode))
     status.type = PFileInfo::SocketDevice;
-#endif // !__BEOS__
+#endif // !__BEOS__ || !P_VXWORKS
 
   status.created     = s.st_ctime;
   status.modified    = s.st_mtime;
@@ -878,7 +1051,15 @@ BOOL PFile::SetPermissions(const PFilePath & name, int permissions)
   if (permissions & PFileInfo::UserRead)
     mode |= S_IRUSR;
 
+#ifdef P_VXWORKS
+  PFile file(name, ReadOnly, MustExist);
+  if (file.IsOpen())
+    return (::ioctl(file.GetHandle(), FIOATTRIBSET, mode) >= 0);
+
+  return FALSE;
+#else	
   return chmod ((const char *)name, mode) == 0;
+#endif // P_VXWORKS
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -945,6 +1126,14 @@ PFilePath::PFilePath(const char * prefix, const char * dir)
   if (dir == NULL) 
     s = PDirectory("/tmp");
 
+#ifdef P_VXWORKS
+  int number = 0;
+  for (;;) {
+    *this = s + prefix + psprintf("%06x", number++);
+    if (!PFile::Exists(*this))
+      break;
+  }
+#else
   PString p;
   srandom(getpid());
   for (;;) {
@@ -952,6 +1141,7 @@ PFilePath::PFilePath(const char * prefix, const char * dir)
     if (!PFile::Exists(*this))
       break;
   }
+#endif // P_VXWORKS
 }
 
 
@@ -1074,7 +1264,12 @@ BOOL PConsoleChannel::Open(ConsoleType type)
 
 PString PConsoleChannel::GetName() const
 {
+#ifdef P_VXWORKS
+  PAssertAlways("PConsoleChannel::GetName - Not implemented for VxWorks");
+  return PString("Not Implemented");
+#else
   return ttyname(os_handle);
+#endif // P_VXWORKS
 }
 
 
@@ -1092,10 +1287,17 @@ BOOL PConsoleChannel::Close()
 
 PTime::PTime()
 {
+#ifdef P_VXWORKS
+  struct timespec ts;
+  clock_gettime(0,&ts);
+  theTime = ts.tv_sec;
+  microseconds = ts.tv_sec*10000 + ts.tv_nsec/100000L;
+#else
   struct timeval tv;
   gettimeofday(&tv, NULL);
   theTime = tv.tv_sec;
   microseconds = tv.tv_usec;
+#endif // P_VXWORKS
 }
 
 
@@ -1350,7 +1552,7 @@ int PTime::GetTimeZone(PTime::TimeZoneType type)
     return tz;
   else
     return tz + ::daylight*60;
-#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined(__BEOS__)
+#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined(__BEOS__) || defined(P_QNX)
   time_t t;
   time(&t);
   struct tm  * tm = localtime(&t);
@@ -1375,7 +1577,7 @@ int PTime::GetTimeZone(PTime::TimeZoneType type)
 
 PString PTime::GetTimeZoneString(PTime::TimeZoneType type) 
 {
-#if defined(P_LINUX) || defined(P_SUN4) || defined(P_SOLARIS) || defined (P_AIX) || defined(P_IRIX)
+#if defined(P_LINUX) || defined(P_SUN4) || defined(P_SOLARIS) || defined (P_AIX) || defined(P_IRIX) || defined(P_QNX)
   const char * str = (type == StandardTime) ? ::tzname[0] : ::tzname[1]; 
   if (str != NULL)
     return str;
