@@ -27,6 +27,37 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: socket.cxx,v $
+ * Revision 1.98  2002/11/22 10:14:07  robertj
+ * QNX port, thanks Xiaodan Tang
+ *
+ * Revision 1.97  2002/11/02 00:32:21  robertj
+ * Further fixes to VxWorks (Tornado) port, thanks Andreas Sikkema.
+ *
+ * Revision 1.96  2002/10/22 10:25:07  rogerh
+ * Fix process_rtentry() following Thomas's patch.
+ *
+ * Revision 1.95  2002/10/22 07:42:52  robertj
+ * Added extra debugging for file handle and thread leak detection.
+ *
+ * Revision 1.94  2002/10/22 06:53:38  craigs
+ * Fixed signed/unsigned problem in GetRoutTable thanks to Thomas Jalsovsky
+ *
+ * Revision 1.93  2002/10/19 06:12:20  robertj
+ * Moved P_fd_set::Zero() from platform independent to platform dependent
+ *   code as Win32 implementation is completely different from Unix.
+ *
+ * Revision 1.92  2002/10/17 13:44:27  robertj
+ * Port to RTEMS, thanks Vladimir Nesic.
+ *
+ * Revision 1.91  2002/10/17 12:57:24  robertj
+ * Added ability to increase maximum file handles on a process.
+ *
+ * Revision 1.90  2002/10/10 04:43:44  robertj
+ * VxWorks port, thanks Martijn Roest
+ *
+ * Revision 1.89  2002/10/08 14:31:43  robertj
+ * Changed for IPv6 support, thanks Sbastien Josset.
+ *
  * Revision 1.88  2002/06/05 12:29:16  craigs
  * Changes for gcc 3.1
  *
@@ -260,7 +291,7 @@
 #define	ifr_macaddr         ifr_hwaddr.sa_data
 #endif
 
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_SOLARIS) || defined(P_MACOSX) || defined(P_MACOS) || defined(P_IRIX)
+#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_SOLARIS) || defined(P_MACOSX) || defined(P_MACOS) || defined(P_IRIX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
 #define ifr_netmask ifr_addr
 
 #include <net/if_dl.h>
@@ -268,17 +299,37 @@
 #include <net/route.h>
 
 #include <netinet/in.h>
+#if !defined(P_QNX)
 #include <netinet/if_ether.h>
+#endif
 
 #define ROUNDUP(a) \
         ((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
 
 #endif
 
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS)
+#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined(P_QNX)
 #include <sys/sysctl.h>
 #endif
 
+int PX_NewHandle(const char *, int);
+
+
+//////////////////////////////////////////////////////////////////////////////
+// P_fd_set
+
+void P_fd_set::Construct()
+{
+  max_fd = PProcess::Current().GetMaxHandles();
+  set = (fd_set *)new char[(max_fd+7)>>3];
+}
+
+
+void P_fd_set::Zero()
+{
+  PAssertNULL(set);
+  memset(set, 0, (max_fd+7)>>3);
+}
 
 
 //////////////////////////////////////////////////////////////////////////////
@@ -313,7 +364,7 @@ int PSocket::os_close()
 
 static int SetNonBlocking(int fd)
 {
-#ifdef __BEOS__
+#if defined __BEOS__ || defined(P_VXWORKS)
   return fd;
 #else
   if (fd < 0)
@@ -326,14 +377,14 @@ static int SetNonBlocking(int fd)
 
   ::close(fd);
   return -1;
-#endif // !__BEOS__
+#endif // !__BEOS__ && !P_VXWORKS
 }
 
 
 int PSocket::os_socket(int af, int type, int protocol)
 {
   // attempt to create a socket
-  return SetNonBlocking(::socket(af, type, protocol));
+  return SetNonBlocking(PX_NewHandle(GetClass(), ::socket(af, type, protocol)));
 }
 
 
@@ -386,9 +437,9 @@ BOOL PSocket::os_accept(PSocket & listener, struct sockaddr * addr, PINDEX * siz
 #if !defined(P_PTHREADS) && !defined(P_MAC_MPTHREADS)
 
 int PSocket::os_select(int maxHandle,
-                   fd_set & readBits,
-                   fd_set & writeBits,
-                   fd_set & exceptionBits,
+                   fd_set * readBits,
+                   fd_set * writeBits,
+                   fd_set * exceptionBits,
           const PIntArray & osHandles,
       const PTimeInterval & timeout)
 {
@@ -401,33 +452,33 @@ int PSocket::os_select(int maxHandle,
   if (stat <= 0)
     return stat;
 
-  struct timeval instant = {0, 0};
-  return ::select(maxHandle, &readBits, &writeBits, &exceptionBits, &instant);
+  P_timeval instant;
+  return ::select(maxHandle, readBits, writeBits, exceptionBits, instant);
 }
                      
 #else
 
 int PSocket::os_select(int width,
-                   fd_set & readBits,
-                   fd_set & writeBits,
-                   fd_set & exceptionBits,
+                   fd_set * readBits,
+                   fd_set * writeBits,
+                   fd_set * exceptionBits,
           const PIntArray & ,
       const PTimeInterval & timeout)
 {
   int unblockPipe = PThread::Current()->unblockPipe[0];
-  FD_SET(unblockPipe, &readBits);
+  FD_SET(unblockPipe, readBits);
   width = PMAX(width, unblockPipe+1);
 
   do {
-    struct timeval tval;
-    int result = ::select(width, &readBits, &writeBits, &exceptionBits, timeout.AsTimeVal(tval));
+    P_timeval tval = timeout;
+    int result = ::select(width, readBits, writeBits, exceptionBits, tval);
     if (result >= 0) {
-      if (FD_ISSET(unblockPipe, &readBits)) {
-        FD_CLR(unblockPipe, &readBits);
+      if (FD_ISSET(unblockPipe, readBits)) {
+        FD_CLR(unblockPipe, readBits);
         if (result == 1) {
           BYTE ch;
           ::read(unblockPipe, &ch, 1);
-          FD_CLR(unblockPipe, &readBits);
+          FD_CLR(unblockPipe, readBits);
           errno = EINTR;
           return -1;
         }
@@ -443,45 +494,54 @@ int PSocket::os_select(int width,
 
 PIPSocket::Address::Address(DWORD dw)
 {
-  s_addr = dw;
+  operator=(dw);
 }
 
 
 PIPSocket::Address & PIPSocket::Address::operator=(DWORD dw)
 {
-  s_addr = dw;
+  if (dw == 0) {
+    version = 0;
+    memset(&v, 0, sizeof(v));
+  }
+  else {
+    version = 4;
+    v.four.s_addr = dw;
+  }
+
   return *this;
 }
 
 
 PIPSocket::Address::operator DWORD() const
 {
-  return (DWORD)s_addr;
+  return version != 4 ? 0 : (DWORD)v.four.s_addr;
 }
 
 BYTE PIPSocket::Address::Byte1() const
 {
-  return *(((BYTE *)&s_addr)+0);
+  return *(((BYTE *)&v.four.s_addr)+0);
 }
 
 BYTE PIPSocket::Address::Byte2() const
 {
-  return *(((BYTE *)&s_addr)+1);
+  return *(((BYTE *)&v.four.s_addr)+1);
 }
 
 BYTE PIPSocket::Address::Byte3() const
 {
-  return *(((BYTE *)&s_addr)+2);
+  return *(((BYTE *)&v.four.s_addr)+2);
 }
 
 BYTE PIPSocket::Address::Byte4() const
 {
-  return *(((BYTE *)&s_addr)+3);
+  return *(((BYTE *)&v.four.s_addr)+3);
 }
 
 PIPSocket::Address::Address(BYTE b1, BYTE b2, BYTE b3, BYTE b4)
 {
-  BYTE * p = (BYTE *)&s_addr;
+  version = 4;
+  BYTE * p = (BYTE *)&v.four.s_addr;
   p[0] = b1;
   p[1] = b2;
   p[2] = b3;
@@ -498,9 +558,9 @@ BOOL PIPSocket::IsLocalHost(const PString & hostname)
 
   // lookup the host address using inet_addr, assuming it is a "." address
   Address addr = hostname;
-  if (addr == 16777343)  // Is 127.0.0.1
+  if (addr.IsLoopback())  // Is 127.0.0.1
     return TRUE;
-  if (addr == (DWORD)-1)
+  if (!addr.IsValid())
     return FALSE;
 
   if (!GetHostAddress(hostname, addr))
@@ -756,7 +816,7 @@ BOOL PEthSocket::EnumInterfaces(PINDEX idx, PString & name)
   PUDPSocket ifsock;
 
   ifreq ifreqs[20]; // Maximum of 20 interfaces
-  ifconf ifc;
+  struct ifconf ifc;
   ifc.ifc_len = sizeof(ifreqs);
   ifc.ifc_buf = (caddr_t)ifreqs;
   if (!ConvertOSError(ioctl(ifsock.GetHandle(), SIOCGIFCONF, &ifc)))
@@ -1015,7 +1075,7 @@ BOOL PIPSocket::GetRouteTable(RouteTable & table)
       return TRUE;
 
     char iface[20];
-    long net_addr, dest_addr, net_mask;
+    unsigned long net_addr, dest_addr, net_mask;
     int flags, refcnt, use, metric;
     procfile >> iface >> ::hex >> net_addr >> dest_addr >> flags 
                       >> ::dec >> refcnt >> use >> metric 
@@ -1032,10 +1092,10 @@ BOOL PIPSocket::GetRouteTable(RouteTable & table)
   }
 }
 
-#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX)
+#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_QNX)
 
-BOOL process_rtentry(struct rt_msghdr *rtm, char *ptr, long *p_net_addr,
-                     long *p_net_mask, long *p_dest_addr, int *p_metric);
+BOOL process_rtentry(struct rt_msghdr *rtm, char *ptr, unsigned long *p_net_addr,
+                     unsigned long *p_net_mask, unsigned long *p_dest_addr, int *p_metric);
 BOOL get_ifname(int index, char *name);
 
 BOOL PIPSocket::GetRouteTable(RouteTable & table)
@@ -1085,7 +1145,7 @@ BOOL PIPSocket::GetRouteTable(RouteTable & table)
   limit = buf + space_needed;
   for (ptr = buf; ptr < limit; ptr += rtm->rtm_msglen) {
 
-    long net_addr, dest_addr, net_mask;
+    unsigned long net_addr, dest_addr, net_mask;
     int metric;
     char name[16];
 
@@ -1109,12 +1169,12 @@ BOOL PIPSocket::GetRouteTable(RouteTable & table)
   return TRUE;
 }
 
-BOOL process_rtentry(struct rt_msghdr *rtm, char *ptr, long *p_net_addr,
-                     long *p_net_mask, long *p_dest_addr, int *p_metric) {
+BOOL process_rtentry(struct rt_msghdr *rtm, char *ptr, unsigned long *p_net_addr,
+                     unsigned long *p_net_mask, unsigned long *p_dest_addr, int *p_metric) {
 
   struct sockaddr_in *sa_in;
 
-  long net_addr, dest_addr, net_mask;
+  unsigned long net_addr, dest_addr, net_mask;
   int metric;
 
   sa_in = (struct sockaddr_in *)(rtm + 1);
@@ -1127,7 +1187,7 @@ BOOL process_rtentry(struct rt_msghdr *rtm, char *ptr, long *p_net_addr,
   }
 
   if ((~rtm->rtm_flags&RTF_LLINFO)
-#if defined(P_NETBSD)
+#if defined(P_NETBSD) || defined(P_QNX)
         && (~rtm->rtm_flags&RTF_CLONED)     // Net BSD has flag one way
 #elif !defined(P_OPENBSD)
         && (~rtm->rtm_flags&RTF_WASCLONED)  // Free BSD/MAC has it another
@@ -1429,7 +1489,26 @@ BOOL PIPSocket::GetRouteTable(RouteTable & table)
 }
 
 
-#else
+#elif defined(P_VXWORKS)
+
+BOOL PIPSocket::GetRouteTable(RouteTable & table)
+{
+  PAssertAlways("PIPSocket::GetRouteTable()");
+  for(;;){
+    char iface[20];
+    unsigned long net_addr, dest_addr, net_mask;
+    int  metric;
+    RouteEntry * entry = new RouteEntry(net_addr);
+    entry->net_mask = net_mask;
+    entry->destination = dest_addr;
+    entry->interfaceName = iface;
+    entry->metric = metric;
+    table.Append(entry);
+    return TRUE;
+  }
+}
+
+#else // unsupported platform
 
 #if 0 
 BOOL PIPSocket::GetRouteTable(RouteTable & table)
@@ -1486,7 +1565,6 @@ BOOL PIPSocket::GetRouteTable(RouteTable & table)
 BOOL PIPSocket::GetInterfaceTable(InterfaceTable & list)
 {
   PUDPSocket sock;
-
 #ifndef __BEOS__
 
   PBYTEArray buffer;
@@ -1530,9 +1608,15 @@ BOOL PIPSocket::GetInterfaceTable(InterfaceTable & list)
               PIPSocket::Address mask = ((sockaddr_in *)&ifReq.ifr_netmask)->sin_addr;
               PINDEX i;
               for (i = 0; i < list.GetSize(); i++) {
+#ifdef P_TORNADO
+                if (list[i].GetName() == name &&
+                    list[i].GetAddress() == addr)
+                    if(list[i].GetNetMask() == mask)
+#else
                 if (list[i].GetName() == name &&
                     list[i].GetAddress() == addr &&
                     list[i].GetNetMask() == mask)
+#endif
                   break;
               }
               if (i >= list.GetSize())
@@ -1542,8 +1626,7 @@ BOOL PIPSocket::GetInterfaceTable(InterfaceTable & list)
         }
       }
 
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX)
-
+#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_VXWORKS) || defined(P_RTEMS) || defined(P_QNX)
 // Define _SIZEOF_IFREQ for platforms (eg OpenBSD) which do not have it.
 #ifndef _SIZEOF_ADDR_IFREQ
 #define _SIZEOF_ADDR_IFREQ(ifr) \
@@ -1557,13 +1640,60 @@ BOOL PIPSocket::GetInterfaceTable(InterfaceTable & list)
 #else
       ifName++;
 #endif
+
     }
   }
 #endif //!__BEOS__
-
   return TRUE;
 }
 
+#ifdef P_VXWORKS
+
+int h_errno;
+
+struct hostent * Vx_gethostbyname(char *name, struct hostent *hp)
+{
+  u_long addr;
+  static char staticgethostname[100];
+
+  hp->h_aliases = NULL;
+  hp->h_addr_list[1] = NULL;
+  if ((int)(addr = inet_addr(name)) != ERROR) {
+    memcpy(staticgethostname, &addr, sizeof(addr));
+    hp->h_addr_list[0] = staticgethostname;
+    h_errno = SUCCESS;
+    return hp;
+  }
+  memcpy(staticgethostname, &addr, sizeof (addr));
+  hp->h_addr_list[0] = staticgethostname;
+  h_errno = SUCCESS;
+  return hp;
+}
+
+struct hostent * Vx_gethostbyaddr(char *name, struct hostent *hp)
+{
+  u_long addr;
+  static char staticgethostaddr[100];
+
+  hp->h_aliases = NULL;
+  hp->h_addr_list = NULL;
+
+  if ((int)(addr = inet_addr(name)) != ERROR) {
+    sprintf(staticgethostaddr,"%s",inet_ntoa(addr));
+    hp->h_name = staticgethostaddr;
+    h_errno = SUCCESS;
+  }
+  else
+  {
+    printf ("_gethostbyaddr: not able to get %s\n",name);
+    h_errno = NOTFOUND;
+  }
+  return hp;
+}
+
+#endif // P_VXWORKS
+
+
 #include "../common/pethsock.cxx"
 
 
