@@ -27,6 +27,35 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: svcproc.cxx,v $
+ * Revision 1.81  2002/12/12 00:54:29  robertj
+ * Fixed issue with setting max file handles causing a log file to be created
+ *   as root (real uid) rather than as the user determined by the -u arg.
+ * Utilised new PProcess functions for setting user and group.
+ *
+ * Revision 1.80  2002/12/02 08:27:43  robertj
+ * Fixed incorrectly set #if statement from RTEMS patch.
+ *
+ * Revision 1.79  2002/12/02 03:57:18  robertj
+ * More RTEMS support patches, thank you Vladimir Nesic.
+ *
+ * Revision 1.78  2002/11/02 00:32:21  robertj
+ * Further fixes to VxWorks (Tornado) port, thanks Andreas Sikkema.
+ *
+ * Revision 1.77  2002/10/30 03:14:15  robertj
+ * Added tab character between time and thread in system log (for db import).
+ *
+ * Revision 1.76  2002/10/23 04:26:27  robertj
+ * Fixed extra debug levels in system log dump.
+ *
+ * Revision 1.75  2002/10/17 13:44:27  robertj
+ * Port to RTEMS, thanks Vladimir Nesic.
+ *
+ * Revision 1.74  2002/10/17 12:57:24  robertj
+ * Added ability to increase maximum file handles on a process.
+ *
+ * Revision 1.73  2002/10/10 04:43:44  robertj
+ * VxWorks port, thanks Martijn Roest
+ *
  * Revision 1.72  2002/10/03 01:27:51  robertj
  * Added fail safe _exit() as it appears raise(SIGQUIT) does not always dump
  *   core and exit the app causing endless SEGV's to appear the log file!
@@ -204,15 +233,28 @@
 #pragma implementation "svcproc.h"
 #include <ptlib/svcproc.h>
 
+#ifdef P_VXWORKS
+#include <logLib.h>
+#define LOG_EMERG			0
+#define LOG_ALERT			1
+#define LOG_CRIT			2
+#define LOG_ERR				3
+#define LOG_WARNING			4
+#define	LOG_NOTICE			5
+#define LOG_INFO			6
+#define LOG_DEBUG			7
+#else
 #include <syslog.h>
+#include <pwd.h>
+#include <grp.h>
+#endif
+
 #include <stdarg.h>
 #if (__GNUC__ >= 3)
 #include <fstream>
 #else
 #include <fstream.h>
 #endif
-#include <pwd.h>
-#include <grp.h>
 #include <signal.h>
 
 #include "uerror.h"
@@ -226,6 +268,7 @@
 
 #define	MAX_LOG_LINE_LEN	1024
 
+#ifndef P_VXWORKS
 static int PwlibLogToUnixLog[PSystemLog::NumLogLevels] = {
   LOG_CRIT,    // LogFatal,   
   LOG_ERR,     // LogError,   
@@ -233,8 +276,12 @@ static int PwlibLogToUnixLog[PSystemLog::NumLogLevels] = {
   LOG_INFO,    // LogInfo,    
   LOG_DEBUG,   // LogDebug
   LOG_DEBUG,
+  LOG_DEBUG,
+  LOG_DEBUG,
+  LOG_DEBUG,
   LOG_DEBUG
 };
+#endif // !P_VXWORKS
 
 static const char * const PLevelName[PSystemLog::NumLogLevels+1] = {
   "Message",
@@ -244,7 +291,10 @@ static const char * const PLevelName[PSystemLog::NumLogLevels+1] = {
   "Info",
   "Debug",
   "Debug2",
-  "Debug3"
+  "Debug3",
+  "Debug4",
+  "Debug5",
+  "Debug6",
 };
 
 #ifdef P_MAC_MPTHREADS
@@ -276,8 +326,14 @@ static pthread_mutex_t logMutex = PTHREAD_MUTEX_INITIALIZER;
 void PSystemLog::Output(Level level, const char * cmsg)
 {
   PString systemLogFileName = PServiceProcess::Current().systemLogFileName;
-  if (systemLogFileName.IsEmpty())
+  if (systemLogFileName.IsEmpty()) {
+#ifdef P_VXWORKS
+    printf("%s\n",cmsg);
+    logMsg((char *)(const char)cmsg,0,0,0,0,0,0);
+#else
     syslog(PwlibLogToUnixLog[level], "%s", cmsg);
+#endif
+  }
   else {
 #ifdef P_PTHREADS
     pthread_mutex_lock(&logMutex);
@@ -294,12 +350,14 @@ void PSystemLog::Output(Level level, const char * cmsg)
       out = new ofstream(systemLogFileName, ios::app);
 
     PTime now;
-    *out << now.AsString("yyyy/MM/dd hh:mm:ss.uuu ");
+    *out << now.AsString("yyyy/MM/dd hh:mm:ss.uuu\t");
 
     PThread * thread = PThread::Current();
     if (thread == NULL) {
 #ifdef P_MAC_MPTHREADS
       unsigned tid = (unsigned)MPCurrentTaskID();
+#elif defined(P_VXWORKS)
+      unsigned tid = ::taskIdSelf();
 #else
       unsigned tid = (unsigned) pthread_self();
 #endif
@@ -384,9 +442,11 @@ PServiceProcess::~PServiceProcess()
   if (!pidFileToRemove)
     PFile::Remove(pidFileToRemove);
 
+#ifndef P_VXWORKS
   // close the system log
   if (systemLogFileName.IsEmpty())
     closelog();
+#endif // !P_VXWORKS
 }
 
 
@@ -398,6 +458,7 @@ PServiceProcess & PServiceProcess::Current()
 }
 
 
+#ifndef P_VXWORKS
 static int KillProcess(int pid, int sig)
 {
   if (kill(pid, sig) != 0)
@@ -422,20 +483,18 @@ static int KillProcess(int pid, int sig)
 
   return 1;
 }
+#endif // !P_VXWORKS
 
 
 void PServiceProcess::_PXShowSystemWarning(PINDEX code, const PString & str)
 {
-  PSYSTEMLOG(Warning, "PWLib/Unix error #"
-                      << code
-                      << "-"
-                      << str
-                      << endl);
+  PSYSTEMLOG(Warning, "PWLib\t" << GetOSClass() << " error #" << code << '-' << str);
 }
 
 
 int PServiceProcess::InitialiseService()
 {
+#ifndef P_VXWORKS
 #if PMEMORY_CHECK
   PMemoryHeap::SetIgnoreAllocations(TRUE);
 #endif
@@ -544,6 +603,24 @@ int PServiceProcess::InitialiseService()
     return 1;
   }
 
+  // Set the gid we are running under
+  if (args.HasOption('g')) {
+    PString gidstr = args.GetOptionString('g');
+    if (!SetGroupName(gidstr)) {
+      cout << "Could not set GID to \"" << gidstr << "\" - " << strerror(errno) << endl;
+      return 1;
+    }
+  }
+
+  // Set the uid we are running under
+  if (args.HasOption('u')) {
+    PString uidstr = args.GetOptionString('u');
+    if (!SetUserName(uidstr)) {
+      cout << "Could not set UID to \"" << uidstr << "\" - " << strerror(errno) << endl;
+      return 1;
+    }
+  }
+
   BOOL helpAndExit = FALSE;
 
   // if displaying help, then do it
@@ -574,7 +651,7 @@ int PServiceProcess::InitialiseService()
     cout << "usage: [-c] -v|-d|-h|-x\n"
             "  -h --help           output this help message and exit\n"
             "  -v --version        display version information and exit\n"
-#ifndef BE_THREADS
+#if !defined(BE_THREADS) && !defined(P_RTEMS)
             "  -d --daemon         run as a daemon\n"
 #endif
             "  -u --uid uid        set user id to run as\n"
@@ -596,77 +673,6 @@ int PServiceProcess::InitialiseService()
     return 0;
   }
 
-  if (args.HasOption('i'))
-    SetConfigurationPath(args.GetOptionString('i'));
-
-  if (args.HasOption('H'))
-    SetMaxFileHandles(args.GetOptionString('H').AsInteger());
-
-  // Set the gid we are running under
-  if (args.HasOption('g')) {
-    PString gidstr = args.GetOptionString('g');
-    if (gidstr.IsEmpty())
-      gidstr = "nobody";
-    int gid;
-    if ((PINDEX)strspn(gidstr, "0123456789") == gidstr.GetLength())
-      gid = gidstr.AsInteger();
-    else {
-      struct group * gr = getgrnam(gidstr);
-      if (gr == NULL) {
-        cout << "Could not find group \"" << gidstr << '"' << endl;
-        return 1;
-      }
-      gid = gr->gr_gid;
-    }
-    if (setgid(gid) != 0) {
-      cout << "Could not set GID to \"" << gidstr << "\" (" << gid << ")"
-              " - " << strerror(errno) << endl;
-      return 1;
-    }
-  }
-
-  // Set the uid we are running under
-  if (args.HasOption('u')) {
-    PString uidstr = args.GetOptionString('u');
-    if (uidstr.IsEmpty())
-      uidstr = "nobody";
-    int uid;
-    if ((PINDEX)strspn(uidstr, "0123456789") == uidstr.GetLength())
-      uid = uidstr.AsInteger();
-    else {
-      struct passwd * pw = getpwnam(uidstr);
-      if (pw == NULL) {
-        cout << "Could not find user \"" << uidstr << '"' << endl;
-        return 1;
-      }
-      uid = pw->pw_uid;
-    }
-    if (setuid(uid) != 0) {
-      cout << "Could not set UID to \"" << uidstr << "\" (" << uid << ")"
-              " - " << strerror(errno) << endl;
-      return 1;
-    }
-  }
-
-  // set the core file size
-  if (args.HasOption('C')) {
-#ifdef P_LINUX
-    struct rlimit rlim;
-    if (getrlimit(RLIMIT_CORE, &rlim) != 0) 
-      cout << "Could not get current core file size : error = " << errno << endl;
-    else {
-      int v = args.GetOptionString('C').AsInteger();
-      rlim.rlim_cur = v;
-      if (setrlimit(RLIMIT_CORE, &rlim) != 0) 
-        cout << "Could not set current core file size to " << v << " : error = " << errno << endl;
-      else {
-        getrlimit(RLIMIT_CORE, &rlim);
-        cout << "Core file size set to " << rlim.rlim_cur << "/" << rlim.rlim_max << endl;
-      }
-    }
-#endif
-  }
-
   // open the system logger for this program
   if (systemLogFileName.IsEmpty())
     openlog((char *)(const char *)GetName(), LOG_PID, LOG_DAEMON);
@@ -680,8 +686,41 @@ int PServiceProcess::InitialiseService()
       return 1;
     }
   }
+  PSYSTEMLOG(StdError, "Starting service process \"" << GetName() << "\" v" << GetVersion(TRUE));
 
-#ifndef BE_THREADS
+  if (args.HasOption('i'))
+    SetConfigurationPath(args.GetOptionString('i'));
+
+  if (args.HasOption('H')) {
+    int uid = geteuid();
+    seteuid(getuid()); // Switch back to starting uid for next call
+    SetMaxHandles(args.GetOptionString('H').AsInteger());
+    seteuid(uid);
+  }
+
+  // set the core file size
+  if (args.HasOption('C')) {
+#ifdef P_LINUX
+    struct rlimit rlim;
+    if (getrlimit(RLIMIT_CORE, &rlim) != 0) 
+      cout << "Could not get current core file size : error = " << errno << endl;
+    else {
+      int uid = geteuid();
+      seteuid(getuid()); // Switch back to starting uid for next call
+      int v = args.GetOptionString('C').AsInteger();
+      rlim.rlim_cur = v;
+      if (setrlimit(RLIMIT_CORE, &rlim) != 0) 
+        cout << "Could not set current core file size to " << v << " : error = " << errno << endl;
+      else {
+        getrlimit(RLIMIT_CORE, &rlim);
+        cout << "Core file size set to " << rlim.rlim_cur << "/" << rlim.rlim_max << endl;
+      }
+      seteuid(uid);
+    }
+#endif
+  }
+
+#if !defined(BE_THREADS) && !defined(P_RTEMS)
   if (!args.HasOption('d'))
     return -1;
 
@@ -743,16 +782,14 @@ int PServiceProcess::InitialiseService()
   // Also if in background, don't want to get blocked on input from stdin
   ::close(STDIN_FILENO);
 
-#endif // BE_THREADS
-
+#endif // !BE_THREADS && !P_RTEMS
+#endif // !P_VXWORKS
   return -1;
 }
 
 int PServiceProcess::_main(void *)
 {
   if ((terminationValue = InitialiseService()) < 0) {
-    PSYSTEMLOG(Warning, "Starting service process \"" << GetName() << "\" v" << GetVersion(TRUE));
-
     // Make sure housekeeping thread is going so signals are handled.
     SignalTimerChange();
 
@@ -804,9 +841,11 @@ void PServiceProcess::Terminate()
   // Do the services stop code
   OnStop();
 
+#ifndef P_VXWORKS
   // close the system log
   if (systemLogFileName.IsEmpty())
     closelog();
+#endif // !P_VXWORKS
 
   // Now end the program
   exit(terminationValue);
@@ -857,6 +896,8 @@ void PServiceProcess::PXOnAsyncSignal(int sig)
 
 #ifdef P_MAC_MPTHREADS
   unsigned tid = (unsigned)MPCurrentTaskID();
+#elif defined(P_VXWORKS)
+  unsigned tid = ::taskIdSelf();
 #else
   unsigned tid = (unsigned) pthread_self();
 #endif
@@ -878,15 +919,19 @@ void PServiceProcess::PXOnAsyncSignal(int sig)
   strcat(msg, ", aborting.\n");
 
   if (systemLogFileName.IsEmpty()) {
-    syslog(LOG_CRIT, 
-#ifdef __BEOS__ // (Some?) BeOS versions of syslog.h have syslog() (wrongly) declared without const 
-    (char *) 
-#endif 
-    msg); 
+#ifdef P_VXWORKS
+  logMsg((char *)msg,0,0,0,0,0,0);
+#else
+    syslog(LOG_CRIT, msg); 
     closelog();
+#endif // !P_VXWORKS
   }
   else {
+#ifdef P_VXWORKS
+    int fd = open(systemLogFileName, O_WRONLY|O_APPEND, FWRITE|FAPPEND);
+#else
     int fd = open(systemLogFileName, O_WRONLY|O_APPEND);
+#endif // !P_VXWORKS
     if (fd >= 0) {
       write(fd, msg, strlen(msg));
       close(fd);
@@ -922,6 +967,7 @@ void PServiceProcess::PXOnSignal(int sig)
       }
       break;
 
+#ifdef SIGWINCH
     case SIGWINCH :
       if (currentLogLevel > PSystemLog::Fatal) {
         currentLogLevel = (PSystemLog::Level)(currentLogLevel-1);
@@ -929,6 +975,7 @@ void PServiceProcess::PXOnSignal(int sig)
         s << "Log level decreased to " << PLevelName[currentLogLevel+1];
       }
       break;
+#endif
   }
 }
 
