@@ -27,6 +27,57 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: tlibthrd.cxx,v $
+ * Revision 1.107  2002/12/11 05:39:26  robertj
+ * Added logging for file handle changes.
+ * Fixd bug where internal maxHandles not set when increased.
+ *
+ * Revision 1.106  2002/12/02 03:57:18  robertj
+ * More RTEMS support patches, thank you Vladimir Nesic.
+ *
+ * Revision 1.105  2002/11/22 10:14:07  robertj
+ * QNX port, thanks Xiaodan Tang
+ *
+ * Revision 1.104  2002/11/04 16:01:27  rogerh
+ * Using pthread_cancel and not pthread_kill with SIGKILL to terminate a thread
+ * On FreeBSD the thread does not have a handler for SIGKILL, it passes it up
+ * to the main process which gets killed! Assume the other BSDs are the same.
+ *
+ * Revision 1.103  2002/10/24 00:40:56  robertj
+ * Put back ability to terminate a thread from that threads context (removed
+ *   in revision 1.101) but requires that destructor not do so.
+ * Changed pipe close to allow for possible EINTR, and retry close.
+ *
+ * Revision 1.102  2002/10/24 00:25:13  robertj
+ * Changed high load thread problem fix from the termination function to start
+ *   function to finally, once and for all (I hope!) fix the race condition.
+ *
+ * Revision 1.101  2002/10/23 14:56:22  craigs
+ * Fixed problem with pipe leak under some circumstances
+ *
+ * Revision 1.100  2002/10/23 04:29:32  robertj
+ * Improved debugging for thread create/start/stop/destroy.
+ * Fixed race condition bug if auto-delete thread starts and completes before
+ *   pthread_create returns, PX_threadId is not set yet!
+ *
+ * Revision 1.99  2002/10/22 07:42:52  robertj
+ * Added extra debugging for file handle and thread leak detection.
+ *
+ * Revision 1.98  2002/10/18 03:05:39  robertj
+ * Fixed thread leak caused by fixing the thread crash a few revisions back,
+ *   caused by strange pthreads behaviour, at least under Linux.
+ *
+ * Revision 1.97  2002/10/17 13:44:27  robertj
+ * Port to RTEMS, thanks Vladimir Nesic.
+ *
+ * Revision 1.96  2002/10/17 12:57:24  robertj
+ * Added ability to increase maximum file handles on a process.
+ *
+ * Revision 1.95  2002/10/16 11:26:29  rogerh
+ * Add missing include. Noticed by Matthias on the GnomeMeeting IRC
+ *
+ * Revision 1.94  2002/10/10 03:09:48  robertj
+ * Fixed high load race condition when starting threads.
+ *
  * Revision 1.93  2002/10/05 05:22:43  robertj
  * Fixed adding GetThreadId() function.
  *
@@ -333,9 +384,21 @@
  *
  */
 
+#include <ptlib/socket.h>
+#include <sched.h>	// for sched_yield
 #include <pthread.h>
 #include <sys/resource.h>
 
+#ifdef P_RTEMS
+#define	SUSPEND_SIG SIGALRM
+#include <sched.h>
+#else
+#define	SUSPEND_SIG SIGVTALRM
+#endif
+
+
+int PX_NewHandle(const char *, int);
+
 
 #define PAssertPTHREAD(func, args) \
   { \
@@ -355,8 +418,12 @@ static BOOL PAssertThreadOp(int retval,
   }
 
   if (errno == EINTR || errno == EAGAIN) {
-    if (++retry < 1000) { \
+    if (++retry < 1000) {
+#if defined(P_RTEMS)
+      rtems_task_wake_after(10);
+#else
       usleep(10000); // Basically just swap out thread to try and clear blockage
+#endif
       return TRUE;   // Return value to try again
     }
     // Give up and assert
@@ -367,9 +434,6 @@ static BOOL PAssertThreadOp(int retval,
 }
 
 
-#define	SUSPEND_SIG SIGVTALRM
-
-
 PDECLARE_CLASS(PHouseKeepingThread, PThread)
   public:
     PHouseKeepingThread()
@@ -400,12 +464,9 @@ void PHouseKeepingThread::Main()
 
     int fd = process.timerChangePipe[0];
 
-    fd_set read_fds;
-    FD_ZERO(&read_fds);
-    FD_SET(fd, &read_fds);
-
-    struct timeval tval;
-    if (::select(fd+1, &read_fds, NULL, NULL, delay.AsTimeVal(tval)) == 1) {
+    P_fd_set read_fds = fd;
+    P_timeval tval = delay;
+    if (::select(fd+1, read_fds, NULL, NULL, tval) == 1) {
       BYTE ch;
       ::read(fd, &ch, 1);
     }
@@ -434,13 +495,17 @@ void PProcess::SignalTimerChange()
 
 void PProcess::Construct()
 {
-  // set the file descriptor limit to something sensible
+#ifndef P_RTEMS
+  // get the file descriptor limit
   struct rlimit rl;
   PAssertOS(getrlimit(RLIMIT_NOFILE, &rl) == 0);
-  rl.rlim_cur = rl.rlim_max;
-  PAssertOS(setrlimit(RLIMIT_NOFILE, &rl) == 0);
+  maxHandles = rl.rlim_cur;
+  PTRACE(4, "PWLib\tMaximum per-process file handles is " << maxHandles);
 
   ::pipe(timerChangePipe);
+#else
+  socketpair(AF_INET,SOCK_STREAM,0,timerChangePipe);
+#endif
 
   // initialise the housekeeping thread
   housekeepingThread = NULL;
@@ -448,21 +513,29 @@ void PProcess::Construct()
   CommonConstruct();
 }
 
-BOOL PProcess::SetMaxFileHandles(int maxFileHandles)
+
+BOOL PProcess::SetMaxHandles(int newMax)
 {
+#ifndef P_RTEMS
   // get the current process limit
   struct rlimit rl;
   PAssertOS(getrlimit(RLIMIT_NOFILE, &rl) == 0);
 
   // set the new current limit
-  rl.rlim_cur = maxFileHandles;
-  if (setrlimit(RLIMIT_NOFILE, &rl) != 0) {
-    PTRACE(1, "PWLib\tCannot set per-process file handle limit to "
-           << maxFileHandles << " - check permissions");
-    return FALSE;
+  rl.rlim_cur = newMax;
+  if (setrlimit(RLIMIT_NOFILE, &rl) == 0) {
+    PAssertOS(getrlimit(RLIMIT_NOFILE, &rl) == 0);
+    maxHandles = rl.rlim_cur;
+    if (maxHandles == newMax) {
+      PTRACE(2, "PWLib\tNew maximum per-process file handles set to " << maxHandles);
+      return TRUE;
+    }
   }
+#endif // !P_RTEMS
 
-  return TRUE;
+  PTRACE(1, "PWLib\tCannot set per-process file handle limit to "
+         << newMax << " (is " << maxHandles << ") - check permissions");
+  return FALSE;
 }
 
 
@@ -503,7 +576,11 @@ void PThread::InitialiseProcessThread()
 
   PX_suspendMutex = MutexInitialiser;
 
+#ifdef P_RTEMS
+  PAssertOS(socketpair(AF_INET,SOCK_STREAM,0,unblockPipe) == 0);
+#else
   PAssertOS(::pipe(unblockPipe) == 0);
+#endif
 
   ((PProcess *)this)->activeThreads.DisallowDeleteObjects();
   ((PProcess *)this)->activeThreads.SetAt((unsigned)PX_threadId, this);
@@ -533,28 +610,38 @@ PThread::PThread(PINDEX stackSize,
 
   PX_suspendMutex = MutexInitialiser;
 
+#ifdef P_RTEMS
+  PAssertOS(socketpair(AF_INET,SOCK_STREAM,0,unblockPipe) == 0);
+#else
   PAssertOS(::pipe(unblockPipe) == 0);
+#endif
+  PX_NewHandle("Thread unblock pipe", PMAX(unblockPipe[0], unblockPipe[1]));
 
   // new thread is actually started the first time Resume() is called.
   PX_firstTimeStart = TRUE;
 
   traceBlockIndentLevel = 0;
+
+  PTRACE(5, "PWLib\tCreated thread " << this << ' ' << threadName);
 }
 
 
 PThread::~PThread()
 {
-  if (!IsTerminated()) 
+  if (PX_threadId != 0 && PX_threadId != pthread_self())
     Terminate();
 
-  ::close(unblockPipe[0]);
-  ::close(unblockPipe[1]);
+  PAssertPTHREAD(::close, (unblockPipe[0]));
+  PAssertPTHREAD(::close, (unblockPipe[1]));
 
 #ifndef P_HAS_SEMAPHORES
   pthread_mutex_destroy(&PX_WaitSemMutex);
 #endif
 
+  pthread_mutex_unlock(&PX_suspendMutex);
   pthread_mutex_destroy(&PX_suspendMutex);
+
+  PTRACE(5, "PWLib\tDestroyed thread " << this << ' ' << threadName);
 }
 
 
@@ -567,7 +654,7 @@ void PThread::Restart()
   pthread_attr_init(&threadAttr);
   pthread_attr_setdetachstate(&threadAttr, PTHREAD_CREATE_DETACHED);
 
-#ifdef P_LINUX
+#if defined(P_LINUX) || defined(P_RTEMS)
   /*
     Set realtime scheduling if our effective user id is root (only then is this
     allowed) AND our priority is Highest.
@@ -689,7 +776,7 @@ void PThread::SetPriority(Priority priorityLevel)
 {
   PX_priority = priorityLevel;
 
-#ifdef P_LINUX
+#if defined(P_LINUX) || defined(P_RTEMS)
   if (IsTerminated())
     return;
 
@@ -712,7 +799,7 @@ void PThread::SetPriority(Priority priorityLevel)
 
 PThread::Priority PThread::GetPriority() const
 {
-#ifdef LINUX
+#if defined(LINUX) || defined(P_RTEMS)
   int schedulingPolicy;
   struct sched_param schedParams;
   
@@ -752,9 +839,8 @@ void PThread::Sleep(const PTimeInterval & timeout)
   PTime lastTime;
   PTime targetTime = lastTime + timeout;
   do {
-    PTimeInterval delay = targetTime - lastTime;
-    struct timeval tval;
-    if (select(0, NULL, NULL, NULL, delay.AsTimeVal(tval)) < 0 && errno != EINTR)
+    P_timeval tval = targetTime - lastTime;
+    if (select(0, NULL, NULL, NULL, tval) < 0 && errno != EINTR)
       break;
 
 #ifndef P_NETBSD
@@ -787,34 +873,33 @@ void PThread::Terminate()
   if (PX_origStackSize <= 0)
     return;
 
+  // don't use PThread::Current, as the thread may already not be in the
+  // active threads list
+  if (PX_threadId == pthread_self()) {
+    pthread_exit(0);
+    return;
+  }
+
   if (IsTerminated())
     return;
 
   PTRACE(2, "PWLib\tForcing termination of thread " << (void *)this);
 
-  if (Current() == this)
-    pthread_exit(NULL);
-  else {
-    PXAbortBlock();
-    WaitForTermination(20);
+  PXAbortBlock();
+  WaitForTermination(20);
 
 #ifndef P_HAS_SEMAPHORES
-    PAssertPTHREAD(pthread_mutex_lock, (&PX_WaitSemMutex));
-    if (PX_waitingSemaphore != NULL) {
-      PAssertPTHREAD(pthread_mutex_lock, (&PX_waitingSemaphore->mutex));
-      PX_waitingSemaphore->queuedLocks--;
-      PAssertPTHREAD(pthread_mutex_unlock, (&PX_waitingSemaphore->mutex));
-      PX_waitingSemaphore = NULL;
-    }
-    PAssertPTHREAD(pthread_mutex_unlock, (&PX_WaitSemMutex));
+  PAssertPTHREAD(pthread_mutex_lock, (&PX_WaitSemMutex));
+  if (PX_waitingSemaphore != NULL) {
+    PAssertPTHREAD(pthread_mutex_lock, (&PX_waitingSemaphore->mutex));
+    PX_waitingSemaphore->queuedLocks--;
+    PAssertPTHREAD(pthread_mutex_unlock, (&PX_waitingSemaphore->mutex));
+    PX_waitingSemaphore = NULL;
+  }
+  PAssertPTHREAD(pthread_mutex_unlock, (&PX_WaitSemMutex));
 #endif
 
-#if defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined (P_AIX)
-    pthread_kill(PX_threadId, SIGKILL);
-#else
-    pthread_cancel(PX_threadId);
-#endif
-  }
+  pthread_cancel(PX_threadId);
 }
 
 
@@ -871,18 +956,35 @@ void * PThread::PX_ThreadStart(void * arg)
   pthread_detach(threadId);
 
   PThread * thread = (PThread *)arg;
+
+  // Added this to guarantee that the thread creation (PThread::Restart)
+  // has completed before we start the thread. Then the PX_threadId has
+  // been set.
+  pthread_mutex_lock(&thread->PX_suspendMutex);
+
   thread->SetThreadName(thread->GetThreadName());
 
+  pthread_mutex_unlock(&thread->PX_suspendMutex);
+
   PProcess & process = PProcess::Current();
 
+  PINDEX newHighWaterMark = 0;
+  static PINDEX highWaterMark = 0;
+
   // add thread to thread list
   process.threadMutex.Wait();
   process.activeThreads.SetAt((unsigned)threadId, thread);
+  if (process.activeThreads.GetSize() > highWaterMark)
+    newHighWaterMark = highWaterMark = process.activeThreads.GetSize();
   process.threadMutex.Signal();
 
+  PTRACE_IF(4, newHighWaterMark > 0, "PWLib\tThread high water mark set: " << newHighWaterMark);
+
   // make sure the cleanup routine is called when the thread exits
   pthread_cleanup_push(PThread::PX_ThreadEnd, arg);
 
+  PTRACE(5, "PWLib\tStarted thread " << thread << ' ' << thread->threadName);
+
   // now call the the thread main routine
   thread->Main();
 
@@ -896,21 +998,28 @@ void * PThread::PX_ThreadStart(void * arg)
 void PThread::PX_ThreadEnd(void * arg)
 {
   PThread * thread = (PThread *)arg;
-  PProcess & process = PProcess::Current();
-  
   pthread_t id = thread->GetThreadId();
-  if (id != 0) {
-
-    // remove this thread from the active thread list
-    process.threadMutex.Wait();
-    process.activeThreads.SetAt((unsigned)id, NULL);
-    process.threadMutex.Signal();
+  if (id == 0) {
+    // Don't know why, but pthreads under Linux at least can call this function
+    // multiple times! Probably a bug, but we have to allow for it.
+    PTRACE(2, "PWLib\tAttempted to multiply end thread " << thread << " ThreadID=" << (void *)id);
+    return;
   }
 
+  PTRACE(5, "PWLib\tEnded thread " << thread << ' ' << thread->threadName);
+
+  // remove this thread from the active thread list
+  PProcess & process = PProcess::Current();
+  process.threadMutex.Wait();
+  process.activeThreads.SetAt((unsigned)id, NULL);
+  process.threadMutex.Signal();
+
   // delete the thread if required, note this is done this way to avoid
   // a race condition, the thread ID cannot be zeroed before the if!
   if (thread->autoDelete) {
     thread->PX_threadId = 0;  // Prevent terminating terminated thread
+
+    // Now should be safe to delete the thread!
     delete thread;
   }
   else
@@ -922,35 +1031,35 @@ int PThread::PXBlockOnIO(int handle, int type, const PTimeInterval & timeout)
 {
   PTRACE(7, "PWLib\tPThread::PXBlockOnIO(" << handle << ',' << type << ')');
 
-  if ((handle < 0) || (handle >= FD_SETSIZE)) {
+  if ((handle < 0) || (handle >= PProcess::Current().GetMaxHandles())) {
+    PTRACE(2, "PWLib\tAttempt to use illegal handle in PThread::PXBlockOnIO, handle=" << handle);
     errno = EBADF;
     return -1;
   }
 
   // make sure we flush the buffer before doing a write
-  fd_set tmp_rfd, tmp_wfd, tmp_efd;
-  fd_set * read_fds      = &tmp_rfd;
-  fd_set * write_fds     = &tmp_wfd;
-  fd_set * exception_fds = &tmp_efd;
+  P_fd_set read_fds;
+  P_fd_set write_fds;
+  P_fd_set exception_fds;
 
   int retval;
   do {
-
-    FD_ZERO(read_fds);
-    FD_ZERO(write_fds);
-    FD_ZERO(exception_fds);
-
     switch (type) {
       case PChannel::PXReadBlock:
       case PChannel::PXAcceptBlock:
-        FD_SET(handle, read_fds);
+        read_fds = handle;
+        write_fds.Zero();
+        exception_fds.Zero();
         break;
       case PChannel::PXWriteBlock:
-        FD_SET(handle, write_fds);
+        read_fds.Zero();
+        write_fds = handle;
+        exception_fds.Zero();
         break;
       case PChannel::PXConnectBlock:
-        FD_SET(handle, write_fds);
-        FD_SET(handle, exception_fds);
+        read_fds.Zero();
+        write_fds = handle;
+        exception_fds = handle;
         break;
       default:
         PAssertAlways(PLogicError);
@@ -958,15 +1067,14 @@ int PThread::PXBlockOnIO(int handle, int type, const PTimeInterval & timeout)
     }
 
     // include the termination pipe into all blocking I/O functions
-    FD_SET(unblockPipe[0], read_fds);
+    read_fds += unblockPipe[0];
 
-    struct timeval tval;
+    P_timeval tval = timeout;
     retval = ::select(PMAX(handle, unblockPipe[0])+1,
-                      read_fds, write_fds, exception_fds,
-                      timeout.AsTimeVal(tval));
+                      read_fds, write_fds, exception_fds, tval);
   } while (retval < 0 && errno == EINTR);
 
-  if ((retval == 1) && FD_ISSET(unblockPipe[0], read_fds)) {
+  if ((retval == 1) && read_fds.IsPresent(unblockPipe[0])) {
     BYTE ch;
     ::read(unblockPipe[0], &ch, 1);
     errno = EINTR;
@@ -1177,6 +1285,9 @@ BOOL PSemaphore::WillBlock() const
 #endif
 }
 
+#if defined(P_QNX) && defined(P_HAS_RECURSIVE_MUTEX)
+#define PTHREAD_MUTEX_RECURSIVE_NP PTHREAD_MUTEX_RECURSIVE
+#endif
 
 PMutex::PMutex()
   : PSemaphore(PXMutex)
