@@ -30,6 +30,15 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: main.cxx,v $
+ * Revision 1.40  2002/11/27 11:42:52  robertj
+ * Rearranged code to avoid GNU compiler problem.
+ * Changed new classheader parameters to be full C literal like string for
+ *   improved backslash conversion.
+ * Incremented version number.
+ *
+ * Revision 1.39  2002/11/26 11:39:10  craigs
+ * Added option to allow adding functions to generated header files
+ *
  * Revision 1.38  2002/09/16 01:08:59  robertj
  * Added #define so can select if #pragma interface/implementation is used on
  *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
@@ -128,6 +137,13 @@
 #include "main.h"
 #include "asn_grammar.h"
 
+
+#define MAJOR_VERSION 1
+#define MINOR_VERSION 6
+#define BUILD_TYPE    ReleaseCode
+#define BUILD_NUMBER 0
+
+
 unsigned lineNumber;
 PString  fileName;
 
@@ -295,12 +311,17 @@ class App : public PProcess
   public:
     App();
     void Main();
+    BOOL SetClassHeaderFile(PArgList & args);
+    BOOL SetClassHeader(PArgList & args);
+    void OutputAdditionalHeaders(ostream & hdr, const PString & className);
+  protected:
+    PStringToString classToHeader;
 };
 
 PCREATE_PROCESS(App);
 
 App::App()
-  : PProcess("Equivalence", "ASNParse", 1, 5, ReleaseCode, 4)
+  : PProcess("Equivalence", "ASNParse", MAJOR_VERSION, MINOR_VERSION, BUILD_TYPE, BUILD_NUMBER)
 {
 }
 
@@ -321,7 +342,9 @@ void App::Main()
              "s-split;"
              "o-output:"
              "m-module:"
-             "r-rename:");
+             "r-rename:"
+             "-classheader:"
+             "-classheaderfile:");
 
   if (args.HasOption('V'))
     return;
@@ -369,6 +392,16 @@ void App::Main()
   fatals     = 0;
   warnings   = 0;
 
+  if (args.HasOption("classheaderfile")) {
+    if (!SetClassHeaderFile(args))
+      return;
+  }
+
+  if (args.HasOption("classheader")) {
+    if (!SetClassHeader(args))
+      return;
+  }
+
   if (args.HasOption('v'))
     cout << "Parsing..." << endl;
 
@@ -391,6 +424,77 @@ void App::Main()
 }
 
 
+BOOL App::SetClassHeaderFile(PArgList & args)
+{
+  PStringArray lines = args.GetOptionString("classheaderfile").Lines();
+  if (lines.IsEmpty()) {
+    PError << GetName() << ": malformed --classheaderfile option\n";
+    return FALSE;
+  }
+
+  for (PINDEX i = 0; i < lines.GetSize(); i++) {
+    PString str = lines[i];
+    PINDEX pos = str.Find("=");
+    if (pos == P_MAX_INDEX) {
+      PError << GetName() << ": malformed --classheaderfile option\n";
+      return FALSE;
+    }
+
+    PFilePath fn = str.Right(pos+1);
+    PTextFile file(fn, PFile::ReadOnly);
+    if (!file.IsOpen()) {
+      PError << GetName() << ": cannot open file required for --classheaderfile option \"" << fn
+                          << "\" :" << file.GetErrorText() << '\n';
+      return FALSE;
+    }
+
+    PString text;
+    PString line;
+    while (file.ReadLine(line))
+      text += PString(PString::Literal, (const char *)line) + '\n';
+    classToHeader.SetAt(str.Left(pos), text);
+  }
+
+  return TRUE;
+}
+
+
+BOOL App::SetClassHeader(PArgList & args)
+{
+  PStringArray lines = args.GetOptionString("classheader").Lines();
+  if (lines.IsEmpty()) {
+    PError << GetName() << ": malformed --classheader option\n";
+    return FALSE;
+  }
+
+  for (PINDEX i = 0; i < lines.GetSize(); i++) {
+    PString str = lines[i];
+    PINDEX pos = str.Find("=");
+    if (pos == P_MAX_INDEX) {
+      PError << GetName() << ": malformed --classheader option\n";
+      return FALSE;
+    }
+
+    PString text(PString::Literal, (const char *)str.Mid(pos+1));
+    classToHeader.SetAt(str.Left(pos), text);
+  }
+
+  return TRUE;
+}
+
+
+void App::OutputAdditionalHeaders(ostream & hdr, const PString & className)
+{
+  if (classToHeader.Contains(className)) {
+    hdr << "// following code added by command line option\n"
+           "\n"
+        << classToHeader[className] << "\n"
+           "\n"
+           "// end of added code\n"
+           "\n";
+  }
+}
+
 /////////////////////////////////////////
 //
 //  miscellaneous
@@ -1142,6 +1246,9 @@ void TypeBase::BeginGenerateCplusplus(ostream & hdr, ostream & cxx)
     hdr << tag.number;
   hdr << ", TagClass tagClass = " << UniversalTagClassNames[tag.type] << ");\n\n";
 
+  App & app = (App &)PProcess::Current();
+  app.OutputAdditionalHeaders(hdr, GetIdentifier());
+
   // Output cxx file implementation of class
   cxx << "//\n"
          "// " << GetName() << "\n"
